@@ -25,6 +25,7 @@
 """Main Clustering set of routines."""
 
 import copy
+import os
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import sklearn.cluster as skc
@@ -32,40 +33,15 @@ from sklearn.metrics import calinski_harabasz_score
 import sklearn.preprocessing as skp
 
 from pygmi.raster.datatypes import Data
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
+from pygmi.misc import BasicModule
 
 
-class Cluster(QtWidgets.QDialog):
-    """
-    Cluster Class.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class Cluster(BasicModule):
+    """Cluster Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-
-        if parent is not None:
-            self.piter = parent.pbar.iter
-        else:
-            self.piter = iter
-
         self.combobox_alg = QtWidgets.QComboBox()
         self.spinbox_branchfac = QtWidgets.QSpinBox()
         self.spinbox_minsamples = QtWidgets.QSpinBox()
@@ -101,7 +77,8 @@ class Cluster(QtWidgets.QDialog):
 
         self.setupui()
 
-        self.combobox_alg.addItems(['k-means', 'DBSCAN', 'Birch'])
+        self.combobox_alg.addItems(['Mini Batch K-Means (fast)', 'K-Means',
+                                    'DBSCAN', 'Birch'])
         self.combobox_alg.currentIndexChanged.connect(self.combo)
         self.combo()
 
@@ -205,7 +182,7 @@ class Cluster(QtWidgets.QDialog):
             self.label_minsamples.show()
             self.spinbox_minsamples.show()
             self.doublespinbox_eps.show()
-        elif i == 'k-means':
+        elif i in ['K-Means', 'Mini Batch K-Means (fast)']:
             self.label_minclusters.show()
             self.spinbox_minclusters.show()
             self.label_maxclusters.show()
@@ -228,12 +205,21 @@ class Cluster(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         tst = np.unique([i.data.shape for i in self.indata['Raster']])
 
         if tst.size > 2:
@@ -248,12 +234,12 @@ class Cluster(QtWidgets.QDialog):
             temp = self.exec_()
             if temp == 0:
                 return False
-
-            self.parent.process_is_active()
+            if self.parent is not None:
+                self.parent.process_is_active()
 
         flag = self.run()
 
-        if not nodialog:
+        if not nodialog and self.parent is not None:
             self.parent.process_is_active(False)
             self.parent.pbar.to_max()
         return flag
@@ -338,25 +324,32 @@ class Cluster(QtWidgets.QDialog):
         None.
 
         """
-        data = copy.copy(self.indata['Raster'])
+        data = self.indata['Raster']
         self.update_vars()
 
         no_clust = range(self.min_cluster, self.max_cluster+1)
 
         self.showprocesslog('Cluster analysis started')
 
-# Section to deal with different bands having different null values.
-        masktmp = data[0].data.mask
+        # Section to deal with different bands having different null values.
+        masktmp = ~data[0].data.mask
         for i in data:
-            masktmp += i.data.mask
-        for i, _ in enumerate(data):
-            data[i].data.mask = masktmp
-        X = np.array([i.data.compressed() for i in data]).T
-        Xorig = X.copy()
+            masktmp += ~i.data.mask
+        masktmp = ~masktmp
+
+        X = []
+        for band in data:
+            tmp = copy.deepcopy(band)
+            tmp.data.mask = masktmp
+            X.append(tmp.data.compressed())
+            del tmp
+        X = np.transpose(X)
 
         if self.radiobutton_sscale.isChecked():
+            self.showprocesslog('Applying standard scaling')
             X = skp.StandardScaler().fit_transform(X)
         elif self.radiobutton_rscale.isChecked():
+            self.showprocesslog('Applying robust scaling')
             X = skp.RobustScaler().fit_transform(X)
 
         dat_out = []
@@ -366,16 +359,22 @@ class Cluster(QtWidgets.QDialog):
             elif i > no_clust[0]:
                 continue
 
-            if self.cltype == 'k-means':
-                cfit = skc.KMeans(n_clusters=i, tol=self.tol,
+            if self.cltype == 'Mini Batch K-Means (fast)':
+                bsize = max(os.cpu_count()*256, 1024)
+                cfit = skc.MiniBatchKMeans(n_clusters=i, tol=self.tol,
+                                           max_iter=self.max_iter,
+                                           # n_init='auto',
+                                           batch_size=bsize).fit(X)
+            elif self.cltype == 'K-Means':
+                cfit = skc.KMeans(n_clusters=i, tol=self.tol, # n_init='auto',
                                   max_iter=self.max_iter).fit(X)
-                # cfit = skc.MiniBatchKMeans(n_clusters=i, tol=self.tol,
-                #                            max_iter=self.max_iter).fit(X)
+
             elif self.cltype == 'DBSCAN':
                 cfit = skc.DBSCAN(eps=self.eps,
                                   min_samples=self.min_samples).fit(X)
 
             elif self.cltype == 'Birch':
+                X = np.ascontiguousarray(X)  # Birch gave an error without this
                 cfit = skc.Birch(n_clusters=i, threshold=self.bthres,
                                  branching_factor=self.branchfac).fit(X)
 
@@ -384,7 +383,7 @@ class Cluster(QtWidgets.QDialog):
                                     'Please change settings.')
 
                 return False
-            elif cfit.labels_.max() < 0 and self.cltype == 'DBSCAN':
+            if cfit.labels_.max() < 0 and self.cltype == 'DBSCAN':
                 self.showprocesslog('Could not find any clusters. '
                                     'Please change settings.')
 
@@ -397,22 +396,29 @@ class Cluster(QtWidgets.QDialog):
                 dat_out[-1].metadata['Cluster']['input_type'].append(k.dataid)
 
             zonal = np.ma.masked_all(data[0].data.shape)
-            alpha = (data[0].data.mask == 0)
-            zonal[alpha == 1] = cfit.labels_
+            zonal[~masktmp] = cfit.labels_
 
             dat_out[-1].data = zonal
-            dat_out[-1].nullvalue = zonal.fill_value
+            dat_out[-1].nodata = zonal.fill_value
             dat_out[-1].metadata['Cluster']['no_clusters'] = i
             dat_out[-1].metadata['Cluster']['center'] = np.zeros([i, len(data)])
             dat_out[-1].metadata['Cluster']['center_std'] = np.zeros([i, len(data)])
             if cfit.labels_.max() > 0:
                 dat_out[-1].metadata['Cluster']['vrc'] = calinski_harabasz_score(X, cfit.labels_)
 
+            # Reloading this hear to save memory. Need unscaled values.
+            X = []
+            for band in data:
+                tmp = copy.deepcopy(band)
+                tmp.data.mask = masktmp
+                X.append(tmp.data.compressed())
+            X = np.transpose(X)
+
             m = []
             s = []
             for i2 in range(cfit.labels_.max()+1):
-                m.append(Xorig[cfit.labels_ == i2].mean(0))
-                s.append(Xorig[cfit.labels_ == i2].std(0))
+                m.append(X[cfit.labels_ == i2].mean(0))
+                s.append(X[cfit.labels_ == i2].std(0))
 
             dat_out[-1].metadata['Cluster']['center'] = np.array(m)
             dat_out[-1].metadata['Cluster']['center_std'] = np.array(s)
@@ -420,22 +426,176 @@ class Cluster(QtWidgets.QDialog):
             self.log = ('Cluster complete' + ' (' + self.cltype+')')
 
         for i in dat_out:
-            i.xdim = data[0].xdim
-            i.ydim = data[0].ydim
             i.dataid = 'Clusters: '+str(i.metadata['Cluster']['no_clusters'])
             if self.cltype == 'DBSCAN':
                 i.dataid = 'Clusters: '+str(int(i.data.max()+1))
-            i.nullvalue = data[0].nullvalue
-            i.extent = data[0].extent
+            i.nodata = data[0].nodata
+            i.set_transform(transform=data[0].transform)
+            i.crs = data[0].crs
 
         self.showprocesslog('Cluster complete' + ' ('+self.cltype + ' ' + ')')
 
         for i in dat_out:
             i.data += 1
             i.data = i.data.astype(np.uint8)
-            i.nullvalue = 0
+            i.nodata = 0
 
         self.outdata['Cluster'] = dat_out
         self.outdata['Raster'] = self.indata['Raster']
 
         return True
+
+
+def _testfn():
+    import sys
+    import matplotlib.pyplot as plt
+    from pygmi.raster.iodefs import get_raster
+
+    ifile = r"D:\Workdata\PyGMI Test Data\Classification\Cut_K_Th_U.ers"
+
+    dat = get_raster(ifile)
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    DM = Cluster()
+    DM.indata['Raster'] = dat
+    DM.settings()
+
+    dat2 = DM.outdata['Raster']
+
+    plt.figure(dpi=150)
+    plt.imshow(dat2[0].data)
+    plt.show()
+
+
+def _test_marinda():
+    import sys
+    import matplotlib.pyplot as plt
+    from scipy.spatial.distance import cdist
+    from pygmi.raster.iodefs import get_raster
+
+    # Import Data
+    ifile = r"D:\Workdata\testdata.hdr"
+
+    dat = get_raster(ifile)
+
+    dat2 = []
+    for i in dat:
+        if i.dataid in ['k: 1', 'th: 1', 'u: 1']:
+            dat2.append(i)
+
+    # Generate layers via cluster analysis
+    app = QtWidgets.QApplication(sys.argv)
+
+    DM = Cluster()
+    DM.indata['Raster'] = dat2
+    DM.spinbox_maxclusters.setProperty('value', 9)
+
+    DM.settings()
+
+    cdata = DM.outdata['Cluster']
+
+    # Get cluster centers
+    centers = {}
+    for dat in cdata:
+        num = dat.metadata['Cluster']['no_clusters']
+        centers[num] = dat.metadata['Cluster']['center']
+
+    icenter = sorted(list(centers.keys()))
+
+    # Master center is dataset with most classes..
+    master = icenter[-1]
+    dist = {}
+    measure = 'euclidean'
+
+    # Relabel classes using class centers from 9 class dataset.
+    for j in icenter:
+        dist[j] = np.argmin(cdist(centers[master], centers[j], measure), 0)
+        dist[j] = dist[j] + 1
+
+    cdata2 = []
+    for dat in cdata:
+        cdata2.append(np.zeros_like(dat.data))
+        cnum = dat.metadata['Cluster']['no_clusters']
+        for i in range(cnum):
+            cdata2[-1][dat.data == i+1] = dist[cnum][i]
+        cdata2[-1] = np.ma.masked_equal(cdata2[-1], 0)
+
+    for dat in cdata2:
+        plt.figure(dpi=150)
+        num = np.ma.unique(dat).compressed().size
+        plt.title(f'Clusters: {num}')
+        plt.imshow(dat)
+        plt.show()
+
+    plt.figure(dpi=150)
+    for dat in cdata2:
+        num = np.ma.unique(dat).compressed().size
+        plt.title(f'Clusters: {num}')
+        plt.imshow(dat, alpha=0.25)
+
+    plt.show()
+
+    mdata = np.ma.mean(cdata2, 0)
+    plt.figure(dpi=150)
+    plt.imshow(mdata)
+    plt.colorbar()
+    plt.show()
+
+    breakpoint()
+
+
+def _test_marinda2():
+    import sys
+    import matplotlib.pyplot as plt
+    from pygmi.raster.iodefs import get_raster
+    from pygmi.raster.ginterp import norm2
+
+    # Import Data
+    ifile = r"D:\Workdata\testdata.hdr"
+
+    dat = get_raster(ifile)
+
+    dat2 = []
+    for i in dat:
+        if i.dataid in ['k: 1', 'th: 1', 'u: 1']:
+            dat2.append(i)
+
+    # Generate layers via cluster analysis
+    app = QtWidgets.QApplication(sys.argv)
+
+    DM = Cluster()
+    DM.indata['Raster'] = dat2
+    DM.spinbox_maxclusters.setProperty('value', 7)
+
+    DM.settings()
+
+    cdata = DM.outdata['Cluster']
+
+    mask = cdata[0].data.mask
+    for dat in cdata:
+        mask = np.logical_or(mask, dat.data.mask)
+        dat.data = dat.data.filled()
+
+    rows, cols = cdata[0].data.shape
+
+    # Get cluster centers
+    colormap = np.ma.ones((rows, cols, 4))
+    colormap[:, :, 0] = norm2(cdata[0].data)
+    colormap[:, :, 1] = norm2(cdata[1].data)
+    colormap[:, :, 2] = norm2(cdata[2].data)
+    colormap[:, :, 3] = np.logical_not(mask)
+
+
+    # colormap[:, :, 3][colormap[:, :, 0]==0] = 0
+
+    plt.figure(dpi=150)
+    plt.imshow(colormap)
+    plt.colorbar()
+    plt.show()
+
+    breakpoint()
+
+
+if __name__ == "__main__":
+    _testfn()
