@@ -26,13 +26,13 @@
 
 import numpy as np
 from PyQt5 import QtWidgets, QtCore
-from matplotlib import cm
+from matplotlib import colormaps
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 from matplotlib.ticker import MaxNLocator
 
-from pygmi.misc import frm
+from pygmi.misc import frm, ContextModule
 from pygmi.raster.modest_image import imshow
 
 
@@ -74,10 +74,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes = self.figure.add_subplot(111)
 
         cdat = data1.data
-        csp = imshow(self.axes, cdat, cmap=cm.get_cmap('jet'),
+        csp = imshow(self.axes, cdat, cmap=colormaps['jet'],
                      extent=data1.extent)
-        # csp = self.axes.imshow(cdat, cmap=cm.get_cmap('jet'),
-        #                        extent=data1.extent)
+
         vals = np.unique(cdat)
         vals = vals.compressed()
         bnds = (vals - 0.5).tolist() + [vals.max() + .5]
@@ -85,11 +84,19 @@ class MyMplCanvas(FigureCanvasQTAgg):
         if len(vals) > 1:
             self.axes.figure.colorbar(csp, boundaries=bnds, values=vals,
                                       ticks=vals)
-        # else:
-        #     self.axes.figure.colorbar(csp)
 
-        self.axes.set_xlabel('Eastings')
-        self.axes.set_ylabel('Northings')
+        if data1.crs is not None:
+            crs = data1.crs.to_dict()
+        else:
+            crs = {}
+
+        if 'proj' in crs and crs['proj'] == 'longlat':
+            self.axes.set_xlabel('Longitude')
+            self.axes.set_ylabel('Latitude')
+        else:
+            self.axes.set_xlabel('Eastings')
+            self.axes.set_ylabel('Northings')
+
         self.axes.xaxis.set_major_formatter(frm)
         self.axes.yaxis.set_major_formatter(frm)
         self.figure.canvas.draw()
@@ -172,35 +179,32 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
 
-        # rdata = self.axes.imshow(data1.metadata['Cluster']['memdat'][mem],
-        #                          extent=data1.extent, cmap=cm.get_cmap('jet'))
-
         rdata = imshow(self.axes, data1.metadata['Cluster']['memdat'][mem],
-                       extent=data1.extent, cmap=cm.get_cmap('jet'))
+                       extent=data1.extent, cmap=colormaps['jet'],
+                       vmin=0., vmax=1.)
 
         self.figure.colorbar(rdata)
 
-#        self.axes.set_title('Data')
-        self.axes.set_xlabel('Eastings')
-        self.axes.set_ylabel('Northings')
+        if data1.crs is not None:
+            crs = data1.crs.to_dict()
+        else:
+            crs = {}
+
+        if 'proj' in crs and crs['proj'] == 'longlat':
+            self.axes.set_xlabel('Longitude')
+            self.axes.set_ylabel('Latitude')
+        else:
+            self.axes.set_xlabel('Eastings')
+            self.axes.set_ylabel('Northings')
+
         self.figure.canvas.draw()
 
 
-class GraphWindow(QtWidgets.QDialog):
-    """
-    Graph Window.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    """
+class GraphWindow(ContextModule):
+    """Graph Window."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -240,23 +244,12 @@ class GraphWindow(QtWidgets.QDialog):
 
 
 class PlotRaster(GraphWindow):
-    """
-    Plot Raster Class.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+    """Plot Raster Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
         self.label2.hide()
         self.combobox2.hide()
-        self.indata = {}
-        self.parent = parent
 
     def change_band(self):
         """
@@ -289,21 +282,10 @@ class PlotRaster(GraphWindow):
 
 
 class PlotMembership(GraphWindow):
-    """
-    Plot Fuzzy Membership data.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+    """Plot Fuzzy Membership data."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
 
     def change_band(self):
         """
@@ -356,27 +338,12 @@ class PlotMembership(GraphWindow):
 
 
 class PlotVRCetc(GraphWindow):
-    """
-    Plot VRC, NCE, OBJ and XBI.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+    """Plot VRC, NCE, OBJ and XBI."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
         self.combobox2.hide()
         self.label2.hide()
-        self.parent = parent
-        self.indata = {}
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def change_band(self):
         """
