@@ -1,5 +1,5 @@
 # -----------------------------------------------------------------------------
-# Name:        graph_tool.py (part of PyGMI)
+# Name:        graphtool.py (part of PyGMI)
 #
 # Author:      Patrick Cole
 # E-Mail:      pcole@geoscience.org.za
@@ -27,7 +27,7 @@
 import numpy as np
 from PyQt5 import QtWidgets, QtCore
 from matplotlib.figure import Figure
-from matplotlib import cm
+from matplotlib import colormaps
 from matplotlib.artist import Artist
 from matplotlib.patches import Polygon
 from matplotlib.lines import Line2D
@@ -35,6 +35,7 @@ from matplotlib.path import Path
 from matplotlib.ticker import NullFormatter
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 
+from pygmi.misc import BasicModule
 
 class GraphHist(FigureCanvasQTAgg):
     """Graph Hist."""
@@ -146,16 +147,17 @@ class GraphHist(FigureCanvasQTAgg):
         # setup 1d histograms
         self.setup_hist()
 
-# Compressed eliminates the masked values so that hist
+        # Compressed eliminates the masked values so that hist
         xymahist = self.get_hist(50)
 
         self.axscatter.get_xaxis().set_visible(False)
         self.axscatter.get_yaxis().set_visible(False)
 
         self.csp = self.axscatter.imshow(xymahist.T, interpolation='nearest',
-                                         cmap=cm.get_cmap('jet'),
+                                         cmap=colormaps['jet'],
                                          aspect='auto')
 
+        self.csp.set_clim(xymahist.min(), xymahist.max())
         self.csp.set_clim(xymahist.min(), xymahist.max())
         self.csp.changed()
         self.figure.canvas.draw()
@@ -190,8 +192,8 @@ class GraphHist(FigureCanvasQTAgg):
         yptp = self.ycoord.ptp()
         xstep = xptp / 50
         ystep = yptp / 50
-        self.xcoord /= xstep
-        self.ycoord /= ystep
+        self.xcoord = self.xcoord/xstep
+        self.ycoord = self.ycoord/ystep
         self.xcoord = self.xcoord.astype(int)
         self.ycoord = self.ycoord.astype(int)
 
@@ -248,6 +250,7 @@ class GraphHist(FigureCanvasQTAgg):
 
         self.csp.set_data(xymahist.T)
         self.csp.set_clim(xymahist.min(), xymahist.max())
+        self.csp.set_clim(xymahist.min(), xymahist.max())
         self.csp.changed()
         self.figure.canvas.draw()
         self.polyi.draw_callback()
@@ -294,7 +297,7 @@ class GraphMap(FigureCanvasQTAgg):
         self.subplot.get_xaxis().set_visible(False)
         self.subplot.get_yaxis().set_visible(False)
 
-        self.csp = self.subplot.imshow(dat.data, cmap=cm.get_cmap('jet'))
+        self.csp = self.subplot.imshow(dat.data, cmap=colormaps['jet'])
         self.subplot.figure.colorbar(self.csp)
 
         self.figure.canvas.draw()
@@ -336,13 +339,26 @@ class GraphMap(FigureCanvasQTAgg):
         mtmp = self.mindx
         dat = self.data[mtmp[0]]
 
+        self.subplot.images[-1].colorbar.remove()
+
         if mtmp[1] > 0:
             cdat = self.cdata[mtmp[1] - 1].data
             self.csp.set_data(cdat)
             self.csp.set_clim(cdat.min(), cdat.max())
+            self.csp.set_clim(cdat.min(), cdat.max())
+            vals = np.unique(cdat)
+            vals = vals.compressed()
+            bnds = (vals - 0.5).tolist() + [vals.max() + .5]
+
+            if len(vals) > 1:
+                self.subplot.figure.colorbar(self.csp, boundaries=bnds,
+                                             values=vals, ticks=vals)
         else:
             self.csp.set_data(dat.data)
             self.csp.set_clim(dat.data.min(), dat.data.max())
+            self.csp.set_clim(dat.data.min(), dat.data.max())
+
+            self.subplot.figure.colorbar(self.csp)
 
         self.csp.changed()
         self.figure.canvas.draw()
@@ -350,11 +366,20 @@ class GraphMap(FigureCanvasQTAgg):
 
 
 class PolygonInteractor(QtCore.QObject):
-    """Polygon Interactor."""
+    """
+    Polygon Interactor for the graph tool.
+
+    Parameters
+    ----------
+        showverts : bool
+        epsilon : int
+        polyi_changed : signal
+
+    """
 
     showverts = True
-    epsilon = 5  # max pixel distance to count as a vertex hit
-    polyi_changed = QtCore.pyqtSignal(list)
+    epsilon = 5
+    polyi_changed = QtCore.pyqtSignal()  #: polygon changed signal.
 
     def __init__(self, axtmp, pntxy):
         super().__init__()
@@ -365,6 +390,7 @@ class PolygonInteractor(QtCore.QObject):
         self.poly.set_alpha(0.5)
         self.pntxy = pntxy
         self.ishist = True
+        self.polymask = None
         self.background = self.canvas.copy_from_bbox(self.ax.bbox)
 
         xtmp, ytmp = list(zip(*self.poly.xy))
@@ -447,7 +473,7 @@ class PolygonInteractor(QtCore.QObject):
         Parameters
         ----------
         event : TYPE
-            DESCRIPTION.
+            Unused.
 
         Returns
         -------
@@ -496,7 +522,6 @@ class PolygonInteractor(QtCore.QObject):
             xys = self.poly.get_transform().transform(self.poly.xy)
             ptmp = self.poly.get_transform().transform([event.xdata,
                                                         event.ydata])
-#            ptmp = event.x, event.y  # display coords
 
             if len(xys) == 1:
                 self.poly.xy = np.array(
@@ -531,9 +556,6 @@ class PolygonInteractor(QtCore.QObject):
                                         [(event.xdata, event.ydata)] +
                                         list(self.poly.xy[i + 1:]))
 
-            # self.poly.xy = np.array(list(self.poly.xy[:i + 1]) +
-            #                         [(event.xdata, event.ydata)] +
-            #                         list(self.poly.xy[i + 1:]))
             self.line.set_data(list(zip(*self.poly.xy)))
 
             self.canvas.restore_region(self.background)
@@ -569,8 +591,9 @@ class PolygonInteractor(QtCore.QObject):
         None.
 
         """
-        polymask = Path(self.poly.xy).contains_points(self.pntxy)
-        self.polyi_changed.emit(polymask.tolist())
+        self.polymask = Path(self.poly.xy).contains_points(self.pntxy)
+        self.polyi_changed.emit()
+
 
     def motion_notify_callback(self, event):
         """
@@ -606,33 +629,15 @@ class PolygonInteractor(QtCore.QObject):
         self.canvas.update()
 
 
-class ScatterPlot(QtWidgets.QDialog):
-    """
-    Main Graph Tool Routine.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class ScatterPlot(BasicModule):
+    """Main Graph Tool Routine."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.m1 = 0
         self.c = [0, 1, 0]
         self.m = [0, 0]
         self.dat_tmp = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
         self.map = GraphMap(self)
         self.hist = GraphHist(self)
@@ -665,7 +670,7 @@ class ScatterPlot(QtWidgets.QDialog):
         group_map = QtWidgets.QGroupBox('Map Settings')
         grid_right = QtWidgets.QGridLayout(group_map)
 
-        self.setWindowTitle('Graph Window')
+        self.setWindowTitle('Scatter Plot Tool')
 
         lbl_combo_left = QtWidgets.QLabel('X Data Band:')
         lbl_combo2_left = QtWidgets.QLabel('Y Data Band:')
@@ -695,7 +700,7 @@ class ScatterPlot(QtWidgets.QDialog):
 
     def on_cp_dpoly(self):
         """
-        On cp dpoly.
+        On cp delete polygon.
 
         Returns
         -------
@@ -714,7 +719,7 @@ class ScatterPlot(QtWidgets.QDialog):
 
     def on_map_dpoly(self):
         """
-        On map dpoly.
+        On map delete polygon.
 
         Returns
         -------
@@ -740,7 +745,8 @@ class ScatterPlot(QtWidgets.QDialog):
         if gstmp != self.c[0]:
             self.c[0] = gstmp
             self.hist.update_graph(clearaxis=True)
-            self.map.polyi.update_plots()
+            self.on_cp_dpoly()
+            # self.map.polyi.update_plots()
 
     def on_cp_combo2(self):
         """
@@ -755,7 +761,8 @@ class ScatterPlot(QtWidgets.QDialog):
         if gstmp != self.c[1]:
             self.c[1] = gstmp
             self.hist.update_graph(clearaxis=True)
-            self.map.polyi.update_plots()
+            self.on_cp_dpoly()
+            # self.map.polyi.update_plots()
 
     def on_cp_combo3(self):
         """
@@ -768,7 +775,8 @@ class ScatterPlot(QtWidgets.QDialog):
         """
         self.c[2] = self.cp_combo3.currentIndex()
         self.hist.update_graph()
-        self.map.polyi.update_plots()
+        self.on_cp_dpoly()
+        # self.map.polyi.update_plots()
 
     def on_map_combo(self):
         """
@@ -781,7 +789,8 @@ class ScatterPlot(QtWidgets.QDialog):
         """
         self.m[0] = self.map_combo.currentIndex()
         self.map.update_graph()
-        self.hist.polyi.update_plots()
+        # self.on_map_dpoly()
+        # self.hist.polyi.update_plots()
 
     def on_map_combo2(self):
         """
@@ -794,11 +803,16 @@ class ScatterPlot(QtWidgets.QDialog):
         """
         self.m[1] = self.map_combo2.currentIndex()
         self.map.update_graph()
-        self.hist.polyi.update_plots()
+        # self.hist.polyi.update_plots()
 
     def settings(self, nodialog=False):
         """
-        Run.
+        Entry point into item.
+
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
@@ -861,6 +875,9 @@ class ScatterPlot(QtWidgets.QDialog):
         self.hist.update_graph(clearaxis=True)
         self.map.update_graph()
 
+        self.exec_()
+
+
         return True
 
     def loadproj(self, projdata):
@@ -892,31 +909,24 @@ class ScatterPlot(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
-    def update_map(self, polymask):
+    def update_map(self):
         """
         Update map.
 
-        Parameters
-        ----------
-        polymask : numpy array
-            Polygon mask.
-
         Returns
         -------
         None.
 
         """
-        if max(polymask) is False:
+        polymask = self.hist.polyi.polymask
+        if polymask.max() is False:
             return
 
         mtmp = self.map_combo.currentIndex()
         mask = self.indata['Raster'][mtmp].data.mask
 
-        polymask = np.array(polymask)
         polymask.shape = mask.shape
         polymask = np.logical_or(~polymask, mask)
 
@@ -925,24 +935,19 @@ class ScatterPlot(QtWidgets.QDialog):
         self.map.csp.changed()
         self.map.figure.canvas.draw()
 
-    def update_hist(self, polymask):
+    def update_hist(self):
         """
         Update histogram.
 
-        Parameters
-        ----------
-        polymask : numpy array
-            Polygon mask.
-
         Returns
         -------
         None.
 
         """
-        if max(polymask) is False:
+        polymask = self.map.polyi.polymask
+        if polymask.max() is False:
             return
 
-        polymask = np.array(polymask)
         dattmp = self.hist.csp.get_array()
         atmp = np.array([self.hist.xcoord[polymask],
                          self.hist.ycoord[polymask]]).T
@@ -994,3 +999,28 @@ def dist_point_to_segment(p, s0, s1):
     pb = s0 + b*v
 
     return np.linalg.norm(p - pb)
+
+
+def _testfn():
+    import sys
+    from pygmi.raster.iodefs import get_raster
+
+    ifile1 = r"D:\Workdata\PyGMI Test Data\Classification\Cut_K_Th_U.ers"
+    ifile2 = r"D:\Workdata\PyGMI Test Data\Classification\export_classes.tif"
+
+    dat1 = get_raster(ifile1)
+    dat2 = get_raster(ifile2)
+    dat = dat1+dat2
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    DM = ScatterPlot()
+    DM.indata['Raster'] = dat
+    DM.indata['Cluster'] = dat2
+    DM.settings()
+
+    aaa=1
+
+
+if __name__ == "__main__":
+    _testfn()
