@@ -30,40 +30,16 @@ import skimage
 import sklearn.preprocessing as skp
 from numba import jit
 from PyQt5 import QtWidgets, QtCore, QtGui
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
+from pygmi.misc import BasicModule
 
 
-class ImageSeg(QtWidgets.QDialog):
-    """
-    Image Segmentation.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
-    """
+class ImageSeg(BasicModule):
+    """Image Segmentation."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-            self.pbar = None
-        else:
-            self.showprocesslog = parent.showprocesslog
-            self.pbar = parent.pbar
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.ifile = ''
 
         self.scale = QtWidgets.QLineEdit('1000')
         self.wcompact = QtWidgets.QLineEdit('0.5')
@@ -87,7 +63,7 @@ class ImageSeg(QtWidgets.QDialog):
         helpdocs = menu_default.HelpButton('pygmi.clust.segmentation')
 
         lbl_wcompact = QtWidgets.QLabel('Compactness weight')
-        lbl_wcolor = QtWidgets.QLabel('Color weight')
+        lbl_wcolor = QtWidgets.QLabel('Colour weight')
         lbl_scale = QtWidgets.QLabel('Maximum allowable cost function')
         lbl_eps = QtWidgets.QLabel('DBSCAN eps')
 
@@ -133,6 +109,11 @@ class ImageSeg(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -140,6 +121,7 @@ class ImageSeg(QtWidgets.QDialog):
 
         """
         if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
             return False
 
         data1 = []
@@ -252,7 +234,7 @@ class ImageSeg(QtWidgets.QDialog):
         scale : TYPE, optional
             Scale. The default is 500.
         wcolor : float, optional
-            Color weight. The default is 0.5.
+            Colour weight. The default is 0.5.
         wcompact : float, optional
             Compactness weight. The default is 0.5.
         doshape : bool, optional
@@ -291,8 +273,8 @@ class ImageSeg(QtWidgets.QDialog):
 
         self.showprocesslog('merging...')
 
-        omap = self.segment2(omap, olist, slist, mlist, nlist, bands,
-                             doshape, wcompact, wcolor, scale)
+        omap = self._segment2(omap, olist, slist, mlist, nlist, bands,
+                              doshape, wcompact, wcolor, scale)
 
         self.showprocesslog('renumbering...')
         tmp = np.unique(omap)
@@ -302,23 +284,23 @@ class ImageSeg(QtWidgets.QDialog):
 
         return omap.astype(int)
 
-    def segment2(self, omap, olist, slist, mlist, nlist, bands, doshape,
-                 wcompact, wcolor, scale):
+    def _segment2(self, omap, olist, slist, mlist, nlist, bands, doshape,
+                  wcompact, wcolor, scale):
         """
         Segment Part 2.
 
         Parameters
         ----------
         omap : numpy array
-            DESCRIPTION.
+            output data from segment1.
         olist : dictionary
-            DESCRIPTION.
+            olist from segment1.
         slist : dictionary
-            DESCRIPTION.
+            slist from segment1.
         mlist : dictionary
-            DESCRIPTION.
+            mlist from segment1.
         nlist : dictionary
-            DESCRIPTION.
+            nlist from segment1.
         bands : int
             Number of bands in data.
         doshape : bool, optional
@@ -326,7 +308,7 @@ class ImageSeg(QtWidgets.QDialog):
         wcompact : float, optional
             Compactness weight. The default is 0.5.
         wcolor : float, optional
-            Color weight. The default is 0.5.
+            Colour weight. The default is 0.5.
         scale : TYPE, optional
             Scale. The default is 500.
 
@@ -334,7 +316,7 @@ class ImageSeg(QtWidgets.QDialog):
         Returns
         -------
         omap : TYPE
-            DESCRIPTION.
+            output data.
 
         """
         wband = np.ones(bands)/bands
@@ -545,17 +527,16 @@ def _testfn():
     import matplotlib.pyplot as plt
     from pygmi.raster.datatypes import Data
     from pygmi.misc import PTime
-    from pygmi.raster import iodefs
+    from matplotlib import rcParams
 
+    rcParams['figure.dpi'] = 300
 
-    APP = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
 
     data1 = skimage.data.coffee()  # 400x600 48.5 secs
 
-
-    # data1 = data1[:, :300]  # 26
-
     plt.imshow(data1)
+    plt.axis('off')
     plt.show()
 
     wcolor = 0.5
@@ -563,7 +544,6 @@ def _testfn():
     doshape = True
     scale = 1000
 
-
     b1 = Data()
     b1.data = np.ma.array(data1[:, :, 0])
     b2 = Data()
@@ -573,74 +553,18 @@ def _testfn():
 
     data = [b1, b2, b3]
 
-    ifile = r'E:\Workdata\testdata.hdr'
-
-    data = iodefs.get_raster(ifile)
-
     ttt = PTime()
     IS = ImageSeg()
     IS.indata = {'Raster': data}
-    # IS.settings(True)
     IS.settings()
     ttt.since_last_call()
 
     odata = IS.outdata['Raster'][0]
 
     plt.imshow(odata.data)
+    plt.axis('off')
     plt.show()
 
-    # means = []
-    # for i in range(odata.data.max()+1):
-    #     tmp = data1[odata.data==i]
-    #     means.append(tmp.mean(0))
-
-    # means = np.array(means)
-    # means = skp.StandardScaler().fit_transform(means)
-
-    # dbout = DBSCAN(eps=0.15).fit_predict(means)
-
-    # data2 = odata.data.copy()
-
-    # newmax = dbout.max()+1
-    # for i, val in enumerate(dbout):
-    #     filt = (odata.data==i)
-    #     if val == -1:
-    #         data2[filt] = newmax
-    #         newmax += 1
-    #         continue
-    #     data2[filt] = val
-
-    # plt.imshow(data2, cmap='inferno')
-    # plt.colorbar()
-    # plt.show()
-
-    breakpoint()
-
-
-    # omap = segment1(data1, scale=scale, wcolor=wcolor,
-    #                 wcompact=wcompact, doshape=doshape)
-
-    # plt.figure(figsize=(8, 8))
-    # plt.imshow(omap)
-    # plt.show()
-
-#    xall = []
-#    yall = []
-#    for shp, _ in rasterio.features.shapes(omap):
-#        for crds in shp['coordinates']:
-#            x = np.array(crds)[:, 0]
-#            y = np.array(crds)[:, 1]
-#            xall.append(x)
-#            yall.append(y)
-#
-#    plt.figure(figsize=(8, 8))
-#    for i, _ in enumerate(xall):
-#        plt.plot(xall[i], yall[i], 'k')
-#    plt.imshow(data1)
-#    plt.show()
-
 
 if __name__ == "__main__":
     _testfn()
-
-    print('Finished!')
