@@ -1,5 +1,5 @@
 # -----------------------------------------------------------------------------
-# Name:        graph_tool.py (part of PyGMI)
+# Name:        super_class.py (part of PyGMI)
 #
 # Author:      Patrick Cole
 # E-Mail:      pcole@geoscience.org.za
@@ -31,7 +31,7 @@ import numpy as np
 from PyQt5 import QtWidgets, QtCore
 import matplotlib.pyplot as plt
 from matplotlib.figure import Figure
-from matplotlib import cm
+from matplotlib import colormaps
 from matplotlib.artist import Artist
 from matplotlib.patches import Polygon as mPolygon
 from matplotlib.lines import Line2D
@@ -49,7 +49,7 @@ from sklearn.tree import DecisionTreeClassifier
 from sklearn.svm import SVC
 
 from pygmi.raster.datatypes import Data
-from pygmi.misc import frm
+from pygmi.misc import frm, BasicModule
 from pygmi.raster.modest_image import imshow
 
 
@@ -91,17 +91,22 @@ class GraphMap(FigureCanvasQTAgg):
 
         self.figure.clf()
         self.subplot = self.figure.add_subplot(111)
-        # self.subplot.get_xaxis().set_visible(False)
-        # self.subplot.get_yaxis().set_visible(False)
 
-        # self.csp = self.subplot.imshow(dat.data, extent=dat.extent,
-        #                                cmap=cm.get_cmap('jet'))
         self.csp = imshow(self.subplot, dat.data, extent=dat.extent,
-                          cmap=cm.get_cmap('jet'))
+                          cmap=colormaps['jet'])
         axes = self.figure.gca()
 
-        axes.set_xlabel('Eastings')
-        axes.set_ylabel('Northings')
+        if dat.crs is not None:
+            crs = dat.crs.to_dict()
+        else:
+            crs = {}
+
+        if 'proj' in crs and crs['proj'] == 'longlat':
+            axes.set_xlabel('Longitude')
+            axes.set_ylabel('Latitude')
+        else:
+            axes.set_xlabel('Eastings')
+            axes.set_ylabel('Northings')
 
         axes.xaxis.set_major_formatter(frm)
         axes.yaxis.set_major_formatter(frm)
@@ -148,21 +153,31 @@ class GraphMap(FigureCanvasQTAgg):
             cdat = self.cdata[mtmp[1] - 1].data
             self.csp.set_data(cdat)
             self.csp.set_clim(cdat.min(), cdat.max())
+            self.csp.set_clim(cdat.min(), cdat.max())
         else:
             self.csp.set_data(dat.data)
             self.csp.set_clim(dat.data.min(), dat.data.max())
+            self.csp.set_clim(dat.data.min(), dat.data.max())
 
         self.csp.changed()
         self.figure.canvas.draw()
-#        self.polyi.draw_callback()
 
 
 class PolygonInteractor(QtCore.QObject):
-    """Polygon Interactor."""
+    """
+    Polygon Interactor for the supervised classification tool.
+
+    Parameters
+    ----------
+        showverts : bool
+        epsilon : int
+        polyi_changed : signal
+
+    """
 
     showverts = True
-    epsilon = 5  # max pixel distance to count as a vertex hit
-    polyi_changed = QtCore.pyqtSignal(list)
+    epsilon = 5
+    polyi_changed = QtCore.pyqtSignal(list)  #: polygon changed signal.
 
     def __init__(self, axtmp, pntxy):
         super().__init__()
@@ -199,7 +214,7 @@ class PolygonInteractor(QtCore.QObject):
         Parameters
         ----------
         event : TYPE, optional
-            DESCRIPTION. The default is None.
+            Unused. The default is None.
 
         Returns
         -------
@@ -314,7 +329,6 @@ class PolygonInteractor(QtCore.QObject):
             xys = self.poly.get_transform().transform(self.poly.xy)
             ptmp = self.poly.get_transform().transform([event.xdata,
                                                         event.ydata])
-#            ptmp = event.x, event.y  # display coords
 
             if len(xys) == 1:
                 self.poly.xy = np.array(
@@ -347,9 +361,7 @@ class PolygonInteractor(QtCore.QObject):
                 self.poly.xy = np.array(list(self.poly.xy[:i + 1]) +
                                         [(event.xdata, event.ydata)] +
                                         list(self.poly.xy[i + 1:]))
-            # self.poly.xy = np.array(list(self.poly.xy[:i + 1]) +
-            #                         [(event.xdata, event.ydata)] +
-            #                         list(self.poly.xy[i + 1:]))
+
             self.line.set_data(list(zip(*self.poly.xy)))
 
             self.canvas.restore_region(self.background)
@@ -425,30 +437,11 @@ class PolygonInteractor(QtCore.QObject):
         self.canvas.blit(self.ax.bbox)
 
 
-class SuperClass(QtWidgets.QDialog):
-    """
-    Main Supervised Classification Tool Routine.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class SuperClass(BasicModule):
+    """Main Supervised Classification Tool Routine."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.m1 = 0
         self.c = [0, 1, 0]
         self.m = [0, 0]
@@ -555,7 +548,7 @@ class SuperClass(QtWidgets.QDialog):
         loadshape.clicked.connect(self.load_shape)
         saveshape.clicked.connect(self.save_shape)
         calcmetrics.clicked.connect(self.calc_metrics)
-#        self.tablewidget.cellChanged.connect(self.ontablechange)
+
         self.tablewidget.currentItemChanged.connect(self.onrowchange)
         self.tablewidget.cellChanged.connect(self.oncellchange)
         self.combo_class.currentIndexChanged.connect(self.class_change)
@@ -604,7 +597,7 @@ class SuperClass(QtWidgets.QDialog):
         if self.df is None:
             return
 
-        classifier, lbls, _, X_test, y_test, tlbls = self.init_classifier()
+        classifier, _, _, X_test, y_test, tlbls = self.init_classifier()
 
         # Predicting the Test set results
         y_pred = classifier.predict(X_test)
@@ -642,8 +635,8 @@ class SuperClass(QtWidgets.QDialog):
 
         Parameters
         ----------
-        xycoords : TYPE, optional
-            DESCRIPTION. The default is None.
+        xycoords : numpy array, optional
+            x, y coordinates. The default is None.
 
         Returns
         -------
@@ -654,9 +647,8 @@ class SuperClass(QtWidgets.QDialog):
         if row == -1:
             return
 
-        self.df.loc[row] = None
+        self.df.loc[row] = pd.Series(dtype='object')
         self.df.loc[row, 'class'] = self.tablewidget.item(row, 0).text()
-#        self.df.loc[row, 'kappa'] = self.tablewidget.item(row, 1).text()
 
         xycoords = self.map.polyi.poly.xy
         if xycoords.size < 8:
@@ -683,6 +675,8 @@ class SuperClass(QtWidgets.QDialog):
         if self.tablewidget.currentItem() is None or col != 0:
             return
 
+        if row not in self.df.index:
+            self.df.loc[row] = pd.Series(dtype='object')
         self.df.loc[row, 'class'] = self.tablewidget.item(row, 0).text()
 
     def onrowchange(self, current, previous):
@@ -691,10 +685,10 @@ class SuperClass(QtWidgets.QDialog):
 
         Parameters
         ----------
-        current : TYPE
-            DESCRIPTION.
-        previous : TYPE
-            DESCRIPTION.
+        current : QTableWidgetItem
+            current item.
+        previous : QTableWidgetItem
+            previous item.
 
         Returns
         -------
@@ -715,27 +709,9 @@ class SuperClass(QtWidgets.QDialog):
 
         self.map.polyi.new_poly(coords)
 
-    def ontablechange(self, row, column):
-        """
-        Entry on table changes.
-
-        Parameters
-        ----------
-        row : TYPE
-            DESCRIPTION.
-        column : TYPE
-            DESCRIPTION.
-
-        Returns
-        -------
-        None.
-
-        """
-        self.showprocesslog(row, column)
-
     def on_apoly(self):
         """
-        On apoly.
+        On add polygon.
 
         Returns
         -------
@@ -758,9 +734,8 @@ class SuperClass(QtWidgets.QDialog):
         self.update_class_polys()
         self.map.polyi.new_poly([[1, 1]])
 
-        self.df.loc[row] = None
+        self.df.loc[row] = pd.Series(dtype='object')
         self.df.loc[row, 'class'] = self.tablewidget.item(row, 0).text()
-#        self.df.loc[row, 'kappa'] = self.tablewidget.item(row, 1).text()
         self.df.loc[row, 'geometry'] = Polygon([])
 
         self.tablewidget.selectRow(row)
@@ -768,7 +743,7 @@ class SuperClass(QtWidgets.QDialog):
 
     def on_dpoly(self):
         """
-        On dpoly.
+        On delete polygon.
 
         Returns
         -------
@@ -835,6 +810,8 @@ class SuperClass(QtWidgets.QDialog):
         coords = list(self.df.loc[0, 'geometry'].exterior.coords)
         self.map.polyi.new_poly(coords)
 
+        self.update_class_polys()
+
         return True
 
     def save_shape(self):
@@ -858,10 +835,12 @@ class SuperClass(QtWidgets.QDialog):
 
     def settings(self, nodialog=False):
         """
-        Entrypoint into class.
+        Entry point into item.
 
-        This is called when the used double clicks the routine from the
-        main PyGMI interface.
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
@@ -903,7 +882,7 @@ class SuperClass(QtWidgets.QDialog):
         yout1 = classifier.predict(datall)
         yout[~mask] = yout1
 
-        data = copy.copy(self.indata['Raster'])
+        data = copy.deepcopy(self.indata['Raster'])
         dat_out = [Data()]
 
         dat_out[-1].metadata['Cluster']['input_type'] = []
@@ -919,12 +898,10 @@ class SuperClass(QtWidgets.QDialog):
         i = len(lbls)
 
         dat_out[-1].data = zonal
-        dat_out[-1].nullvalue = zonal.fill_value
+        dat_out[-1].nodata = zonal.fill_value
         dat_out[-1].metadata['Cluster']['no_clusters'] = i
         dat_out[-1].metadata['Cluster']['center'] = np.zeros([i, len(data)])
         dat_out[-1].metadata['Cluster']['center_std'] = np.zeros([i, len(data)])
-        # if cfit.labels_.max() > 0:
-        #     dat_out[-1].metadata['Cluster']['vrc'] = skm.calinski_harabasz_score(X, cfit.labels_)
 
         m = []
         s = []
@@ -935,19 +912,15 @@ class SuperClass(QtWidgets.QDialog):
         dat_out[-1].metadata['Cluster']['center'] = np.array(m)
         dat_out[-1].metadata['Cluster']['center_std'] = np.array(s)
 
-        # self.log = ('Cluster complete' + ' (' + self.cltype+')')
-
-        dat_out[-1].wkt = data[0].wkt
-        dat_out[-1].xdim = data[0].xdim
-        dat_out[-1].ydim = data[0].ydim
+        dat_out[-1].crs = data[0].crs
         dat_out[-1].dataid = 'Clusters: '+str(dat_out[-1].metadata['Cluster']['no_clusters'])
-        dat_out[-1].nullvalue = data[0].nullvalue
-        dat_out[-1].extent = data[0].extent
+        dat_out[-1].nodata = data[0].nodata
+        dat_out[-1].set_transform(transform=data[0].transform)
 
         for i in dat_out:
             i.data += 1
             i.data = i.data.astype(np.uint8)
-            i.nullvalue = 0
+            i.nodata = 0
             i.data.data[i.data.mask] = 0
 
         self.showprocesslog('Cluster complete')
@@ -977,7 +950,6 @@ class SuperClass(QtWidgets.QDialog):
         self.DTcriterion.setCurrentText(projdata['DTcriterion'])
         self.RFcriterion.setCurrentText(projdata['RFcriterion'])
         self.SVCkernel.setCurrentText(projdata['SVCkernel'])
-#        self.df.read_json(projdata['classes'])
 
         return False
 
@@ -998,7 +970,6 @@ class SuperClass(QtWidgets.QDialog):
         projdata['DTcriterion'] = self.DTcriterion.currentText()
         projdata['RFcriterion'] = self.RFcriterion.currentText()
         projdata['SVCkernel'] = self.SVCkernel.currentText()
-#        projdata['classes'] = self.df.to_json()
 
         return projdata
 
@@ -1018,6 +989,8 @@ class SuperClass(QtWidgets.QDialog):
             X test dataset.
         y_test : numpy array
             Y test dataset.
+        tlbls : numpy array
+            Class labels.
 
         """
         ctext = self.combo_class.currentText()
@@ -1038,7 +1011,6 @@ class SuperClass(QtWidgets.QDialog):
         rows, cols = self.map.data[0].data.shape
         masks = {}
         for _, row in self.df.iterrows():
-            # pixels = list(row['geometry'].exterior.coords)
             pixels = np.array(row['geometry'].exterior.coords)
             pixels[:, 0] = pixels[:, 0]-self.map.data[0].extent[0]
             pixels[:, 0] /= self.map.data[0].xdim
@@ -1078,11 +1050,7 @@ class SuperClass(QtWidgets.QDialog):
 
         if len(lbls) < 2:
             self.showprocesslog('Error: You need at least two classes')
-            # return False
 
-        # Encoding categorical data
-        # labelencoder = LabelEncoder()
-        # y = labelencoder.fit_transform(y)
         X_train, X_test, y_train, y_test = train_test_split(x, y, stratify=y)
 
         classifier.fit(X_train, y_train)
@@ -1124,7 +1092,7 @@ class SuperClass(QtWidgets.QDialog):
 
         [p.remove() for p in reversed(axes.patches)]
 
-        for index, row in self.df.iterrows():
+        for _, row in self.df.iterrows():
             if row['geometry'] is None:
                 return
             crds = np.array(row['geometry'].exterior.coords)
@@ -1183,16 +1151,13 @@ def _testfn():
     sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                  '..//..')))
     from pygmi.raster import iodefs
-    from pygmi.misc import ProgressBarText
+
     app = QtWidgets.QApplication(sys.argv)
 
-    piter = ProgressBarText().iter
+    ifile = r"D:\Workdata\PyGMI Test Data\Classification\Cut_K_Th_U.ers"
 
-    ifile = r'E:\Workdata\people\janinetest2\coal_12052020_pan.img'
-
-    data = iodefs.get_raster(ifile, piter=piter)
-    # data = iodefs.get_raster(r'D:\Workdata\BV1_17_fx_extracted_image_1.hdr')
-    os.chdir(r'e:\Workdata\people\janinetest2')
+    data = iodefs.get_raster(ifile)
+    # os.chdir(r'd:\Workdata\people\janinetest2')
 
     tmp = SuperClass(None)
     tmp.indata['Raster'] = data
