@@ -27,21 +27,20 @@
 import sys
 import os
 import copy
+from contextlib import redirect_stdout
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
-from pymatsolver import Pardiso
 import discretize
 from SimPEG import (maps, data_misfit, regularization,
                     optimization, inversion, inverse_problem, directives)
 from SimPEG.electromagnetics import time_domain
 import SimPEG.data as Sdata
-import pygmi.menu_default as menu_default
 
-from pygmi.misc import QLabelVStack
-from contextlib import redirect_stdout
+from pygmi import menu_default
+from pygmi.misc import QLabelVStack, BasicModule
 
 
 class MyMplCanvas2(FigureCanvasQTAgg):
@@ -122,32 +121,20 @@ class MyMplCanvas2(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class TDEM1D(QtWidgets.QDialog):
+class TDEM1D(BasicModule):
     """Occam 1D inversion."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.outdata = {}
         self.data = None
-        self.parent = parent
         self.cursoln = 0
         self.times = None
 
-        if parent is None:
-            self.showprocesslog = sys.stdout
-        else:
-            self.showprocesslog = self.parent.stdio_redirect
-
         self.setWindowTitle('TDEM 1D Inversion')
         helpdocs = menu_default.HelpButton('pygmi.em.tdem1d')
 
-        # sizepolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed,
-        #                                    QtWidgets.QSizePolicy.Fixed)
-
         vbl = QtWidgets.QVBoxLayout()
         hbl = QtWidgets.QHBoxLayout(self)
-        gbl = QtWidgets.QGridLayout()
         gbl = QLabelVStack()
 
         self.mmc1 = MyMplCanvas2(self)
@@ -174,7 +161,7 @@ class TDEM1D(QtWidgets.QDialog):
         self.combobalt = QtWidgets.QComboBox()
         self.loopturns = QtWidgets.QLineEdit('1.0')
         self.loopcurrent = QtWidgets.QLineEdit('1.0')
-        self.mu = QtWidgets.QLineEdit('1.2566370614359173e-06')
+        self.mu = QtWidgets.QLineEdit('1.25663706212e-06')
         self.txarea = QtWidgets.QLineEdit('313.98')
         self.txofftime = QtWidgets.QLineEdit('0.0100286')
         self.txrampoff1 = QtWidgets.QLineEdit('0.01')
@@ -326,17 +313,17 @@ class TDEM1D(QtWidgets.QDialog):
         # cs, ncx, ncz, npad = 1., 10., 10., 20
         hx = [(cs, ncx), (cs, npad, padrate)]
         hz = [(cs, npad, -padrate), (cs, ncz*2), (cs, npad, padrate)]
-        mesh = discretize.CylMesh([hx, 1, hz], '00C')
+        mesh = discretize.CylindricalMesh([hx, 1, hz], '00C')
 
         # Step2: Set a SurjectVertical1D mapping
         # Note: this sets our inversion model as 1D log conductivity
         # below subsurface
 
-        active = mesh.vectorCCz < 0.
+        active = mesh.cell_centers_z < 0.
         actmap = maps.InjectActiveCells(mesh, active, np.log(1e-8),
-                                        nC=mesh.nCz)
+                                        nC=mesh.shape_cells[2])
         mapping = maps.ExpMap(mesh) * maps.SurjectVertical1D(mesh) * actmap
-        sigma = np.ones(mesh.nCz) * sig_air
+        sigma = np.ones(mesh.shape_cells[2]) * sig_air
         sigma[active] = sig_half
 
         # Initial and reference model
@@ -357,8 +344,8 @@ class TDEM1D(QtWidgets.QDialog):
         # Note: we are Using theoretical VTEM waveform,
         # but effectively fits SkyTEM waveform
 
-        dbdt_z = time_domain.Rx.Point_dbdt(locs=rxloc, times=times,
-                                           orientation=rxori)  # vertical db_dt
+        dbdt_z = time_domain.Rx.PointMagneticFluxTimeDerivative(
+            locations=rxloc, times=times, orientation=rxori)  # vertical db_dt
         rxlist = [dbdt_z]  # list of receivers
 
         wform = self.update_wave()
@@ -368,13 +355,13 @@ class TDEM1D(QtWidgets.QDialog):
                                                     N=loopturns,
                                                     mu=mu,
                                                     current=loopcurrent,
-                                                    loc=srcloc,
+                                                    location=srcloc,
                                                     radius=radius,
                                                     orientation=txori,
                                                     waveform=wform)]
         elif stype == 'MagDipole':
             srclist = [time_domain.Src.MagDipole(rxlist,
-                                                 loc=srcloc,
+                                                 location=srcloc,
                                                  mu=mu,
                                                  orientation=txori,
                                                  waveform=wform)]
@@ -388,10 +375,9 @@ class TDEM1D(QtWidgets.QDialog):
 
         survey = time_domain.Survey(srclist)
         sim = time_domain.Simulation3DElectricField(mesh,
-                                                    timeSteps=timesteps,
+                                                    time_steps=timesteps,
                                                     sigmaMap=mapping,
-                                                    survey=survey,
-                                                    Solver=Pardiso)
+                                                    survey=survey)
 
         src = srclist[0]
         wave = []
@@ -410,12 +396,13 @@ class TDEM1D(QtWidgets.QDialog):
         dmisfit = data_misfit.L2DataMisfit(data=data, simulation=sim)
 
         # Regularization
-        regmesh = discretize.TensorMesh([mesh.hz[mapping.maps[-1].indActive]])
+        regmesh = discretize.TensorMesh([mesh.h[2][mapping.maps[-1].indActive]])
         # reg = regularization.Simple(regmesh, mapping=maps.IdentityMap(regmesh))
         # reg.alpha_s = 1e-2
         # reg.alpha_x = 1.
 
-        reg = regularization.Tikhonov(regmesh, alpha_s=1e-2)
+        # reg = regularization.Tikhonov(regmesh, alpha_s=1e-2)
+        reg = regularization.WeightedLeastSquares(regmesh)
 
         # Optimization
         opt = optimization.InexactGaussNewton(maxIter=maxiter, LSshorten=0.5)
@@ -436,18 +423,19 @@ class TDEM1D(QtWidgets.QDialog):
 
         # run the inversion
         try:
-            with redirect_stdout(self.showprocesslog):
+            with redirect_stdout(self.stdout_redirect):
                 mopt_sky = inv.run(m0)
         except Exception as e:
             QtWidgets.QMessageBox.warning(self.parent, 'Error', str(e),
                                           QtWidgets.QMessageBox.Ok)
             return
 
-        dpred_sky = invprob.dpred
+        dpred_sky = np.array(invprob.dpred)
 
         sigma = np.repeat(np.exp(mopt_sky), 2, axis=0)
-        z = np.repeat(mesh.vectorCCz[active][1:], 2, axis=0)
-        z = np.r_[mesh.vectorCCz[active][0], z, mesh.vectorCCz[active][-1]]
+        z = np.repeat(mesh.cell_centers_z[active][1:], 2, axis=0)
+        z = np.r_[mesh.cell_centers_z[active][0], z,
+                  mesh.cell_centers_z[active][-1]]
 
         times_off = ((times - offtime)*1e6)
         zobs = dobs_sky/txarea
@@ -455,8 +443,6 @@ class TDEM1D(QtWidgets.QDialog):
 
         self.mmc.update_line(sigma, z, times_off, zobs, zpred)
 
-#        self.outdata['Line'] = self.data
-
     def change_source(self):
         """
         Change Source.
@@ -532,17 +518,17 @@ class TDEM1D(QtWidgets.QDialog):
         wtype = self.combowtype.currentText()
 
         if wtype == 'VTEMWaveform':
-            wform = time_domain.sources.VTEMWaveform(offTime=offtime,
-                                                     peakTime=peaktime)
+            wform = time_domain.sources.VTEMWaveform(off_time=offtime,
+                                                     peak_time=peaktime)
         elif wtype == 'TrapezoidWaveform':
             wform = time_domain.sources.TrapezoidWaveform(ramp_on=rampon,
                                                           ramp_off=rampoff)
         elif wtype == 'TriangularWaveform':
-            wform = time_domain.sources.TriangularWaveform(peakTime=peaktime,
-                                                           offTime=offtime)
+            wform = time_domain.sources.TriangularWaveform(peak_time=peaktime,
+                                                           off_time=offtime)
         elif wtype == 'QuarterSineRampOnWaveform':
-            wform = time_domain.sources.QuarterSineRampOnWaveform(ramp_on=rampon,
-                                                                  ramp_off=rampoff)
+            wform = time_domain.sources.QuarterSineRampOnWaveform(
+                ramp_on=rampon, ramp_off=rampoff)
         elif wtype == 'HalfSineWaveform':
             wform = time_domain.sources.HalfSineWaveform(ramp_on=rampon,
                                                          ramp_off=rampoff)
@@ -598,6 +584,11 @@ class TDEM1D(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -623,7 +614,6 @@ class TDEM1D(QtWidgets.QDialog):
 
         cnames = list(self.data.columns[filt])
 
-#        cnames = list(self.data.data.values())[0].dtype.names
         self.combobalt.addItems(cnames)
         for i, tmp in enumerate(cnames):
             tmp = tmp.lower()
@@ -635,18 +625,12 @@ class TDEM1D(QtWidgets.QDialog):
         lines = list(self.data.line.unique().astype(str))
         self.comboline.addItems(lines)
 
-#        for i in self.data.data:
-#            self.comboline.addItem(i)
-#        fid = self.data.fid.values.astype(str)
-
         self.comboline.setCurrentIndex(0)
         line = self.comboline.currentText()
 
         fid = self.data.fid[self.data.line.astype(str) == line].values.astype(str)
         self.combofid.addItems(fid)
 
-#        self.combofid.addItems(self.data.data[line]['fid'].astype(str))
-
         self.comboline.currentIndexChanged.connect(self.change_line)
 
         tmp = self.exec_()
@@ -654,8 +638,6 @@ class TDEM1D(QtWidgets.QDialog):
         if tmp != 1:
             return False
 
-#        self.acceptall()
-
         return True
 
     def loadproj(self, projdata):
@@ -687,8 +669,6 @@ class TDEM1D(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
 
@@ -728,8 +708,8 @@ def _testfn():
     app = QtWidgets.QApplication(sys.argv)
 
     # Load in line data
-    filename = r'C:\Work\Workdata\EM\SK655CS_Bookpurnong_ZX_HM_TxInc_newDTM.txt'
-    wfile = r'C:\Work\Workdata\EM\wtimes.txt'
+    filename = r'D:\Workdata\PyGMI Test Data\EM\SK655CS_Bookpurnong_ZX_HM_TxInc_newDTM.txt'
+    wfile = r'D:\Workdata\PyGMI Test Data\EM\wtimes.txt'
 
     IO = ImportLineData()
     IO.filt = 'Tab Delimited (*.txt)'
@@ -746,7 +726,6 @@ def _testfn():
 
     tmp.indata = IO.outdata
 
-    # os.environ['KMP_WARNINGS'] ='0'
     tmp.settings(True)
 
 
