@@ -22,43 +22,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # -----------------------------------------------------------------------------
-"""A set of Data Preparation routines."""
+"""A set of data processing routines for gravity."""
 
 import sys
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import matplotlib.pyplot as plt
-import pygmi.menu_default as menu_default
-import pygmi.grav.iodefs as iodefs
+
+from pygmi import menu_default
+from pygmi.grav import iodefs
+from pygmi.misc import BasicModule
 
 
-class ProcessData(QtWidgets.QDialog):
+class ProcessData(BasicModule):
     """
     Process Gravity Data.
 
     This class processes gravity data.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-
         self.dsb_dxy = QtWidgets.QDoubleSpinBox()
         self.dsb_null = QtWidgets.QDoubleSpinBox()
         self.dataid = QtWidgets.QComboBox()
@@ -121,6 +105,11 @@ class ProcessData(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -224,7 +213,7 @@ class ProcessData(QtWidgets.QDialog):
 
         """
         pdat = self.indata['Line']['Gravity']
-#        pdat.sort_values(by=['DECTIMEDATE'], inplace=True)
+        pdat.sort_values(by=['DECTIMEDATE'], inplace=True)
 
         basethres = float(self.basethres.text())
         kstat = self.knownstat.text()
@@ -233,26 +222,27 @@ class ProcessData(QtWidgets.QDialog):
         else:
             kstat = float(kstat)
 
-# Make sure there are no local base stations before the known base
+        # Make sure there are no local base stations before the known base
         if kstat in pdat['STATION']:
             tmp = (pdat['STATION'] == kstat)
             itmp = np.nonzero(tmp)[0][0]
             pdat = pdat[itmp:]
 
-# Drift Correction, to abs base value
+        # Drift Correction, to abs base value
         tmp = pdat[pdat['STATION'] >= basethres]
 
         driftdat = tmp[tmp['STATION'] != kstat]
         pdat = pdat[pdat['STATION'] < basethres]
 
-#        x = pdat['DECTIMEDATE'].values
-#        xp1 = driftdat['DECTIMEDATE'].values
+        if tmp.STATION.unique()[0] == kstat and tmp.STATION.unique().size == 1:
+            driftdat = tmp[tmp['STATION'] == kstat]
+
         xp1 = driftdat['TIME'].apply(time_convert)
 
         fp = driftdat['GRAV'].values
 
-        x = pdat.index.values
-        xp = driftdat.index.values
+        x = pdat['DECTIMEDATE'].values
+        xp = driftdat['DECTIMEDATE'].values
 
         dcor = np.interp(x, xp, fp)
 
@@ -279,9 +269,15 @@ class ProcessData(QtWidgets.QDialog):
             x2 = xp1[filt].values/60.
             dcor2 = fp[filt]
             drifttime = (x2[-1]-x2[0])
-            driftrate = (dcor2[-1]-dcor2[0])/drifttime
-            self.showprocesslog(f'Day {iday+1} drift: {driftrate:.3e} '
-                                f'mGal/min over {drifttime:.3f} minutes.')
+            if drifttime == 0.:
+                self.showprocesslog(f'Day {iday+1} drift: Only one reading, '
+                                    'no drift result possible.')
+                driftrate = np.nan
+
+            else:
+                driftrate = (dcor2[-1]-dcor2[0])/drifttime
+                self.showprocesslog(f'Day {iday+1} drift: {driftrate:.3e} '
+                                    f'mGal/min over {drifttime:.3f} minutes.')
             dday.append(iday+1+x2[-1]/1440)
             drate.append(driftrate)
             dtime.append(drifttime)
@@ -303,28 +299,26 @@ class ProcessData(QtWidgets.QDialog):
             plt.ylabel('mGal/min')
             plt.grid(True)
             plt.plot(dday, drate, '.-')
-            # plt.xticks(range(1, ix[-1]+2, 1))
             plt.tight_layout()
 
             plt.get_current_fig_manager().window.setWindowIcon(self.parent.windowIcon())
             plt.show()
 
         gobs = pdat['GRAV'] - dcor + float(self.absbase.text())
-###################################################################
 
-# Variables used
+        # Variables used
         lat = np.deg2rad(pdat.latitude)
         h = pdat['elevation']  # This is the ellipsoidal (gps) height
         dens = float(self.density.text())
 
-# Corrections
+        # Corrections
         gT = theoretical_gravity(lat)
         gATM = atmospheric_correction(h)
         gHC = height_correction(lat, h)
         gSB = spherical_bouguer(h, dens)
 
-# Bouguer Anomaly
-        gba = gobs - gT + gATM - gHC - gSB  # add or subtract atm
+        # Bouguer Anomaly
+        gba = gobs - gT + gATM - gHC - gSB
 
         pdat = pdat.assign(dcor=dcor)
         pdat = pdat.assign(gobs_drift=gobs)
@@ -367,11 +361,14 @@ class ProcessData(QtWidgets.QDialog):
                                           txt, QtWidgets.QMessageBox.Ok)
             return
 
-# Drift Correction, to abs base value
+        # Drift Correction, to abs base value
         tmp = pdat[pdat['STATION'] > basethres]
         kbasevals = tmp[tmp['STATION'] == kstat]
         abasevals = tmp[tmp['STATION'] != kstat]
 
+        if tmp.STATION.unique()[0] == kstat and tmp.STATION.unique().size == 1:
+            abasevals = kbasevals
+
         x = abasevals['DECTIMEDATE']
         grv = abasevals['GRAV']
         xp = kbasevals['DECTIMEDATE']
@@ -548,41 +545,46 @@ def time_convert(x):
 
 def _testfn():
     """Test routine."""
-    APP = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
 
-    grvfile = r'C:\Work\Workdata\gravity\skeifontein 2018.txt'
-    gpsfile = r'C:\Work\Workdata\gravity\skei_dgps.csv'
+    grvfile = r'd:\Workdata\gravity\skeifontein 2018.txt'
+    gpsfile = r'd:\Workdata\gravity\skei_dgps.csv'
+    kbase = '88888'
+    bthres = '10000'
 
-    # grvfile = r'C:\Work\Workdata\gravity\Laxeygarvity until2511.txt'
-    # gpsfile = r'C:\Work\Workdata\gravity\laxey.dgps.csv'
+    grvfile = r"D:\Workdata\bugs\grav\Combined_Profile1_Processed.txt"
+    gpsfile = r"D:/Workdata/bugs/grav/GPS profil 1 lat lon.csv"
+    kbase = '111111'
+    bthres = '100000'
 
-# Import Data
+    # Import Data
     IO = iodefs.ImportCG5(None)
+    IO.basethres.setText(bthres)
     IO.get_cg5(grvfile)
     IO.get_gps(gpsfile)
     IO.settings(True)
 
-# Process Data
+    # Process Data
     PD = ProcessData()
     PD.indata = IO.outdata
-    PD.knownstat.setText('88888.0')
-    PD.knownbase.setText('978864.74')
+    PD.basethres.setText(bthres)
+    PD.knownstat.setText(kbase)
+    PD.knownbase.setText('978794.53')
     PD.calcbase()
 
-    PD.settings(False)
+    PD.settings(True)
 
     datout = PD.outdata['Line']
 
     gdf = datout['Gravity']
 
-    gdf = gdf[(gdf.STATION > 4470) & (gdf.STATION < 4472)]
-#    gdf = gdf[(gdf.STATION > 2213) & (gdf.STATION < 2900)]
+    for i in ['GRAV', 'gobs_drift', 'BOUGUER', 'dcor', 'elevation',
+              'gT', 'gATM', 'gHC', 'gSB']:
+        plt.title(i)
 
-    plt.plot(gdf.longitude, gdf.latitude, '.')
-    plt.show()
+        plt.plot(gdf.x, gdf[i])
+        plt.show()
 
 
 if __name__ == "__main__":
     _testfn()
-
-    print('Finished!')
