@@ -28,40 +28,20 @@ import os
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import pandas as pd
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
+from pygmi.misc import BasicModule
 
 
-class ImportCG5(QtWidgets.QDialog):
+class ImportCG5(BasicModule):
     """
     Import Line Data.
 
     This class imports ASCII point data.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
     """
 
     def __init__(self, parent):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.ifile = ''
 
         self.df_cg5 = None
         self.df_gps = None
@@ -135,9 +115,6 @@ class ImportCG5(QtWidgets.QDialog):
         gridlayout_main.addWidget(label_bthres, 7, 0, 1, 1)
         gridlayout_main.addWidget(self.basethres, 7, 1, 1, 1)
 
-#        gridlayout_main.addWidget(label_nodata, 2, 0, 1, 1)
-#        gridlayout_main.addWidget(self.nodata, 2, 1, 1, 1)
-
         gridlayout_main.addWidget(helpdocs, 8, 0, 1, 1)
         gridlayout_main.addWidget(buttonbox, 8, 1, 1, 3)
 
@@ -150,6 +127,11 @@ class ImportCG5(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -228,21 +210,10 @@ class ImportCG5(QtWidgets.QDialog):
                                             downcast='float')
 
         # Merge data
-#        self.df_cg5.sort_values(by=['LINE', 'STATION'], inplace=True)
-#        self.df_gps.sort_values(by=['line', 'station'], inplace=True)
-
         dfmerge = pd.merge(self.df_cg5, self.df_gps,
                            left_on=['LINE', 'STATION'],
                            right_on=['line', 'station'], how='left')
 
-#        dfmerge = pd.merge_asof(self.df_cg5, self.df_gps,
-#                                left_on=['STATION'],
-#                                right_on=['station'],
-#                                left_by=['LINE'],
-#                                right_by=['line'],
-#                                direction='forward',
-#                                tolerance=0.9)
-
         # eliminate ordinary stations (not base stations) without coordinates
         filt = dfmerge['STATION'] < float(self.basethres.text())
 
@@ -259,7 +230,7 @@ class ImportCG5(QtWidgets.QDialog):
         dtest = dfmerge.duplicated(['LINE', 'STATION'])
         dlist = dfmerge[['LINE', 'STATION']].loc[dtest]
         dlist = dlist[~dlist.duplicated()]
-        dlist = dlist[dlist.STATION < 10000]
+        dlist = dlist[dlist.STATION < float(self.basethres.text())]
 
         if dlist.size > 0:
             self.showprocesslog('Warning, the following are duplicated:')
@@ -321,9 +292,6 @@ class ImportCG5(QtWidgets.QDialog):
         projdata['gpsfile'] = self.gpsfile.text()
         projdata['basethres'] = self.basethres.text()
 
-
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
     def get_cg5(self, filename=''):
@@ -350,7 +318,7 @@ class ImportCG5(QtWidgets.QDialog):
 
         os.chdir(os.path.dirname(filename))
 
-        with open(filename) as fno:
+        with open(filename, encoding='utf-8') as fno:
             tmp = fno.readlines()
 
         data = []
@@ -400,7 +368,6 @@ class ImportCG5(QtWidgets.QDialog):
         os.chdir(os.path.dirname(filename))
 
         df2 = pd.read_csv(filename)
-#        df2.columns = map(str.lower, df2.columns)
         df2.columns = df2.columns.str.lower()
 
         self.df_gps = df2
@@ -420,7 +387,7 @@ class ImportCG5(QtWidgets.QDialog):
             elif 'lat' in tmp.lower():
                 yind = i
             elif ('elev' in tmp.lower() or 'alt' in tmp.lower() or
-                  'height' in tmp.lower()):
+                  'height' in tmp.lower() or tmp.lower() == 'z'):
                 zind = i
             elif 'stat' in tmp.lower():
                 sind = i
