@@ -29,23 +29,19 @@ from PyQt5 import QtWidgets, QtCore
 import numpy as np
 from scipy.signal import tukey
 import scipy.interpolate as si
-import scipy.signal as signal
-import pygmi.menu_default as menu_default
+from scipy import signal
+from pygmi import menu_default
 
 from pygmi.raster.datatypes import Data
+from pygmi.misc import BasicModule
 
 
-class Tilt1(QtWidgets.QDialog):
+class Tilt1(BasicModule):
     """
     Class used to gather information via a GUI, for function tilt1.
 
     Attributes
     ----------
-    parent : parent
-    indata : dictionary
-        PyGMI input data in a dictionary
-    outdata :
-        PyGMI input data in a dictionary
     azi : float
         directional filter azimuth in degrees from East
     smooth : int
@@ -55,17 +51,9 @@ class Tilt1(QtWidgets.QDialog):
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
         self.azi = 75
         self.smooth = 0
 
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
-
         self.sb_azi = QtWidgets.QSpinBox()
         self.sb_s = QtWidgets.QSpinBox()
 
@@ -115,12 +103,21 @@ class Tilt1(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         if not nodialog:
             temp = self.exec_()
             if temp == 0:
@@ -151,7 +148,7 @@ class Tilt1(QtWidgets.QDialog):
             data2[-1].dataid += ' Total Derivative'
 
         for i in data2:
-            i.data.data[i.data.mask] = i.nullvalue
+            i.data.data[i.data.mask] = i.nodata
 
         self.outdata['Raster'] = data2
         return True
@@ -203,7 +200,7 @@ def tilt1(data, azi, s):
 
     Parameters
     ----------
-    data : numpy array
+    data : numpy masked array
         matrix of double to be filtered
     azi : float
         directional filter azimuth in degrees from East
@@ -217,7 +214,7 @@ def tilt1(data, azi, s):
     th : numpy masked array
         Hyperbolic tilt angle
     t2 : numpy masked array
-        2nd order tilt angle
+        Second order tilt angle
     ta : numpy masked array
         Tilt Based Directional Derivative
     tdx : numpy masked array
@@ -241,14 +238,13 @@ def tilt1(data, azi, s):
     azi = azi*dtr
 
     dy, dx = np.gradient(data)
-#    dx = dx.astype(np.float64)
-#    dy = dy.astype(np.float64)
     dxtot = np.ma.sqrt(dx*dx+dy*dy)
     nmax = np.max([nr, nc])
     npts = int(2**nextpow2(nmax))
     dz = vertical(data, npts, 1)
     t1 = np.ma.arctan(dz/dxtot)
     th = np.real(np.arctanh(np.nan_to_num(dz/dxtot)+(0+0j)))
+
     tdx = np.real(np.ma.arctan(dxtot/abs(dz)))
 
     dx1 = dx*np.cos(azi)+dy*np.sin(azi)  # Standard directional derivative
@@ -258,11 +254,11 @@ def tilt1(data, azi, s):
 
     # 2nd order Tilt angle
 
-    ts = t1
-    if s < 3:
-        s = 3
+    s = max(s, 3)
     se = np.ones([s, s])/(s*s)
-    ts = signal.convolve2d(t1, se, 'same')
+    ts = signal.convolve2d(t1.filled(t1.mean()), se, 'same')
+    ts = np.ma.array(ts, mask=t1.mask)
+
     [dxs, dys] = np.gradient(ts)
     dzs = vertical(ts, npts, 1)
     dxtots = np.ma.sqrt(dxs*dxs+dys*dys)
@@ -270,13 +266,6 @@ def tilt1(data, azi, s):
 
     # Standard tilt angle, hyperbolic tilt angle, 2nd order tilt angle,
     # Tilt Based Directional Derivative, Total Derivative
-    t1 = np.ma.array(t1)
-    th = np.ma.array(th)
-    th.mask = np.ma.getmaskarray(t1)
-    t2 = np.ma.array(t2)
-    t2.mask = np.ma.getmaskarray(t1)
-    ta = np.ma.array(ta)
-    tdx = np.ma.array(tdx)
 
     return t1, th, t2, ta, tdx
 
@@ -318,6 +307,8 @@ def vertical(data, npts=None, xint=1, order=1):
         Number of points. The default is None.
     xint : float, optional
         X interval. The default is 1.
+    order : int
+        Order of derivative. The default is 1.
 
     Returns
     -------
@@ -360,32 +351,12 @@ def vertical(data, npts=None, xint=1, order=1):
     return dz
 
 
-class RTP(QtWidgets.QDialog):
-    """
-    Perform Reduction to the Pole on Magnetic data.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class RTP(BasicModule):
+    """Perform Reduction to the Pole on Magnetic data."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
-
         self.dataid = QtWidgets.QComboBox()
         self.dsb_inc = QtWidgets.QDoubleSpinBox()
         self.dsb_dec = QtWidgets.QDoubleSpinBox()
@@ -438,6 +409,11 @@ class RTP(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -446,6 +422,7 @@ class RTP(QtWidgets.QDialog):
         """
         tmp = []
         if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
             return False
 
         for i in self.indata['Raster']:
@@ -529,18 +506,23 @@ class RTP(QtWidgets.QDialog):
 
 def fftprep(data):
     """
-    FFT preparation.
+    FFT Preparation.
 
     Parameters
     ----------
-    data : TYPE
-        DESCRIPTION.
-    dxy : TYPE
-        DESCRIPTION.
+    data : numpy array
+        Input dataset.
 
     Returns
     -------
-    None.
+    zfin : numpy array.
+        Output prepared data.
+    rdiff : int
+        rows divided by 2.
+    cdiff : int
+        columns divided by 2.
+    datamedian : float
+        Median of data.
 
     """
     datamedian = np.ma.median(data.data)
@@ -573,7 +555,7 @@ def fftprep(data):
 
     points = np.transpose([x, y])
 
-    zfin = si.griddata(points, z, (x1, y1), method='linear')
+    zfin = si.griddata(points, z, (x1, y1), method='linear', fill_value=0.)
 
     nr, nc = zfin.shape
     zfin *= tukey(nc)
@@ -588,14 +570,19 @@ def fft_getkxy(fftmod, xdim, ydim):
 
     Parameters
     ----------
-    fftmod : TYPE
-        DESCRIPTION.
-    dxy : TYPE
-        DESCRIPTION.
+    fftmod : numpy array
+        FFT data.
+    xdim : float
+        cell x dimension.
+    ydim : float
+        cell y dimension.
 
     Returns
     -------
-    None.
+    KX : numpy array
+        x sample frequencies.
+    KY : numpy array
+        y sample frequencies.
 
     """
     ny, nx = fftmod.shape
@@ -632,7 +619,6 @@ def rtp(data, I_deg, D_deg):
     ndat, rdiff, cdiff, datamedian = fftprep(data)
     fftmod = np.fft.fft2(ndat)
 
-    ny, nx = fftmod.shape
     KX, KY = fft_getkxy(fftmod, xdim, ydim)
 
     I = np.deg2rad(I_deg)
@@ -650,41 +636,65 @@ def rtp(data, I_deg, D_deg):
     dat = Data()
     dat.data = np.ma.masked_invalid(zout)
     dat.data.mask = np.ma.getmaskarray(data.data)
-    dat.nullvalue = data.data.fill_value
+    dat.nodata = data.data.fill_value
     dat.dataid = 'RTP_'+data.dataid
-    dat.extent = data.extent
-    dat.xdim = data.xdim
-    dat.ydim = data.ydim
+    dat.set_transform(transform=data.transform)
+    dat.crs = data.crs
 
     return dat
 
 
-def _testfn():
+def _testfn_rtp():
     """RTP testing routine."""
     import matplotlib.pyplot as plt
-    from matplotlib import cm
+    from matplotlib import colormaps
     from pygmi.pfmod.grvmag3d import quick_model, calc_field
     from IPython import get_ipython
     get_ipython().run_line_magic('matplotlib', 'inline')
 
-# quick model
+    # quick model
     finc = -57
     fdec = 50
 
     lmod = quick_model(numx=300, numy=300, numz=30, finc=finc, fdec=fdec)
     lmod.lith_index[100:200, 100:200, 0:10] = 1
-#    lmod.lith_index[:, :, 10] = 1
     lmod.mht = 100
     calc_field(lmod, magcalc=True)
 
-# Calculate the field
+    # Calculate the field
 
     magval = lmod.griddata['Calculated Magnetics'].data
-    plt.imshow(magval, cmap=cm.get_cmap('jet'))
+    plt.imshow(magval, cmap=colormaps['jet'])
     plt.show()
 
     dat2 = rtp(lmod.griddata['Calculated Magnetics'], finc, fdec)
-    plt.imshow(dat2.data, cmap=cm.get_cmap('jet'))
+    plt.imshow(dat2.data, cmap=colormaps['jet'])
+    plt.show()
+
+
+def _testfn():
+    """RTP testing routine."""
+    import matplotlib.pyplot as plt
+    from pygmi.raster.iodefs import get_raster
+
+    from IPython import get_ipython
+    get_ipython().run_line_magic('matplotlib', 'inline')
+
+    ifile = 'd:/Workdata/bugs/detlef/TMI_norm_wdw.tif'
+    ifile = r'C:/Workdata/raster/ER Mapper/magmicrolevel.PD.ers'
+
+    dat = get_raster(ifile)[0]
+
+    t1, th, t2, ta, tdx = tilt1(dat.data, 75, 0)
+
+    plt.figure(dpi=150)
+    plt.imshow(t2)
+    plt.colorbar()
+    plt.show()
+
+    plt.figure(dpi=150)
+    plt.imshow(t1)
+    plt.colorbar()
     plt.show()
 
 
