@@ -45,12 +45,11 @@ from PyQt5 import QtWidgets, QtCore, QtGui
 import numpy as np
 from matplotlib import rcParams
 from matplotlib import interactive
-import pygmi
-import pygmi.menu_default as menu_default
-import pygmi.misc as misc
 
-# rcParams['figure.dpi'] = 120
-# rcParams['font.size'] = 6.0
+import pygmi
+from pygmi import menu_default
+from pygmi import misc
+
 rcParams['axes.formatter.limits'] = [-12, 12]
 rcParams['axes.formatter.useoffset'] = False
 interactive(False)
@@ -70,7 +69,7 @@ class Arrow(QtWidgets.QGraphicsLineItem):
         Starting DiagramItem object. This will send information to my_end_item
     my_end_item : DiagramItem
         End DiagramItem object. This will get information from my_start_item
-    my_color : QtCore color (default is QtCore.Qt.black)
+    my_color : QtCore colour (default is QtCore.Qt.black)
         Color
     """
 
@@ -300,10 +299,11 @@ class DiagramItem(QtWidgets.QGraphicsPolygonItem):
         """
         self.scene().clearSelection()
         self.setSelected(True)
-        exclude = ['GenFPS', 'SceneList', 'RasterFileList']
+        exclude = ['GenFPS', 'SceneList']
 
         tmp = self.context_menu['Basic'].actions()
-        if 'Raster' in self.my_class.indata:
+        if ('Raster' in self.my_class.indata and
+                'RasterFileList' not in self.my_class.indata):
             tmp += self.context_menu['inRaster'].actions()
 
         tmplist = list(self.my_class.outdata.keys())
@@ -319,7 +319,7 @@ class DiagramItem(QtWidgets.QGraphicsPolygonItem):
         Mouse double click event.
 
         This event is used to activate an item. It does this by calling the
-        settings() method of the item. The event also changes the color of the
+        settings() method of the item. The event also changes the colour of the
         item to reflect whether it is busy working.
         """
         self.setBrush(QtGui.QColor(255, 0, 0, 127))
@@ -369,7 +369,6 @@ class DiagramItem(QtWidgets.QGraphicsPolygonItem):
             method.
 
         """
-        # self.update_indata()
         if self.my_class.indata == {} and self.is_import is False:
             QtWidgets.QMessageBox.warning(self.parent, 'Warning',
                                           ' You need to connect data first!',
@@ -548,7 +547,7 @@ class MainWidget(QtWidgets.QMainWindow):
     context_menu : dictionary
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, nocgs=True):
         super().__init__(parent)
 
         ipth = os.path.dirname(menu_default.__file__)+r'/images/'
@@ -558,8 +557,6 @@ class MainWidget(QtWidgets.QMainWindow):
         self.context_menu = {}
         self.add_to_context('Basic')
 
-        self.stdio_redirect = EmittingStream(self.showprocesslog)
-
         self.menubar = QtWidgets.QMenuBar()
 
         self.statusbar = QtWidgets.QStatusBar()
@@ -604,27 +601,13 @@ class MainWidget(QtWidgets.QMainWindow):
                 onerror=lambda x: None):
             menus.append(modname)
 
-        # menus.pop(menus.index('pygmi.em.menu'))
+        if nocgs is True and 'pygmi.cgs.menu' in menus:
+            menus.pop(menus.index('pygmi.cgs.menu'))
         raster_menu = menus.pop(menus.index('pygmi.raster.menu'))
         vector_menu = menus.pop(menus.index('pygmi.vector.menu'))
         menus = [raster_menu, vector_menu]+menus
         menus = [i for i in menus if 'menu' in i[-5:]]
 
-        # menus = [
-        #          'pygmi.raster.menu',
-        #          'pygmi.em.menu',
-        #          'pygmi.vector.menu',
-        #          'pygmi.bholes.menu',
-        #          'pygmi.clust.menu',
-        #          'pygmi.eseis.menu',
-        #          'pygmi.grav.menu',
-        #          'pygmi.mag.menu',
-        #          'pygmi.mt.menu',
-        #          'pygmi.pfmod.menu',
-        #          'pygmi.rsense.menu',
-        #          'pygmi.seis.menu',
-        #          ]
-
         start = Startup(len(menus)+1)
         start.update()
 
@@ -647,7 +630,7 @@ class MainWidget(QtWidgets.QMainWindow):
         self.view = self.graphics_view
         self.view.setScene(self.scene)
 
-# Menus
+        # Menus
         self.action_run.triggered.connect(self.run)
         self.action_pointer.triggered.connect(self.pointer)
         self.action_linepointer.triggered.connect(self.linepointer)
@@ -656,7 +639,7 @@ class MainWidget(QtWidgets.QMainWindow):
         self.action_send_to_back.triggered.connect(self.send_to_back)
         self.action_help.triggered.connect(self.help_docs)
 
-# Start of Functions
+    # Start of Functions
     def setupui(self):
         """
         Set up UI.
@@ -680,8 +663,6 @@ class MainWidget(QtWidgets.QMainWindow):
 
         self.textbrowser_datainfo.setSizePolicy(sizepolicy)
         self.textbrowser_processlog.setSizePolicy(sizepolicy)
-        self.textbrowser_processlog.setStyleSheet(
-            '* { background-color: rgb(255, 255, 255); }')
 
         self.grid_layout.addWidget(self.graphics_view, 0, 0, 4, 2)
         self.grid_layout.addWidget(self.textbrowser_datainfo, 1, 2, 1, 1)
@@ -772,7 +753,7 @@ class MainWidget(QtWidgets.QMainWindow):
 
     def keyPressEvent(self, event):
         """
-        Intercept keypress for custom key presses.
+        Intercept key press for custom key presses.
 
         Parameters
         ----------
@@ -823,8 +804,8 @@ class MainWidget(QtWidgets.QMainWindow):
         """Help Routine."""
         menu_default.HelpDocs(self, 'pygmi.main')
 
-    def item_insert(self, item_type, item_name, class_name, runimport=True,
-                    params=None):
+    def item_insert(self, item_type, item_name, class_name,
+                    projimport=False, **kwargs):
         """
         Item insert.
 
@@ -846,10 +827,7 @@ class MainWidget(QtWidgets.QMainWindow):
         item : DiagramItem
             Return a DiagramItem object
         """
-        if params is None:
-            class_name_active = class_name(self)
-        else:
-            class_name_active = class_name(self, params)
+        class_name_active = class_name(self, **kwargs)
 
         item = DiagramItem(item_type, self.scene.my_item_menu,
                            class_name_active, self)
@@ -859,34 +837,35 @@ class MainWidget(QtWidgets.QMainWindow):
 
         item.my_class_name = item_name.replace('\n', ' ')
 
-        if 'Import' in item_name:
+        if item_type == 'Io':
             item.is_import = True
 
-        if 'Import' in item_name and runimport is True:
+        if item_type == 'Io' and projimport is False:
             iflag = item.settings()
             if iflag is False:
                 return None
-            ifile = os.path.basename(item.my_class.ifile)
-            if len(ifile) > len(item_name):
-                ifile = ifile[:len(item_name)]+'\n'+ifile[len(item_name):]
-            if len(ifile) > 2*len(item_name):
-                ifile = ifile[:2*len(item_name)]+'...\n'
-            item_name += ':\n'+ifile
+            if item.my_class.ifile != '':
+                ifile = os.path.basename(item.my_class.ifile)
+                if len(ifile) > len(item_name):
+                    ifile = ifile[:len(item_name)]+'\n'+ifile[len(item_name):]
+                if len(ifile) > 2*len(item_name):
+                    ifile = ifile[:2*len(item_name)]+'...\n'
+                item_name += ':\n'+ifile
             item_color = QtGui.QColor(0, 255, 0, 127)
 
-# Do text first, since this determines size of polygon
+        # Do text first, since this determines size of polygon
         text_item = QtWidgets.QGraphicsTextItem()
         text_item.setPlainText(item_name)
         text_item.setFont(self.scene.my_font)
         text_item.setZValue(1000.0)
         text_item.setDefaultTextColor(self.scene.my_text_color)
 
-# Rectangle for text label
+        # Rectangle for text label
         rect_item = QtWidgets.QGraphicsRectItem(text_item.boundingRect())
         rect_item.setZValue(500.0)
         rect_item.setBrush(self.scene.my_item_color)
 
-# Actual polygon item
+        # Actual polygon item
         text_width = text_item.boundingRect().width()
         item.np_poly *= 1.5*text_width/item.np_poly[:, 0].ptp()
         item.np_poly[:, 0] += (text_item.boundingRect().left() -
@@ -901,10 +880,10 @@ class MainWidget(QtWidgets.QMainWindow):
         item.setPolygon(item.my_polygon)
         item.setBrush(item_color)
 
-# Menu Stuff
+        # Menu Stuff
         item.context_menu = self.context_menu
 
-# Add item to scene and merge
+        # Add item to scene and merge
         self.scene.addItem(item)
 
         xxyy = self.view.mapToScene(self.view.width()//2,
@@ -914,7 +893,7 @@ class MainWidget(QtWidgets.QMainWindow):
         text_item.setParentItem(item)
         rect_item.setParentItem(item)
 
-# Enable moving
+        # Enable moving
         self.scene.my_mode = 'MoveItem'
         return item
 
@@ -964,7 +943,7 @@ class MainWidget(QtWidgets.QMainWindow):
 
     def process_is_active(self, isactive=True):
         """
-        Change process log color when a process is active.
+        Change process log colour when a process is active.
 
         Parameters
         ----------
@@ -973,11 +952,11 @@ class MainWidget(QtWidgets.QMainWindow):
         """
         if isactive:
             self.textbrowser_processlog.setStyleSheet(
-                '* { background-color: rgba(255, 0, 0, 127); }')
+                'QTextBrowser { background-color: rgba(255, 0, 0, 127); }')
             self.pbar.setValue(0)
         else:
             self.textbrowser_processlog.setStyleSheet(
-                '* { background-color: rgb(255, 255, 255); }')
+                'QTextBrowser { background-color: rgb(255, 255, 255); }')
 
     def load(self):
         """
@@ -996,7 +975,7 @@ class MainWidget(QtWidgets.QMainWindow):
         if ifile == '':
             return False
 
-        with open(ifile, 'r') as fromdisk:
+        with open(ifile, 'r', encoding='utf-8') as fromdisk:
             ilist = json.load(fromdisk)
 
         ditems = {}
@@ -1010,12 +989,12 @@ class MainWidget(QtWidgets.QMainWindow):
             my_class = my_class.rsplit('.', 1)
 
             class_name = getattr(sys.modules[my_class[0]], my_class[1])
-            # class_name = class_name(self)
+
             item_name = item['my_class_name']
             item_type = item['diagram_type']
 
             citem = self.item_insert(item_type, item_name, class_name,
-                                     runimport=False)
+                                     projimport=True)
 
             citem.setX(item['x'])
             citem.setY(item['y'])
@@ -1080,7 +1059,7 @@ class MainWidget(QtWidgets.QMainWindow):
                 ilist[str(item)] = {'my_start_item': sname,
                                     'my_end_item': ename}
 
-        with open(ofile, 'w') as todisk:
+        with open(ofile, 'w', encoding='utf-8') as todisk:
             json.dump(ilist, todisk, indent=4)
         return True
 
@@ -1128,7 +1107,6 @@ class MainWidget(QtWidgets.QMainWindow):
             if not i.is_import:
                 i.my_class.indata = {}
                 i.my_class.outdata = {}
-                # i.setBrush(self.scene.my_item_color)
 
         # Update data
         while ilist:
@@ -1274,66 +1252,35 @@ class Startup(QtWidgets.QDialog):
         QtWidgets.QApplication.processEvents()
 
 
-class EmittingStream(QtCore.QObject):
-    """Class to intercept stdout for later use in a textbox."""
-
-    def __init__(self, textWritten):
-        self.textWritten = textWritten
-
-    def write(self, text):
-        """
-        Write text.
-
-        Parameters
-        ----------
-        text : TYPE
-            DESCRIPTION.
-
-        Returns
-        -------
-        None.
-
-        """
-        self.textWritten(str(text))
-
-    def flush(self):
-        """
-        Flush.
-
-        Returns
-        -------
-        None.
-
-        """
-
-    def fileno(self):
-        """
-        Fileno.
-
-        Returns
-        -------
-        int
-            Returns -1.
-
-        """
-        return -1
-
-
-def main():
+def main(nocgs=False):
     """Entry point for the PyGMI software."""
+    # Set environment variables.
+    if 'GDAL_DATA' not in os.environ:
+        import osgeo
+        gdalpath = os.path.join(osgeo.__path__[0], r'data\gdal')
+        if os.path.exists(gdalpath):
+            os.environ['GDAL_DATA'] = gdalpath
+        else:
+            print('GDAL_PATH not set.')
+
+    if 'PROJ_DATA' not in os.environ:
+        import pyproj
+        projpath = os.path.join(pyproj.__path__[0], r'proj_dir\share\proj')
+        if os.path.exists(projpath):
+            os.environ['PROJ_DATA'] = projpath
+        else:
+            print('PROJ_DATA not set.')
+
+    # Start program.
     app = QtWidgets.QApplication(sys.argv)
-    # The try statement is because this command is only available for qt>=5.10
-    try:
-        app.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton)
-    except AttributeError:
-        pass
+    app.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton)
 
     screen_resolution = app.desktop().screenGeometry()
     width, height = screen_resolution.width(), screen_resolution.height()
     width = int(width*0.75)
     height = int(height*0.75)
 
-    wid = MainWidget()
+    wid = MainWidget(nocgs=nocgs)
     wid.resize(width, height)
 
     wid.setWindowState(wid.windowState() & ~QtCore.Qt.WindowMinimized |
@@ -1343,13 +1290,11 @@ def main():
     wid.show()
     wid.activateWindow()
 
-    try:
-        __IPYTHON__
-    except NameError:
-        sys.exit(app.exec_())
-    else:
+    if hasattr(__builtins__, '__IPYTHON__'):
         app.exec_()
+    else:
+        sys.exit(app.exec_())
 
 
 if __name__ == "__main__":
-    main()
+    main(nocgs=False)
