@@ -64,35 +64,17 @@ Note, it will still be necessary for the end-user to compile the code.
 import os
 import functools
 from PyQt5 import QtWidgets, QtCore, QtGui
-# import pygmi.menu_default as menu_default
-# import mtpy.processing.birrp as MTbp
+
+from pygmi.misc import BasicModule
 
 
-class BIRRP(QtWidgets.QDialog):
-    """
-    Class to export config file for BIRRP .
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
-    """
+class BIRRP(BasicModule):
+    """Class to export config file for BIRRP."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
-        self.parent = parent
         self.indata = {'tmp': True}
-        self.outdata = {}
-        self.ifile = ''
 
         self.df_gps = None
 
@@ -169,7 +151,7 @@ class BIRRP(QtWidgets.QDialog):
         # helpdocs = menu_default.HelpButton('pygmi.grav.iodefs.importpointdata')
         pb_importbirrp = QtWidgets.QPushButton('Import BIRRP configuration '
                                                'file')
-        pb_runbirrp = QtWidgets.QPushButton('Save BIRRP configuration file')  #' and run BIRRP')
+        pb_runbirrp = QtWidgets.QPushButton('Save BIRRP configuration file')
 
         self.ilev.setDisabled(True)
         self.imode.setDisabled(True)
@@ -293,7 +275,7 @@ class BIRRP(QtWidgets.QDialog):
         if filename == '':
             return
 
-        with open(filename) as ifile:
+        with open(filename, encoding='utf-8') as ifile:
             data = ifile.read()
 
         data = data.replace('\n', ' ')
@@ -439,7 +421,7 @@ class BIRRP(QtWidgets.QDialog):
         jmode = self.jmode.currentIndex()
         nar = int(self.nar.text())
 
-        with open(filename, 'w+') as ofile:
+        with open(filename, 'w+', encoding='utf-8') as ofile:
             ofile.write('0\n')  # ilev == 0
             ofile.write(str(self.nout.currentIndex()+2)+'\n')
             ofile.write(str(self.ninp.currentIndex()+2)+'\n')
@@ -493,8 +475,8 @@ class BIRRP(QtWidgets.QDialog):
 
         Parameters
         ----------
-        widget : TYPE
-            DESCRIPTION.
+        widget : widget
+            widget whose text is set to filename..
 
         Returns
         -------
@@ -733,6 +715,11 @@ class BIRRP(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
