@@ -29,20 +29,22 @@ import sys
 import copy
 import glob
 import platform
+from contextlib import redirect_stdout
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 from matplotlib.patches import Rectangle
-import mtpy.modeling.occam1d as occam1d
+from mtpy.modeling import occam1d
 from mtpy.core.mt import MT
 from mtpy.core.z import Z, Tipper
 
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
+from pygmi.misc import BasicModule, ContextModule
 
 
-class Metadata(QtWidgets.QDialog):
+class Metadata(ContextModule):
     """
     Edit Metadata.
 
@@ -50,33 +52,18 @@ class Metadata(QtWidgets.QDialog):
 
     Attributes
     ----------
-    name : oldtxt
-        old text
     banddata : dictionary
         band data
     bandid : dictionary
         dictionary of strings containing band names.
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
-        self.indata = {}
-        self.outdata = {}
         self.banddata = {}
         self.dataid = {}
         self.oldtxt = ''
-        self.parent = parent
 
         self.combobox_bandid = QtWidgets.QComboBox()
         self.pb_rename_id = QtWidgets.QPushButton('Rename Station Name')
@@ -205,7 +192,6 @@ class Metadata(QtWidgets.QDialog):
             if self.dsb_utmy.text() != 'None':
                 odata.north = float(self.dsb_utmy.text())
             odata.elev = float(self.dsb_elev.text())
-#            odata.utm_zone = float(self.dsb_utmzone.text())
             odata.rotation_angle = float(self.dsb_rot.text())
         except ValueError:
             self.showprocesslog('Value error - abandoning changes')
@@ -374,20 +360,12 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class StaticShiftEDI(QtWidgets.QDialog):
+class StaticShiftEDI(BasicModule):
     """Static shift EDI data."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
         self.data = None
-        self.parent = parent
 
         self.setWindowTitle('Remove Static Shift')
         helpdocs = menu_default.HelpButton('pygmi.mt.static')
@@ -481,7 +459,6 @@ class StaticShiftEDI(QtWidgets.QDialog):
 
         if self.checkbox.isChecked():
             for i in self.data:
-                i = self.combobox1.currentText()
                 self.data[i].Z = self.data[i].remove_static_shift(ssx, ssy)
         else:
             i = self.combobox1.currentText()
@@ -524,6 +501,11 @@ class StaticShiftEDI(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -596,20 +578,12 @@ class StaticShiftEDI(QtWidgets.QDialog):
         return projdata
 
 
-class RotateEDI(QtWidgets.QDialog):
+class RotateEDI(BasicModule):
     """Rotate EDI data."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
         self.data = None
-        self.parent = parent
 
         self.setWindowTitle('Rotate EDI data')
 
@@ -738,6 +712,11 @@ class RotateEDI(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -820,6 +799,7 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
     def __init__(self, parent=None):
         fig = Figure()
+
         self.axes = fig.add_subplot(111)
         self.line = None
         self.ind = None
@@ -840,8 +820,7 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
                                        self.button_release_callback)
         self.figure.canvas.mpl_connect('motion_notify_event',
                                        self.motion_notify_callback)
-        self.figure.canvas.mpl_connect('resize_event',
-                                       self.resize)
+        self.figure.canvas.mpl_connect('resize_event', self.revent)
 
     def button_press_callback(self, event):
         """
@@ -849,8 +828,8 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -869,8 +848,8 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -889,8 +868,8 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -935,8 +914,8 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -956,16 +935,14 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
 
         return True
 
-    def resize(self, name, canvas=None):
+    def revent(self, width):
         """
         Resize event.
 
         Parameters
         ----------
-        name : TYPE
-            DESCRIPTION.
-        canvas : TYPE, optional
-            DESCRIPTION. The default is None.
+        event : event
+            unused.
 
         Returns
         -------
@@ -1005,7 +982,6 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
         ax1 = self.figure.add_subplot(211, label='Profile')
         ax1.grid(True, 'both')
 
-#        ax1 = self.figure.add_subplot(3, 1, 1)
         ax1.set_title(ival)
         self.axes = ax1
         x = 1/data1.Z.freq
@@ -1053,7 +1029,6 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
         ax2.grid(True, 'both')
 
         self.axes2 = ax2
-#        ax2.set_title('Normalised stacked graphs')
 
         ax2.errorbar(x, pha1, yerr=pha1_err, label=label3,
                      ls=' ', marker='.', mfc='b', mec='b', ecolor='b')
@@ -1082,20 +1057,12 @@ class MyMplCanvasPick(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class EditEDI(QtWidgets.QDialog):
+class EditEDI(BasicModule):
     """Edit EDI Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
         self.data = None
-        self.parent = parent
 
         self.setWindowTitle('Mask and Interpolate')
         helpdocs = menu_default.HelpButton('pygmi.mt.edit')
@@ -1225,6 +1192,11 @@ class EditEDI(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -1286,8 +1258,6 @@ class EditEDI(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
 
@@ -1308,8 +1278,8 @@ class MySlider(QtWidgets.QSlider):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -1327,8 +1297,8 @@ class MySlider(QtWidgets.QSlider):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            event variable.
 
         Returns
         -------
@@ -1365,7 +1335,7 @@ class MyMplCanvas2(FigureCanvasQTAgg):
         res : numpy array, optional
             Resistivities. The default is None.
         title : str or None, optional
-            DESCRIPTION. The default is None.
+            Title text. The default is None.
 
         Returns
         -------
@@ -1422,20 +1392,13 @@ class MyMplCanvas2(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class Occam1D(QtWidgets.QDialog):
+class Occam1D(BasicModule):
     """Occam 1D inversion."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
-        self.indata = {}
-        self.outdata = {}
         self.data = None
-        self.parent = parent
         self.cursoln = 0
 
         self.setWindowTitle('Occam 1D Inversion')
@@ -1495,8 +1458,6 @@ class Occam1D(QtWidgets.QDialog):
         label5 = QtWidgets.QLabel('Phase Errorbar (Data or %):')
         label6 = QtWidgets.QLabel('Resistivity Error Floor (%):')
         label7 = QtWidgets.QLabel('Phase Error Floor (degrees):')
-        label7a = QtWidgets.QLabel(r'Remove Resistivity/Phase values out of'
-                                   r' 1st/3rd Quadrant (True/False):')
         label8 = QtWidgets.QLabel('Height of air layer:')
         label9 = QtWidgets.QLabel('Bottom of model:')
         label10 = QtWidgets.QLabel('Depth of target to investigate:')
@@ -1628,55 +1589,54 @@ class Occam1D(QtWidgets.QDialog):
         else:
             os.makedirs(save_path)
 
-        d1 = occam1d.Data()
-        d1.write_data_file(edi_file=edi_file,
-                           mode=mode,
-                           save_path=save_path,
-                           res_err=parm['rerr'],
-                           phase_err=parm['perr'],
-                           res_errorfloor=parm['rerrflr'],
-                           phase_errorfloor=parm['perrflr'],
-                           remove_outofquadrant=parm['routq']
-                           )
+        with redirect_stdout(self.stdout_redirect):
+            d1 = occam1d.Data()
+            d1.write_data_file(edi_file=edi_file,
+                               mode=mode,
+                               save_path=save_path,
+                               res_err=parm['rerr'],
+                               phase_err=parm['perr'],
+                               res_errorfloor=parm['rerrflr'],
+                               phase_errorfloor=parm['perrflr'],
+                               remove_outofquadrant=parm['routq']
+                               )
 
-        m1 = occam1d.Model(target_depth=parm['tdepth'],
-                           n_layers=parm['nlayers'],
-                           bottom_layer=parm['blayer'],
-                           z1_layer=parm['z1layer'],
-                           air_layer_height=parm['alayer']
-                           )
-        m1.write_model_file(save_path=d1.save_path)
+            m1 = occam1d.Model(target_depth=parm['tdepth'],
+                               n_layers=parm['nlayers'],
+                               bottom_layer=parm['blayer'],
+                               z1_layer=parm['z1layer'],
+                               air_layer_height=parm['alayer']
+                               )
+            m1.write_model_file(save_path=d1.save_path)
 
-        s1 = occam1d.Startup(data_fn=d1.data_fn,
-                             model_fn=m1.model_fn,
-                             max_iter=parm['miter'],
-                             target_rms=parm['trms'])
+            s1 = occam1d.Startup(data_fn=d1.data_fn,
+                                 model_fn=m1.model_fn,
+                                 max_iter=parm['miter'],
+                                 target_rms=parm['trms'])
 
-        s1.write_startup_file()
+            s1.write_startup_file()
 
-        occam_path = os.path.dirname(__file__)[:-2]+r'\bin\occam1d'
-        if platform.system() == 'Windows':
-            occam_path += '.exe'
+            occam_path = os.path.dirname(__file__)[:-2]+r'\bin\occam1d'
+            if platform.system() == 'Windows':
+                occam_path += '.exe'
 
-        if not os.path.exists(occam_path):
-            text = ('No Occam1D executable found. Please place it in the bin '
-                    'directory. You may need to obtain the source code from '
-                    'https://marineemlab.ucsd.edu/Projects/Occam/1DCSEM/ '
-                    'and compile it. It should be called occam1d for '
-                    'non-windows platforms and occam1d.exe for windows.')
-            QtWidgets.QMessageBox.warning(self.parent, 'Error', text,
-                                          QtWidgets.QMessageBox.Ok)
-            return
+            if not os.path.exists(occam_path):
+                text = ('No Occam1D executable found. Please place it in the bin '
+                        'directory. You may need to obtain the source code from '
+                        'https://marineemlab.ucsd.edu/Projects/Occam/1DCSEM/ '
+                        'and compile it. It should be called occam1d for '
+                        'non-windows platforms and occam1d.exe for windows.')
+                QtWidgets.QMessageBox.warning(self.parent, 'Error', text,
+                                              QtWidgets.QMessageBox.Ok)
+                return
 
-        self.mmc.figure.clear()
-        self.mmc.figure.set_facecolor('r')
-        self.mmc.figure.suptitle('Busy, please wait...', fontsize=14, y=0.5)
-#        ax = self.mmc.figure.gca()
-#        ax.text(0.5, 0.5, 'Busy, please wait...')
-        self.mmc.figure.canvas.draw()
-        QtWidgets.QApplication.processEvents()
+            self.mmc.figure.clear()
+            self.mmc.figure.set_facecolor('r')
+            self.mmc.figure.suptitle('Busy, please wait...', fontsize=14, y=0.5)
+            self.mmc.figure.canvas.draw()
+            QtWidgets.QApplication.processEvents()
 
-        occam1d.Run(s1.startup_fn, occam_path, mode=mode)
+            occam1d.Run(s1.startup_fn, occam_path, mode=mode)
 
         self.mmc.figure.set_facecolor('w')
 
@@ -1763,6 +1723,11 @@ class Occam1D(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -1897,7 +1862,7 @@ def tonumber(test, alttext=None):
 
 def _testfn_occam():
     """Test routine."""
-    datadir = r'C:\Work\workdata\MT\\'
+    datadir = r'd:\workdata\MT\\'
     edi_file = datadir+r"synth02.edi"
 
     # Create an MT object
@@ -1913,11 +1878,10 @@ def _testfn_occam():
 
 def _testfn():
     """Test routine."""
-    datadir = r'C:\Work\workdata\MT\\'
+    datadir = r'd:\Work\workdata\MT\\'
     allfiles = glob.glob(datadir+'\\*.edi')
 
     for edi_file in allfiles:
-        # Create an MT object
         mt_obj = MT(edi_file)
 
     print('loading complete')
@@ -1929,4 +1893,4 @@ def _testfn():
 
 
 if __name__ == "__main__":
-    _testfn()
+    _testfn_occam()
