@@ -38,7 +38,7 @@ class LithModel():
     Attributes
     ----------
     mlut : dictionary
-        color table for lithologies
+        colour table for lithologies
     numx : int
         number of columns per layer in model
     numy : int):
@@ -51,16 +51,12 @@ class LithModel():
         dimension of cubes in the z direction
     lith_index : numpy array
         3D array of lithological indices.
-    curlayer : int
-        Current layer
     xrange : list
         minimum and maximum x coordinates
     yrange : list
         minimum and maximum y coordinates
     zrange : list
         minimum and maximum z coordinates
-    curprof : int
-        current profile (in x or y direction)
     griddata : dictionary
         dictionary of Data classes with raster data
     custprofx : dictionary
@@ -79,8 +75,6 @@ class LithModel():
         height of gravity sensor
     gregional : float
         gravity regional correction
-    name : str
-        name of the model
 
     """
 
@@ -108,6 +102,7 @@ class LithModel():
         self.gregional = 0.
         self.dataid = '3D Model'
         self.tmpfiles = None
+        self.name = None
         # Next line calls a function to update the variables above.
         self.update(50, 40, 5, 0., 0., 0., 100., 100., 100., 0.)
 
@@ -121,11 +116,6 @@ class LithModel():
         self.onumy = None
         self.onumz = None
 
-        # Obsolete
-#        self.curlayer = None
-#        self.is_ew = True
-#        self.curprof = None
-
     def lithold_to_lith(self, nodtm=False, pbar=None):
         """
         Transfers an old lithology to the new one, using update parameters.
@@ -236,12 +226,10 @@ class LithModel():
                 if ycrd2 == grows:
                     ycrd2 = grows-1
 
-#                if (ycrd2 >= 0 and xcrd2 >= 0 and ycrd2 < grows and
-#                        xcrd2 < gcols):
                 if (0 <= ycrd2 < grows and 0 <= xcrd2 < gcols):
                     alt = curgrid.data.data[ycrd2, xcrd2]
-                    if (curgrid.data.mask[ycrd2, xcrd2] or
-                            np.isnan(alt) or alt == curgrid.nullvalue):
+                    if (np.ma.getmaskarray(curgrid.data)[ycrd2, xcrd2] or
+                            np.isnan(alt) or alt == curgrid.nodata):
                         alt = curgrid.data.mean()
                     k_2 = int((utlz - alt) / self.d_z)
                     self.lith_index[i, j, :k_2] = -1
@@ -261,12 +249,15 @@ class LithModel():
             PyGMI raster dataset.
 
         """
+        crs = None
+        for i in self.griddata:
+            if self.griddata[i].crs is not None:
+                crs = self.griddata[i].crs
+
         grid = Data()
         grid.data = data
-        grid.xdim = self.dxy
-        grid.ydim = self.dxy
-        grid.extent = [self.xrange[0], self.xrange[1],
-                       self.yrange[0], self.yrange[1]]
+        grid.set_transform(self.dxy, self.xrange[0], self.dxy, self.yrange[1])
+        grid.crs = crs
 
         return grid
 
