@@ -38,19 +38,15 @@ Blakely, R.J., 1996. Potential Theory in Gravity and Magnetic Applications,
 
 import copy
 import tempfile
-# from math import sqrt
 from PyQt5 import QtWidgets, QtCore
 
 import numpy as np
-# from scipy.linalg import norm
-from osgeo import gdal
 from numba import jit, prange
-from matplotlib import cm
+from matplotlib import colormaps
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 
-from pygmi.raster.dataprep import gdal_to_dat
-from pygmi.raster.dataprep import data_to_gdal_mem
+from pygmi.raster.dataprep import data_reproject
 from pygmi.pfmod.datatypes import LithModel
 from pygmi.misc import PTime
 from pygmi.misc import frm
@@ -78,7 +74,6 @@ class GravMag():
         else:
             self.pbars = None
         self.oldlithindex = None
-#        self.mfname = self.parent.modelfilename
         self.tmpfiles = {}
 
         self.actionregionaltest = QtWidgets.QAction('Regional\nTest')
@@ -119,8 +114,6 @@ class GravMag():
         self.actioncalculate2.triggered.connect(self.calc_field_mag)
         self.actioncalculate3.triggered.connect(self.calc_field_grav_changes)
         self.actioncalculate4.triggered.connect(self.calc_field_mag_changes)
-        self.actioncalculate3.setEnabled(False)
-        self.actioncalculate4.setEnabled(False)
 
     def calc_field_mag(self):
         """
@@ -208,7 +201,7 @@ class GravMag():
         showreports : bool, optional
             Flag for showing reports. The default is False.
         magcalc : bool, optional
-            Flac for choosing the magnetic calculation. The default is False.
+            Flag for choosing the magnetic calculation. The default is False.
 
         Returns
         -------
@@ -226,11 +219,13 @@ class GravMag():
         Calculate magnetic and gravity regional.
 
         Calculates a gravity and magnetic regional value based on a single
-        solid lithology model. This gets used in tab_param. The principle is
-        that the maximum value for a solid model with fixed extents and depth,
-        using the most COMMON lithology, would be the MAXIMUM AVERAGE value for
-        any model which we would do. Therefore the regional is simply:
-        REGIONAL = OBS GRAVITY MEAN - CALC GRAVITY MAX
+        solid lithology model. The principle is that the maximum value for a
+        solid model with fixed extents and depth, using the most COMMON
+        lithology, would be the MAXIMUM AVERAGE value for any model which we
+        would do. Therefore the regional is simply:
+
+            REGIONAL = OBS GRAVITY MEAN - CALC GRAVITY MAX
+
         This routine calculates the last term.
 
         Returns
@@ -323,9 +318,7 @@ class GravMag():
         mint = (magtmp.std()*4)/10.
         if magtmp.ptp() > 0:
             csrange = np.arange(mmin, mmax, mint)
-#            cns = plt.contour(magtmp, levels=csrange, colors='b', extent=etmp)
             plt.contour(magtmp, levels=csrange, colors='b', extent=etmp)
-#            plt.clabel(cns, inline=1, fontsize=10)
         cbar = plt.colorbar(ims, orientation='horizontal', format=frm)
         cbar.set_label('nT')
 
@@ -340,8 +333,6 @@ class GravMag():
         if grvtmp.ptp() > 0:
             csrange = np.arange(mmin, mmax, mint)
             plt.contour(grvtmp, levels=csrange, colors='y', extent=etmp)
-#            cns = plt.contour(grvtmp, levels=csrange, colors='y', extent=etmp)
-#            plt.clabel(cns, inline=1, fontsize=10)
         cbar = plt.colorbar(ims, orientation='horizontal', format=frm)
         cbar.set_label('mGal')
         plt.tight_layout()
@@ -745,7 +736,6 @@ class GeoData():
         if demag is True:
             m3 = calc_demag(m3, self.susc,
                             np.abs(self.x12[1]-self.x12[0]),
-                            np.abs(self.y12[1]-self.y12[0]),
                             np.abs(self.z12[1]-self.z12[0]))
 
         mt = np.sqrt(m3 @ m3)
@@ -782,7 +772,7 @@ class GeoData():
         self.mlayers = self.mlayers[:-1]-self.mlayers[1:]
 
 
-def calc_demag(mvec, k, dx, dy, dz):
+def calc_demag(mvec, k, dxy, dz):
     """
     Calculate demagnetisation correction.
 
@@ -790,14 +780,12 @@ def calc_demag(mvec, k, dx, dy, dz):
     ----------
     mvec : numpy array
         Body Magnetisation.
-    k : TYPE
-        DESCRIPTION.
-    dx : TYPE
-        DESCRIPTION.
-    dy : TYPE
-        DESCRIPTION.
-    dz : TYPE
-        DESCRIPTION.
+    k : float
+        susceptibility.
+    dxy : float
+        cell width.
+    dz : float
+        cell height.
 
     Returns
     -------
@@ -806,9 +794,10 @@ def calc_demag(mvec, k, dx, dy, dz):
 
     """
     # Y = dy/2
-    t = dx
+    t = dxy  # this was dx
     d = dz
 
+    # These lines below assume dx is not equal to dy.
     # Ndz = 4*np.pi*(2*Y*t/(d*t+2*Y*t+2*Y*d))
     # Ndy = 4*np.pi*(d*t/(d*t+2*Y*t+2*Y*d))
     # Ndx = 4*np.pi*(2*Y*d/(d*t+2*Y*t+2*Y*d))
@@ -890,34 +879,13 @@ def gridmatch(lmod, ctxt, rtxt):
         Numpy array of data.
 
     """
-    rgrv = lmod.griddata[rtxt]
-    cgrv = lmod.griddata[ctxt]
+    data = lmod.griddata[rtxt]
+    data2 = lmod.griddata[ctxt]
 
-    data = rgrv
-    data2 = cgrv
-    orig_wkt = data.wkt
-    orig_wkt2 = data2.wkt
-
-    doffset = 0.0
-    if data.data.min() <= 0:
-        doffset = data.data.min()-1.
-        data.data = data.data - doffset
-
-    rows, cols = data.data.shape
     rows2, cols2 = data2.data.shape
 
-    gtr0 = data.get_gtr()
-    gtr = data2.get_gtr()
-    src = data_to_gdal_mem(data, gtr0, orig_wkt, cols, rows)
-    dest = data_to_gdal_mem(data, gtr, orig_wkt2, cols2, rows2, True)
-
-    gdal.ReprojectImage(src, dest, orig_wkt, orig_wkt2, gdal.GRA_Bilinear)
-
-    dat = gdal_to_dat(dest, data.dataid)
-
-    if doffset != 0.0:
-        dat.data = dat.data + doffset
-        data.data = data.data + doffset
+    dat = data_reproject(data, data.crs, data2.crs, data2.transform,
+                         rows2, cols2)
 
     return dat.data
 
@@ -970,36 +938,38 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
 
     tmpfiles = {}
 
-# model index
+    # model index
     modind = lmod.lith_index.copy()
     if magcalc:
         modindcheck = lmod.lith_index_mag_old.copy()
     else:
         modindcheck = lmod.lith_index_grv_old.copy()
 
+    if modind.shape != modindcheck.shape:
+        modindcheck = np.zeros_like(modind) - 1
+
     tmp = (modind == modindcheck)
-# If modind and modindcheck have different shapes, then tmp == False. The next
-# line checks for that.
-    if not isinstance(tmp, bool):
-        modind[tmp] = -1
-        modindcheck[tmp] = -1
+    modind[tmp] = -1
+    modindcheck[tmp] = -1
+
+    # If modind and modindcheck have different shapes, then tmp == False.
+    # The next line checks for that.
+    # if not isinstance(tmp, bool):
+    #     modind[tmp] = -1
+    #     modindcheck[tmp] = -1
 
     modindmax = modind.max()
     modindcheckmax = modindcheck.max()
-#    if np.unique(modind).size == 1:
-#        showtext('No changes to model!')
-#        return None
 
     if False not in tmp:
         showtext('No changes to model!')
         return None
 
-# get height corrections
+    # get height corrections
     tmp = np.copy(lmod.lith_index)
     tmp[tmp > -1] = 0
     hcor = np.abs(tmp.sum(2))
 
-#    if np.unique(modindcheck).size == 1 and np.unique(modindcheck)[0] == -1:
     for mlist in lmod.lith_list.items():
         mijk = mlist[1].lith_index
         if mijk not in modind and mijk not in modindcheck:
@@ -1023,7 +993,7 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
 
     QtCore.QCoreApplication.processEvents()
 
-# Get mlayers and glayers with correct rho and netmagn
+    # Get mlayers and glayers with correct rho and netmagn
 
     if pbars is not None:
         pbars.resetsub(maximum=(len(lmod.lith_list)-1))
@@ -1035,7 +1005,11 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
     hcorflat = numz-hcor.flatten()
     aaa = np.reshape(np.mgrid[0:numx, 0:numy], [2, numx*numy])
 
-    for mlist in piter(lmod.lith_list.items()):
+    # These two lines are to eliminate background and improve time estimate.
+    lith_list = lmod.lith_list.copy()
+    del lith_list['Background']
+
+    for mlist in piter(lith_list.items()):
         if mlist[0] == 'Background':
             continue
         mijk = mlist[1].lith_index
@@ -1087,7 +1061,6 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
     mgvalin = mgvalin[::-1]
     mgvalin = np.ma.array(mgvalin)
 
-#    if np.unique(modindcheck).size > 1:
     if modindcheckmax > -1:
         if magcalc:
             mgvalin += lmod.griddata['Calculated Magnetics'].data
@@ -1125,7 +1098,6 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
         lmod.griddata['Gravity Residual'].dataid = 'Gravity Residual'
 
     if parent is not None:
-        # tmp = [i for i in set(lmod.griddata.values())]
         tmp = list(set(lmod.griddata.values()))
         parent.outdata['Raster'] = tmp
     showtext('Calculation Finished')
@@ -1146,7 +1118,7 @@ def calc_field(lmod, pbars=None, showtext=None, parent=None,
     return lmod.griddata
 
 
-@jit(nopython=True, parallel=True)
+@jit(nopython=True, parallel=False)
 def sum_fields(k, mgval, numx, numy, modind, aaa0, aaa1, mlayers, hcorflat,
                mijk):
     """
@@ -1279,9 +1251,10 @@ def quick_model(numx=50, numy=40, numz=5, dxy=100., d_z=100.,
         clrtmp = np.array([0])
     else:
         clrtmp = np.arange(len(inputliths))/(len(inputliths)-1)
-    clrtmp = cm.get_cmap('jet')(clrtmp)[:, :-1]
+    clrtmp = colormaps['jet'](clrtmp)[:, :-1]
     clrtmp *= 255
     clrtmp = clrtmp.astype(int)
+    clrtmp = clrtmp.tolist()
 
     for i in inputliths:
         j += 1
@@ -1595,32 +1568,31 @@ def dat_extent(dat, axes):
 
 def _testfn():
     """Test routine."""
-#    from pygmi.pfmod.iodefs import ImportMod3D
+    # from pygmi.pfmod.iodefs import ImportMod3D
 
-# Import model file
-#    filename = r'C:\Work\Programming\pygmi\data\Magmodel_Area3_Delph.npz'
-#    imod = ImportMod3D(None)
-#    imod.ifile = filename
-#    imod.lmod.griddata.clear()
-#    imod.lmod.lith_list.clear()
-#    indict = np.load(filename)
-#    imod.dict2lmod(indict)
-#    calc_field(imod.lmod, magcalc=True)
+    # Import model file
+    # filename = r'd:\Work\Programming\pygmi\data\Magmodel_Area3_Delph.npz'
+    # imod = ImportMod3D(None)
+    # imod.ifile = filename
+    # imod.lmod.griddata.clear()
+    # imod.lmod.lith_list.clear()
+    # indict = np.load(filename)
+    # imod.dict2lmod(indict)
+    # calc_field(imod.lmod, magcalc=True)
 
-
-# quick model
+    # quick model
     lmod = quick_model(numx=300, numy=300, numz=30)
     lmod.lith_index[:, :, 0] = 1
-#    lmod.lith_index[:, :, 10] = 1
+    # lmod.lith_index[:, :, 10] = 1
     lmod.mht = 100
     calc_field(lmod, magcalc=True)
 
-# Calculate the field
+    # Calculate the field
 
-    magval = lmod.griddata['Calculated Magnetics'].data
+    # magval = lmod.griddata['Calculated Magnetics'].data
 
-#    plt.imshow(magval, cmap=cm.get_cmap('jet'))
-#    plt.show()
+    # plt.imshow(magval, cmap=colormaps['jet'])
+    # plt.show()
 
 
 if __name__ == "__main__":
