@@ -32,12 +32,15 @@ from PyQt5 import QtWidgets, QtCore
 import numpy as np
 from osgeo import osr, ogr
 import matplotlib.pyplot as plt
+import pandas as pd
+from rasterio.crs import CRS
 
 from pygmi.pfmod.datatypes import LithModel
-import pygmi.pfmod.grvmag3d as grvmag3d
-import pygmi.pfmod.cubes as mvis3d
-import pygmi.menu_default as menu_default
+from pygmi.pfmod import grvmag3d
+from pygmi.pfmod import mvis3d
+from pygmi import menu_default
 import pygmi.raster.dataprep as dp
+from pygmi.misc import BasicModule, ContextModule
 # This is necessary for loading npz files, since I moved the location of
 # datatypes.
 from pygmi.pfmod import datatypes
@@ -45,27 +48,24 @@ from pygmi.pfmod import datatypes
 sys.modules['datatypes'] = datatypes
 
 
-class ImportMod3D():
+class ImportMod3D(BasicModule):
     """Import Data."""
 
     def __init__(self, parent=None):
-        self.parent = parent
+        super().__init__(parent)
+
         self.lmod = LithModel()
-
-        self.ifile = ''
         self.filt = ''
-        self.indata = {}
-        self.outdata = {}
-
-        if parent is not None:
-            self.piter = parent.pbar.iter
-        else:
-            self.piter = iter
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -75,8 +75,8 @@ class ImportMod3D():
         if not nodialog:
             ext = ('npz (*.npz);;'
                    'Leapfrog Block Model (*.csv);;'
-                   'x,y,z,label (*.csv);;'
-                   'x,y,z,label (*.txt)')
+                   'x, y, z, label (*.csv);;'
+                   'x, y, z, label (*.txt)')
 
             self.ifile, self.filt = QtWidgets.QFileDialog.getOpenFileName(
                 self.parent, 'Open File', '.', ext)
@@ -84,15 +84,14 @@ class ImportMod3D():
             if self.ifile == '':
                 return False
         os.chdir(os.path.dirname(self.ifile))
-        # self.parent.modelfilename = self.ifile.rpartition('.')[0]
 
-# Reset Variables
+        # Reset Variables
         self.lmod.griddata.clear()
         self.lmod.lith_list.clear()
 
         if self.filt == 'Leapfrog Block Model (*.csv)':
             self.import_leapfrog_csv(self.ifile)
-        elif self.filt in ('x,y,z,label (*.csv)', 'x,y,z,label (*.txt)'):
+        elif self.filt in ('x, y, z, label (*.csv)', 'x, y, z, label (*.txt)'):
             self.import_ascii_xyz_model(self.ifile)
         else:
             indict = np.load(self.ifile, allow_pickle=True)
@@ -165,7 +164,7 @@ class ImportMod3D():
         None.
 
         """
-        with open(filename) as fno:
+        with open(filename, encoding='utf-8') as fno:
             tmp = fno.readlines()
 
         while tmp[0][0] == '#':
@@ -272,39 +271,40 @@ class ImportMod3D():
         None.
 
         """
-        if filename.find('.csv') > -1:
-            tmp = np.genfromtxt(filename, delimiter=',', dtype=np.str)
-        else:
-            tmp = np.genfromtxt(filename, dtype=np.str)
+        names = ['x', 'y', 'z', 'label']
+        try:
+            if filename.find('.csv') > -1:
+                df1 = pd.read_csv(filename, sep=',', names=names)
+            else:
+                df1 = pd.read_csv(filename, sep=' ', names=names)
+        except:
+            self.showprocesslog('Unable to import file')
+            return
 
-        x = tmp[:, 0].astype(np.float)
-        y = tmp[:, 1].astype(np.float)
-        z = tmp[:, 2].astype(np.float)
-        label = tmp[:, 3]
-        labelu = np.unique(label)
+        x = df1.x.to_numpy(float)
+        y = df1.y.to_numpy(float)
+        z = df1.z.to_numpy(float)
+        label = df1.label.to_numpy(str)
+
+        x_u = df1.x.unique()
+        y_u = df1.y.unique()
+        z_u = df1.z.unique()
+        labelu = df1.label.unique()
+
+        x_u.sort()
+        y_u.sort()
+        z_u.sort()
 
-        idx = np.unique(x, return_index=True)[1]
-        x_u = x[np.sort(idx)]
         dx_u = np.diff(x_u)
-        idx = np.unique(y, return_index=True)[1]
-        y_u = y[np.sort(idx)]
         dy_u = np.diff(y_u)
-        idx = np.unique(z, return_index=True)[1]
-        z_u = z[np.sort(idx)]
         dz_u = np.diff(z_u)
 
-        if dx_u[0] < 0:
-            dx_u *= -1
-        if dy_u[0] < 0:
-            dy_u *= -1
-        if dz_u[0] < 0:
-            dz_u *= -1
-
-        xcell = np.max(dx_u)
-        ycell = np.max(dy_u)
-        zcell = np.max(dz_u)
+        xcell = np.min(dx_u)
+        ycell = np.min(dy_u)
+        zcell = np.min(dz_u)
 
         lmod = self.lmod
+        lmod.lith_list['Background'] = grvmag3d.GeoData(self.parent)
 
         lmod.dxy = min(xcell, ycell)
         lmod.d_z = zcell
@@ -315,11 +315,7 @@ class ImportMod3D():
         lmod.numy = int(np.ptp(lmod.yrange)/lmod.dxy+1)
         lmod.numz = int(np.ptp(lmod.zrange)/lmod.d_z+1)
 
-
-# Section to load lithologies.
-        if 'Generic 1' in lmod.lith_list:
-            lmod.lith_list.pop('Generic 1')
-
+        # Section to load lithologies.
         lindx = 0
         for itxt in labelu:
             lindx += 1
@@ -341,7 +337,7 @@ class ImportMod3D():
 
         for i, xi in enumerate(x):
             col = int((xi-lmod.xrange[0])/lmod.dxy)
-            row = int((lmod.yrange[1]-y[i])/lmod.dxy)
+            row = int((y[i]-lmod.yrange[0])/lmod.dxy)
             layer = int((lmod.zrange[1]-z[i])/lmod.d_z)
             lmod.lith_index[col, row, layer] = \
                 lmod.lith_list[label[i]].lith_index
@@ -375,6 +371,13 @@ class ImportMod3D():
         lmod.dxy = indict[pre+'dxy']
         lmod.d_z = indict[pre+'d_z']
         lmod.lith_index = indict[pre+'lith_index']
+
+        if pre+'lith_index_grv_old' in indict:
+            lmod.lith_index_grv_old = indict[pre+'lith_index_grv_old']
+
+        if pre+'lith_index_mag_old' in indict:
+            lmod.lith_index_mag_old = indict[pre+'lith_index_mag_old']
+
         lmod.xrange = np.array(indict[pre+'xrange']).tolist()
         lmod.yrange = np.array(indict[pre+'yrange']).tolist()
         lmod.zrange = np.array(indict[pre+'zrange']).tolist()
@@ -395,7 +398,6 @@ class ImportMod3D():
                 lmod.custprofy[i] += lmod.custprofy[i]
 
         lmod.mlut = indict[pre+'mlut'].item()
-        lmod.init_calc_grids()
 
         lmod.griddata = indict[pre+'griddata'].item()
 
@@ -408,8 +410,25 @@ class ImportMod3D():
             for i in lmod.profpics:
                 lmod.profpics[i].data = np.ma.array(lmod.profpics[i].data)
 
-        # This gets rid of a legacy variable name
+        # This gets rid of a legacy variable names and updates to new ones
         for i in lmod.griddata:
+            if not hasattr(lmod.griddata[i], 'units'):
+                lmod.griddata[i].units = ''
+            if not hasattr(lmod.griddata[i], 'isrgb'):
+                lmod.griddata[i].isrgb = False
+            if not hasattr(lmod.griddata[i], 'metadata'):
+                lmod.griddata[i].metadata = {'Cluster': {}, 'Raster': {}}
+            if not hasattr(lmod.griddata[i], 'filename'):
+                lmod.griddata[i].filename = ''
+
+            if not hasattr(lmod.griddata[i], 'nodata'):
+                lmod.griddata[i].nodata = lmod.griddata[i].nullvalue
+            if not hasattr(lmod.griddata[i], 'crs'):
+                wkt = lmod.griddata[i].wkt
+                if wkt == '' or wkt is None:
+                    lmod.griddata[i].crs = None
+                else:
+                    lmod.griddata[i].crs = CRS.from_wkt(wkt)
             if not hasattr(lmod.griddata[i], 'dataid'):
                 lmod.griddata[i].dataid = ''
             if hasattr(lmod.griddata[i], 'bandid'):
@@ -417,33 +436,35 @@ class ImportMod3D():
                     lmod.griddata[i].dataid = lmod.griddata[i].bandid
                 del lmod.griddata[i].bandid
             if not hasattr(lmod.griddata[i], 'extent'):
-                if lmod.griddata[i].data.ndim == 2:
-                    rows, cols = lmod.griddata[i].data.shape
-                elif lmod.griddata[i].data.ndim == 3:
-                    rows, cols, _ = lmod.griddata[i].data.shape
-
                 xmin = lmod.griddata[i].tlx
                 ymax = lmod.griddata[i].tly
-                ymin = ymax - rows*lmod.griddata[i].ydim
-                xmax = xmin + cols*lmod.griddata[i].xdim
 
-                lmod.griddata[i].extent = [xmin, xmax, ymin, ymax]
+                ydim = lmod.griddata[i].ydim
+                xdim = lmod.griddata[i].xdim
+
+                lmod.griddata[i].set_transform(xdim, xmin, ydim, ymax)
                 del lmod.griddata[i].tlx
                 del lmod.griddata[i].tly
+            elif not hasattr(lmod.griddata[i], 'transform'):
+                ydim = lmod.griddata[i].ydim
+                xdim = lmod.griddata[i].xdim
+                xmin, _, _, ymax = lmod.griddata[i].extent
+                lmod.griddata[i].set_transform(xdim, xmin, ydim, ymax)
+
+        crsfin = CRS.from_string('LOCAL_CS["Arbitrary",UNIT["metre",1,'
+                                 'AUTHORITY["EPSG","9001"]],'
+                                 'AXIS["Easting",EAST],'
+                                 'AXIS["Northing",NORTH]]')
 
-        wktfin = None
         for i in lmod.griddata:
-            wkt = lmod.griddata[i].wkt
-            if wkt != '' and wkt is not None:
-                wktfin = wkt
+            if lmod.griddata[i].crs is not None:
+                crsfin = lmod.griddata[i].crs
 
-        if wktfin is not None:
-            for i in lmod.griddata:
-                wkt = lmod.griddata[i].wkt
-                if wkt == '' or wkt is None:
-                    lmod.griddata[i].wkt = wktfin
+        for i in lmod.griddata:
+            if lmod.griddata[i].crs is None:
+                lmod.griddata[i].crs = crsfin
 
-# Section to load lithologies.
+        # Section to load lithologies.
         lmod.lith_list['Background'] = grvmag3d.GeoData(self.parent)
 
         for itxt in lithkeys:
@@ -479,19 +500,14 @@ class ImportMod3D():
             lmod.lith_list[itxt].set_xyz12()
 
 
-class ExportMod3D():
+class ExportMod3D(ContextModule):
     """Export Data."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
+
+        self.ofile = ''
         self.lmod = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def run(self):
         """
@@ -507,18 +523,17 @@ class ExportMod3D():
             return
 
         for self.lmod in self.indata['Model3D']:
-            filename, _ = QtWidgets.QFileDialog.getSaveFileName(
+            self.ofile, _ = QtWidgets.QFileDialog.getSaveFileName(
                 self.parent, 'Save File', '.',
                 'npz (*.npz);;shapefile (*.shp);;kmz (*.kmz);;csv (*.csv)')
 
-            if filename == '':
+            if self.ofile == '':
                 return
 
-            os.chdir(os.path.dirname(filename))
-            self.ifile = str(filename)
-            ext = filename[-3:]
+            os.chdir(os.path.dirname(self.ofile))
+            ext = self.ofile[-3:]
 
-            self.showprocesslog('Saving '+self.ifile+'...')
+            self.showprocesslog('Saving '+self.ofile+'...')
 
         # Pop up save dialog box
             if ext == 'npz':
@@ -539,16 +554,13 @@ class ExportMod3D():
         None.
 
         """
-# Open file
-        filename = self.ifile
-
-# Construct output dictionary
+        # Construct output dictionary
         outdict = {}
         outdict = self.lmod2dict(outdict)
 
-# Save data
+        # Save data
         try:
-            np.savez_compressed(filename, **outdict)
+            np.savez_compressed(self.ofile, **outdict)
             self.showprocesslog('Model save complete!')
         except:
             self.showprocesslog('ERROR! Model save failed!')
@@ -588,7 +600,10 @@ class ExportMod3D():
         outdict[pre+'custprofx'] = self.lmod.custprofx
         outdict[pre+'custprofy'] = self.lmod.custprofy
 
-# Section to save lithologies.
+        outdict[pre+'lith_index_grv_old'] = self.lmod.lith_index_grv_old
+        outdict[pre+'lith_index_mag_old'] = self.lmod.lith_index_mag_old
+
+        # Section to save lithologies.
         outdict[pre+'lithkeys'] = list(self.lmod.lith_list.keys())
 
         for i in self.lmod.lith_list.items():
@@ -693,7 +708,7 @@ class ExportMod3D():
         zrng = np.array(self.lmod.zrange, dtype=float)
 
         if 'Raster' in self.indata:
-            wkt = self.indata['Raster'][0].wkt
+            wkt = self.indata['Raster'][0].crs.wkt
         else:
             wkt = ''
         prjkmz = Exportkmz(wkt)
@@ -726,12 +741,12 @@ class ExportMod3D():
         res = prj.TransformPoint(xrng[1], yrng[1])
         loneast, latnorth = res[0], res[1]
 
-# Get Save Name
+        # Get Save Name
         filename = self.ifile
 
         self.showprocesslog('kmz export starting...')
 
-# Move to 3d model tab to update the model stuff
+        # Move to 3d model tab to update the model stuff
         self.showprocesslog('updating 3d model...')
 
         mvis_3d.spacing = [self.lmod.dxy, self.lmod.dxy, self.lmod.d_z]
@@ -755,7 +770,7 @@ class ExportMod3D():
         lato = str(latsouth)
         lono = str(lonwest)
 
-# update colors
+        # update colours
         self.lmod.update_lith_list_reverse()
 
         dockml = (
@@ -975,56 +990,55 @@ class ExportMod3D():
                 '  </scene>\r\n'
                 '</COLLADA>')
 
-        zfile = zipfile.ZipFile(filename, 'w')
-        for i, _ in enumerate(modeldae):
-            zfile.writestr('models\\mod3d'+str(i)+'.dae', modeldae[i])
+        with zipfile.ZipFile(filename, 'w') as zfile:
+            for i, modeldaei in enumerate(modeldae):
+                zfile.writestr('models\\mod3d'+str(i)+'.dae', modeldaei)
 
-        for i in self.lmod.griddata:
-            x_1, x_2, y_1, y_2 = self.lmod.griddata[i].extent
+            for i in self.lmod.griddata:
+                x_1, x_2, y_1, y_2 = self.lmod.griddata[i].extent
 
-            res = prj.TransformPoint(x_1, y_1)
-            lonwest, latsouth = res[0], res[1]
-            res = prj.TransformPoint(x_2, y_2)
-            loneast, latnorth = res[0], res[1]
+                res = prj.TransformPoint(x_1, y_1)
+                lonwest, latsouth = res[0], res[1]
+                res = prj.TransformPoint(x_2, y_2)
+                loneast, latnorth = res[0], res[1]
+
+                dockml += (
+                    '    <GroundOverlay>\r\n'
+                    '        <name>' + i + '</name>\r\n'
+                    '        <description></description>\r\n'
+                    '        <Icon>\r\n'
+                    '            <href>models/' + i + '.png</href>\r\n'
+                    '        </Icon>\r\n'
+                    '        <LatLonBox>\r\n'
+                    '            <north>' + str(latnorth) + '</north>\r\n'
+                    '            <south>' + str(latsouth) + '</south>\r\n'
+                    '            <east>' + str(loneast) + '</east>\r\n'
+                    '            <west>' + str(lonwest) + '</west>\r\n'
+                    '            <rotation>0.0</rotation>\r\n'
+                    '        </LatLonBox>\r\n'
+                    '    </GroundOverlay>\r\n')
+
+                fig = plt.figure('tmp930', frameon=False)
+                ax1 = plt.Axes(fig, [0., 0., 1., 1.])
+                ax1.set_axis_off()
+                fig.add_axes(ax1)
+
+                plt.imshow(self.lmod.griddata[i].data,
+                           extent=(lonwest, loneast, latsouth, latnorth),
+                           aspect='auto',
+                           interpolation='nearest')
+                plt.savefig('tmp930.png')
+
+                zfile.write('tmp930.png', 'models\\'+i+'.png')
+                os.remove('tmp930.png')
 
             dockml += (
-                '    <GroundOverlay>\r\n'
-                '        <name>' + i + '</name>\r\n'
-                '        <description></description>\r\n'
-                '        <Icon>\r\n'
-                '            <href>models/' + i + '.png</href>\r\n'
-                '        </Icon>\r\n'
-                '        <LatLonBox>\r\n'
-                '            <north>' + str(latnorth) + '</north>\r\n'
-                '            <south>' + str(latsouth) + '</south>\r\n'
-                '            <east>' + str(loneast) + '</east>\r\n'
-                '            <west>' + str(lonwest) + '</west>\r\n'
-                '            <rotation>0.0</rotation>\r\n'
-                '        </LatLonBox>\r\n'
-                '    </GroundOverlay>\r\n')
+                '  </Folder>\r\n'
+                '  \r\n'
+                '  </kml>')
 
-            fig = plt.figure('tmp930', frameon=False)
-            ax1 = plt.Axes(fig, [0., 0., 1., 1.])
-            ax1.set_axis_off()
-            fig.add_axes(ax1)
+            zfile.writestr('doc.kml', dockml)
 
-            plt.imshow(self.lmod.griddata[i].data,
-                       extent=(lonwest, loneast, latsouth, latnorth),
-                       aspect='auto',
-                       interpolation='nearest')
-            plt.savefig('tmp930.png')
-
-            zfile.write('tmp930.png', 'models\\'+i+'.png')
-            os.remove('tmp930.png')
-
-        dockml += (
-            '  </Folder>\r\n'
-            '  \r\n'
-            '  </kml>')
-
-        zfile.writestr('doc.kml', dockml)
-
-        zfile.close()
         self.showprocesslog('kmz export complete!')
 
     def mod3dtoshp(self):
@@ -1047,7 +1061,7 @@ class ExportMod3D():
         zrng = np.array(self.lmod.zrange, dtype=float)
 
         if 'Raster' in self.indata:
-            wkt = self.indata['Raster'][0].wkt
+            wkt = self.indata['Raster'][0].crs.wkt
         else:
             wkt = ''
         prjkmz = Exportkmz(wkt)
@@ -1060,7 +1074,7 @@ class ExportMod3D():
 
         self.showprocesslog('shapefile export starting...')
 
-# Move to 3d model tab to update the model stuff
+        # Move to 3d model tab to update the model stuff
         if smooth is True:
             self.showprocesslog('updating and smoothing 3d model...')
         else:
@@ -1081,7 +1095,7 @@ class ExportMod3D():
 
         driver = ogr.GetDriverByName('ESRI Shapefile')
 
-# update colors
+        # update colours
         self.lmod.update_lith_list_reverse()
 
         mvis_3d.update_for_kmz()
@@ -1122,7 +1136,7 @@ class ExportMod3D():
                 ring1.AddPoint(tmp[2, 0], tmp[2, 1], tmp[2, 2])
                 ring1.AddPoint(tmp[0, 0], tmp[0, 1], tmp[0, 2])
 
-            # Create polygon #1
+                # Create polygon #1
                 poly1 = ogr.Geometry(ogr.wkbPolygon25D)
                 poly1.AddGeometry(ring1)
                 multipolygon.AddGeometry(poly1)
@@ -1246,3 +1260,32 @@ class MessageCombo(QtWidgets.QDialog):
 
         """
         return self.master.currentText()
+
+
+def _testfn():
+    """Test."""
+    from IPython import get_ipython
+    get_ipython().run_line_magic('matplotlib', 'inline')
+
+    print('Starting')
+
+    ifile = r"d:\Workdata\modelling\small_upper.npz"
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    DM = ImportMod3D()
+    DM.ifile = ifile
+    DM.settings(nodialog=True)
+
+    odata = DM.outdata['Raster']
+
+    for dat in odata:
+        plt.figure(dpi=150)
+        plt.title(dat.dataid)
+        plt.imshow(dat.data, extent=dat.extent)
+        plt.colorbar()
+        plt.show()
+
+
+if __name__ == "__main__":
+    _testfn()
