@@ -27,11 +27,11 @@
 import time
 from PyQt5 import QtWidgets, QtCore, QtGui
 import numpy as np
-from osgeo import gdal
+import rasterio
 
-import pygmi.menu_default as menu_default
-from pygmi.raster.dataprep import data_to_gdal_mem
-from pygmi.raster.dataprep import gdal_to_dat
+from pygmi import menu_default
+from pygmi.raster.dataprep import data_reproject
+from pygmi.misc import BasicModule
 
 
 def update_lith_lw(lmod, lwidget):
@@ -263,34 +263,14 @@ class ProgressBar():
         QtCore.QCoreApplication.processEvents()
 
 
-class MergeMod3D(QtWidgets.QDialog):
-    """
-    Perform Merge of two models.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class MergeMod3D(BasicModule):
+    """Perform Merge of two models."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-        self.pbar = self.parent.pbar
-
         self.master = QtWidgets.QComboBox()
         self.slave = QtWidgets.QComboBox()
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
         self.setupui()
 
@@ -330,6 +310,11 @@ class MergeMod3D(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -388,8 +373,6 @@ class MergeMod3D(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
     def acceptall(self):
@@ -443,14 +426,16 @@ class MergeMod3D(QtWidgets.QDialog):
         self.outdata['Raster'] = []
 
         for i in datmaster.griddata:
-            if (i in ('DTM Dataset', 'Magnetic Dataset',
-                      'Gravity Dataset', 'Study Area Dataset',
-                      'Gravity Regional')):
+            if i in ('DTM Dataset', 'Magnetic Dataset',
+                     'Gravity Dataset', 'Study Area Dataset',
+                     'Gravity Regional'):
                 if i in datslave.griddata:
                     datmaster.griddata[i] = gmerge(datmaster.griddata[i],
                                                    datslave.griddata[i],
                                                    xrange, yrange)
                 self.outdata['Raster'].append(datmaster.griddata[i])
+            if i == 'Other':
+                self.outdata['Raster'].append(datmaster.griddata[i])
 
         datmaster.update(cols, rows, layers, utlx, utly, utlz, dxy, d_z,
                          usedtm=False)
@@ -484,7 +469,6 @@ class MergeMod3D(QtWidgets.QDialog):
         datmaster.mlut = newmlut
         datmaster.lith_index[datmaster.lith_index == 0] = \
             datslave.lith_index[datmaster.lith_index == 0]
-#        datmaster.lith_index[datmaster.lith_index > 9000] -= 9000
 
         for lith in datslave.lith_list:
             if lith not in datmaster.lith_list:
@@ -522,7 +506,6 @@ def gmerge(master, slave, xrange=None, yrange=None):
 
     xdim = master.xdim
     ydim = master.ydim
-    orig_wkt = master.wkt
 
     xmin = xrange[0]
     xmax = xrange[-1]
@@ -531,37 +514,13 @@ def gmerge(master, slave, xrange=None, yrange=None):
 
     cols = int((xmax - xmin)//xdim)+1
     rows = int((ymax - ymin)//ydim)+1
-    gtr = (xmin, xdim, 0.0, ymax, 0.0, -ydim)
+    otransform = rasterio.Affine(xdim, 0, xmin, 0, -1*ydim, ymax)
 
     dat = []
 
     for data in [master, slave]:
-        doffset = 0.
-        data.data = data.data.astype(float)
-        if data.data.min() <= 0.:
-            doffset = data.data.min()-1.
-            data.data = data.data - doffset
-        data.data.set_fill_value(0.)
-        tmp = data.data.filled()
-        data.data = np.ma.masked_equal(tmp, 0.)
-        data.nullvalue = 0
-
-        drows, dcols = data.data.shape
-
-        gtr0 = data.get_gtr()
-        src = data_to_gdal_mem(data, gtr0, orig_wkt, dcols, drows)
-        dest = data_to_gdal_mem(data, gtr, orig_wkt, cols, rows, True)
-
-        gdal.ReprojectImage(src, dest, orig_wkt, orig_wkt, gdal.GRA_Bilinear)
-
-        dat.append(gdal_to_dat(dest, data.dataid))
-        dat[-1].data = np.ma.masked_outside(dat[-1].data, 0.1,
-                                            data.data.max() + 1000)
-        dat[-1].data = dat[-1].data + doffset
-        dat[-1].data.set_fill_value(1e+20)
-        tmp = dat[-1].data.filled()
-        dat[-1].data = np.ma.masked_equal(tmp, 1e+20)
-        dat[-1].nullvalue = 1e+20
+        dat.append(data_reproject(data, data.crs, data.crs, otransform,
+                                  rows, cols))
 
     imask = np.logical_and(dat[0].data.mask, np.logical_not(dat[1].data.mask))
     if imask.size > 1:
@@ -569,3 +528,36 @@ def gmerge(master, slave, xrange=None, yrange=None):
         dat[0].data.mask[imask] = False
 
     return dat[0]
+
+
+def _testfn():
+    """Test routine."""
+    import sys
+    from pygmi.pfmod.iodefs import ImportMod3D
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    ifile = r"D:\Workdata\modelling\mergetest\3dmodel_test.npz"
+    ifile2 = r"D:\Workdata\modelling\mergetest\3dmodel_test2.npz"
+
+    IO1 = ImportMod3D()
+    IO1.ifile = ifile
+    IO1.settings(True)
+
+    IO2 = ImportMod3D()
+    IO2.ifile = ifile2
+    IO2.settings(True)
+
+    data = {'Model3D': IO1.outdata['Model3D']+IO2.outdata['Model3D'],
+            'Raster': IO1.outdata['Raster']+IO2.outdata['Raster']}
+
+    MM = MergeMod3D()
+    MM.indata = data
+    MM.settings(True)
+
+    for i in MM.outdata['Raster']:
+        print(i.dataid)
+
+
+if __name__ == "__main__":
+    _testfn()
