@@ -24,9 +24,9 @@
 # -----------------------------------------------------------------------------
 """The main program for the modelling package."""
 
+import sys
 from PyQt5 import QtWidgets, QtCore
 
-# Other dependancies
 from pygmi.pfmod import misc
 from pygmi.pfmod import tab_prof
 from pygmi.pfmod import tab_param
@@ -34,7 +34,7 @@ from pygmi.pfmod import tab_mext
 from pygmi.pfmod import grvmag3d
 from pygmi.pfmod import iodefs
 from pygmi.pfmod.datatypes import LithModel
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
 import pygmi.misc as pmisc
 
 
@@ -55,11 +55,9 @@ class MainWidget(QtWidgets.QMainWindow):
         self.parent = parent
         self.lmod1 = LithModel()  # actual model
 
-# General
+        # General
         self.txtmsg = ''
-        # self.modelfilename = r'./tmp'
 
-        # self.toolbar = QtWidgets.QToolBar()
         self.toolbardock = QtWidgets.QToolBar()
         self.statusbar = QtWidgets.QStatusBar()
 
@@ -77,7 +75,7 @@ class MainWidget(QtWidgets.QMainWindow):
         self.lithnotes = tab_param.LithNotes(self)
         self.profile = tab_prof.ProfileDisplay(self)
 
-# Toolbars
+        # Toolbars
         self.action_mext = QtWidgets.QAction('Model\nExtent\nParameters')
         self.toolbardock.addAction(self.action_mext)
         self.action_mext.triggered.connect(self.mext.tab_activate)
@@ -90,7 +88,7 @@ class MainWidget(QtWidgets.QMainWindow):
         self.toolbardock.addAction(self.action_lnotes)
         self.action_lnotes.triggered.connect(self.lithnotes.tab_activate)
 
-# Dock Widgets
+        # Dock Widgets
         dock = QtWidgets.QDockWidget('Editor')
         dock.setWidget(self.profile)
         self.addDockWidget(QtCore.Qt.TopDockWidgetArea, dock)
@@ -118,13 +116,8 @@ class MainWidget(QtWidgets.QMainWindow):
         self.setStatusBar(self.statusbar)
         self.setCentralWidget(centralwidget)
 
-#        self.toolbar.setStyleSheet('QToolBar{spacing:10px;}')
-#        self.addToolBar(QtCore.Qt.TopToolBarArea, self.toolbar)
-#        self.addToolBarBreak()
         self.addToolBar(self.toolbardock)
         self.textbrowser.setFrameShape(QtWidgets.QFrame.StyledPanel)
-#        self.toolbar.setMovable(True)
-#        self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
         self.toolbardock.addAction(self.actionsave)
 
         self.setWindowTitle('Potential Field Modelling')
@@ -174,6 +167,11 @@ class MainWidget(QtWidgets.QMainWindow):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -233,8 +231,6 @@ class MainWidget(QtWidgets.QMainWindow):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
     def data_reset(self):
@@ -282,17 +278,32 @@ def _testfn():
     """Test routine."""
     import sys
     from pygmi.pfmod.iodefs import ImportMod3D
+    from pygmi.pfmod.misc import MergeMod3D
     app = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
 
-    ifile = r'C:\Work\Workdata\people\Mpumi\model2.npz'
+    ifile = r"D:\Workdata\modelling\mergetest\3dmodel_test.npz"
+    ifile2 = r"D:\Workdata\modelling\mergetest\3dmodel_test2.npz"
 
-    IO = ImportMod3D()
+    IO1 = ImportMod3D()
+    IO1.ifile = ifile
+    IO1.settings(True)
 
-    IO.ifile = ifile
-    IO.settings(True)
+    IO2 = ImportMod3D()
+    IO2.ifile = ifile2
+    IO2.settings(True)
+
+    data = {'Model3D': IO1.outdata['Model3D']+IO2.outdata['Model3D'],
+            'Raster': IO1.outdata['Raster']+IO2.outdata['Raster']}
+
+    MM = MergeMod3D()
+    MM.indata = data
+    MM.settings(True)
+
+    for i in MM.outdata['Raster']:
+        print(i.dataid)
 
     PF = MainWidget()
-    PF.indata = IO.outdata
+    PF.indata = MM.outdata
     PF.settings()
 
     app.exec_()
