@@ -27,7 +27,8 @@
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import scipy.interpolate as si
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
 import pygmi.misc as pmisc
 
 
@@ -84,25 +85,22 @@ class MextDisplay(QtWidgets.QDialog):
         buttonbox = QtWidgets.QDialogButtonBox()
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
         buttonbox.setStandardButtons(buttonbox.Cancel | buttonbox.Ok)
+        buttonbox.button(buttonbox.Cancel).setText('No changes')
+        buttonbox.button(buttonbox.Ok).setText('Apply changes '
+                                               '(reset calculations)')
 
-# Current Models Groupbox
+        # Current Models Groupbox
         h_model = QtWidgets.QHBoxLayout()
 
-#        gb_model = QtWidgets.QGroupBox('Current Models')
-#        gl_model = QtWidgets.QGridLayout(gb_model)
-
         lbl1_model = QtWidgets.QLabel('Current Model:')
 
         self.combo_model.addItems(['None'])
         self.combo_model.setSizePolicy(sizepolicy)
 
-#        gl_model.addWidget(lbl1_model, 0, 0, 1, 1)
-#        gl_model.addWidget(self.combo_model, 0, 1, 1, 1)
-
         h_model.addWidget(lbl1_model)
         h_model.addWidget(self.combo_model)
 
-# Data Information Groupbox
+        # Data Information Groupbox
         gb_data_info = QtWidgets.QGroupBox('Dataset Information')
         gl_data_info = QtWidgets.QGridLayout(gb_data_info)
 
@@ -133,7 +131,7 @@ class MextDisplay(QtWidgets.QDialog):
         gl_data_info.addWidget(self.combo_reggrv, 3, 1, 1, 1)
         gl_data_info.addWidget(self.combo_other, 4, 1, 1, 1)
 
-# Data Extents Groupbox
+        # Data Extents Groupbox
         gb_extent = QtWidgets.QGroupBox('Model Extent Properties')
         gl_extent = QtWidgets.QGridLayout(gb_extent)
 
@@ -211,15 +209,14 @@ class MextDisplay(QtWidgets.QDialog):
         hlayout.addWidget(self.pbar)
         hlayout.addWidget(buttonbox)
 
-# Assign to main layout
+        # Assign to main layout
 
-#        verticallayout.addWidget(gb_model)
         verticallayout.addLayout(h_model)
         verticallayout.addWidget(gb_data_info)
         verticallayout.addWidget(gb_extent)
         verticallayout.addLayout(hlayout)
 
-# Link functions
+        # Link functions
         self.dsb_xycell.valueChanged.connect(self.xycell)
         self.dsb_zcell.valueChanged.connect(self.zcell)
         self.dsb_utlx.valueChanged.connect(self.upd_layers)
@@ -263,13 +260,12 @@ class MextDisplay(QtWidgets.QDialog):
         d_z = self.dsb_zcell.value()
 
         self.lmod1.update(cols, rows, layers, utlx, utly, utlz, dxy, d_z,
-                          pbar=self.pbar)
+                          pbar=self.pbar, usedtm=True)
 
         self.update_vals()
 
         # This line is to avoid duplicates since study area and dtm are often
         # the same dataset
-        # tmp = [i for i in set(self.lmod1.griddata.values())]
         tmp = list(set(self.lmod1.griddata.values()))
         self.parent.outdata['Raster'] = tmp
         self.showtext('Changes applied.')
@@ -392,8 +388,8 @@ class MextDisplay(QtWidgets.QDialog):
             utly = curgrid.extent[-1]
             xextent = ccols*curgrid.xdim
             yextent = crows*curgrid.ydim
-            cols = xextent/self.dsb_xycell.value()
-            rows = yextent/self.dsb_xycell.value()
+            cols = int(xextent/self.dsb_xycell.value())
+            rows = int(yextent/self.dsb_xycell.value())
 
             self.dsb_utlx.setValue(utlx)
             self.dsb_utly.setValue(utly)
@@ -411,7 +407,7 @@ class MextDisplay(QtWidgets.QDialog):
         None.
 
         """
-    # Extent Parameters
+        # Extent Parameters
         self.dsb_utlx.setValue(0.0)
         self.dsb_utly.setValue(0.0)
         self.dsb_utlz.setValue(0.0)
@@ -512,27 +508,45 @@ class MextDisplay(QtWidgets.QDialog):
             lkeys = list(self.lmod1.griddata.keys())
             if 'DTM Dataset' in lkeys:
                 tmp = self.lmod1.griddata['DTM Dataset'].dataid
-                self.combo_dtm.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_dtm.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['DTM Dataset']
 
             if 'Magnetic Dataset' in lkeys:
                 tmp = self.lmod1.griddata['Magnetic Dataset'].dataid
-                self.combo_mag.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_mag.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['Magnetic Dataset']
 
             if 'Gravity Dataset' in lkeys:
                 tmp = self.lmod1.griddata['Gravity Dataset'].dataid
-                self.combo_grv.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_grv.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['Gravity Dataset']
 
             if 'Gravity Regional' in lkeys:
                 tmp = self.lmod1.griddata['Gravity Regional'].dataid
-                self.combo_reggrv.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_reggrv.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['Gravity Regional']
 
             if 'Study Area Dataset' in lkeys:
                 tmp = self.lmod1.griddata['Study Area Dataset'].dataid
-                self.combo_dataset.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_dataset.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['Study Area Dataset']
 
             if 'Other' in lkeys:
                 tmp = self.lmod1.griddata['Other'].dataid
-                self.combo_other.setCurrentIndex(gkeys.index(tmp))
+                if tmp in gkeys:
+                    self.combo_other.setCurrentIndex(gkeys.index(tmp))
+                else:
+                    del self.lmod1.griddata['Other']
 
         self.combo_dataset.currentIndexChanged.connect(self.get_area)
 
@@ -624,7 +638,6 @@ class MextDisplay(QtWidgets.QDialog):
         """
         self.update_model_combos()
         self.choose_model()
-#        self.choose_regional()
         self.update_vals()
         self.update_combos()
 
