@@ -29,24 +29,22 @@ import os
 import random
 from PyQt5 import QtWidgets, QtCore, QtGui
 import numpy as np
-import scipy.ndimage as ndimage
+from scipy import ndimage
 from scipy import interpolate
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 from matplotlib.figure import Figure
-from matplotlib import cm
-from osgeo import gdal
+from matplotlib import colormaps
 from osgeo import osr
 import pandas as pd
 import pygmi.raster.iodefs as ir
 
 from pygmi.pfmod import grvmag3d
 from pygmi.pfmod import misc
-import pygmi.menu_default as menu_default
-from pygmi.raster.dataprep import gdal_to_dat
-from pygmi.raster.dataprep import data_to_gdal_mem
+from pygmi import menu_default
+from pygmi.raster.dataprep import data_reproject
 from pygmi.raster.iodefs import get_raster
-from pygmi.misc import frm
+from pygmi.misc import frm, BasicModule
 
 
 class ProfileDisplay(QtWidgets.QWidget):
@@ -87,7 +85,6 @@ class ProfileDisplay(QtWidgets.QWidget):
         self.hs_overview = MySlider()
         self.hs_sideview = MySlider()
         self.combo_overview = QtWidgets.QComboBox()
-#        self.label_sideview = QtWidgets.QLabel('None')
         self.combo_proftype = QtWidgets.QComboBox()
 
         self.sb_layer = QtWidgets.QSpinBox()
@@ -171,7 +168,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         self.sb_profile_linethick.setMaximum(1000)
         self.sb_profile_linethick.setPrefix('Line Thickness: ')
 
-# Set groupboxes and layouts
+        # Set groupboxes and layouts
         gridlayout = QtWidgets.QGridLayout(self)
 
         hl_proftype = QtWidgets.QHBoxLayout()
@@ -193,7 +190,6 @@ class ProfileDisplay(QtWidgets.QWidget):
         hl_pics = QtWidgets.QHBoxLayout()
         hl_pics.addWidget(self.combo_overview)
         hl_pics.addWidget(self.hs_overview)
-#        hl_pics.addWidget(self.label_sideview)
         hl_pics.addWidget(self.hs_sideview)
 
         self.gb_cprof = QtWidgets.QGroupBox('Custom Profile')
@@ -228,7 +224,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         gridlayout.addLayout(vl_plots, 0, 0, 8, 1)
         gridlayout.addLayout(vl_tools, 0, 1, 8, 1)
 
-    # Buttons etc
+        # Buttons etc
         self.sb_profile_linethick.valueChanged.connect(self.setwidth)
         self.lw_prof_defs.currentItemChanged.connect(self.change_defs)
         self.hs_profnum.valueChanged.connect(self.hprofnum)
@@ -357,7 +353,6 @@ class ProfileDisplay(QtWidgets.QWidget):
             self.hs_sideview.setEnabled(False)
             self.prof_dir()
             self.cproflim = None
-#            self.sprofnum()
         else:
             self.gb_dir.setHidden(True)
             self.gb_cprof.setHidden(False)
@@ -480,7 +475,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         if 'Borehole' not in self.parent.indata:
             return
 
-        if self.parent.indata['Raster'][0].wkt == '':
+        if self.parent.indata['Raster'][0].crs.wkt == '':
             return
 
         data = self.parent.indata['Borehole']
@@ -490,7 +485,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         orig.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
 
         targ = osr.SpatialReference()
-        targ.ImportFromWkt(self.parent.indata['Raster'][0].wkt)
+        targ.ImportFromWkt(self.parent.indata['Raster'][0].crs.wkt)
         targ.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
 
         prj = osr.CoordinateTransformation(orig, targ)
@@ -545,7 +540,7 @@ class ProfileDisplay(QtWidgets.QWidget):
                         lithfin[j].append(lith[i])
 
             lithlist = []
-            for i in lithfin:
+            for i in lithfin.keys():
                 lithfin[i] = list(set(lithfin[i]))
                 lithfin[i].sort()
                 lithfin[i] = "".join(i+'/' for i in lithfin[i])[:-1]
@@ -622,22 +617,34 @@ class ProfileDisplay(QtWidgets.QWidget):
             data = self.lmod1.griddata['Calculated Gravity']
 
             for i in self.lmod1.griddata:
-                data1 = self.lmod1.griddata[i]
                 if i in ('Study Area Dataset', 'Gravity Residual',
                          'Magnetic Residual'):
                     continue
+
+                data1 = copy.deepcopy(self.lmod1.griddata[i])
+                if data1.isrgb is True:
+                    # self.showprocesslog(f'Skipping RGB image {data1.dataid}')
+                    continue
                 if 'Calculated Gravity' in i:
                     data1.data = data1.data + self.lmod1.gregional
 
                 dtlx = data.extent[0]
                 d2tlx = data1.extent[0]
-                dtly = data.extent[-1]
-                d2tly = data1.extent[-1]
+                # dtly = data.extent[-1]
+                # d2tly = data1.extent[-1]
 
-                rxxx2 = (dtlx-d2tlx+self.rxxx*data.xdim)/data1.xdim
-                ryyy2 = (d2tly-dtly+self.ryyy*data.ydim)/data1.ydim
+                dbly = data.extent[-2]
+                d2bly = data1.extent[-2]
 
-                tmp = data1.data.filled(np.nan)
+                if 'Calculated' in i:
+                    rxxx2 = self.rxxx
+                    ryyy2 = self.ryyy
+                else:
+                    rxxx2 = (dtlx-d2tlx+self.rxxx*data.xdim)/data1.xdim + 1
+                    # ryyy2 = (d2tly-dtly+self.ryyy*data.ydim)/data1.ydim + 1
+                    ryyy2 = (dbly-d2bly+self.ryyy*data.ydim)/data1.ydim + 1
+
+                tmp = data1.data.astype(float).filled(np.nan)
                 data2[i] = ndimage.map_coordinates(tmp[::-1],
                                                    [ryyy2-0.5,
                                                     rxxx2-0.5],
@@ -647,7 +654,8 @@ class ProfileDisplay(QtWidgets.QWidget):
                 dfall = pd.DataFrame(data2)
             else:
                 dtmp = pd.DataFrame(data2)
-                dfall = dfall.append(dtmp)
+                # dfall = dfall.append(dtmp)
+                dfall = pd.concat([dfall, dtmp])
 
         dfall = dfall.dropna(thresh=4)
         dfall.to_csv(filename, index=False)
@@ -671,7 +679,6 @@ class ProfileDisplay(QtWidgets.QWidget):
         tmp = dtmp.settings()
         if tmp is False:
             return
-#        curgrid = dtmp.outdata['Raster'][0].data[::-1]
         curgrid = dtmp.outdata['Raster'][0]
 
         self.pbar.setValue(100)
@@ -698,11 +705,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         regz = self.lmod1.zrange[1]
         d_z = self.lmod1.d_z
 
-#        gxrng = np.array([tlx+i*d_x for i in range(cols)])
-#        gyrng = np.array([(tly-(rows-1)*d_y)+i*d_y for i in range(rows)])
-
-# This section gets rid of null values quickly
-#        xt, yt = np.meshgrid(gxrng, gyrng)
+        # This section gets rid of null values quickly
         zt = curgrid.data.data
 
         if isdepths is True:
@@ -714,13 +717,7 @@ class ProfileDisplay(QtWidgets.QWidget):
 
         msk = np.logical_not(np.logical_or(curgrid.data.mask, np.isnan(zt)))
 
-#        zt = zt[msk]
-#        xy1 = np.transpose([xt[msk], yt[msk]])
-#        xy2 = np.transpose([xt, yt])
-#        newgrid = np.transpose(si.griddata(xy1, zt, xy2, 'nearest'))
-
-# Back to splines
-#        fgrid = si.RectBivariateSpline(gyrng, gxrng, newgrid)
+        # Back to splines
 
         for i in range(self.lmod1.numx):
             for j in range(self.lmod1.numy):
@@ -730,16 +727,13 @@ class ProfileDisplay(QtWidgets.QWidget):
                 igrd = int((imod-tlx)/d_x)
                 jgrd = int((tly-jmod)/d_y)
 
-#                if igrd >= 0 and jgrd >= 0 and igrd < cols and jgrd < rows:
                 if 0 <= igrd < cols and 0 <= jgrd < rows:
                     if not msk[jgrd, igrd]:
                         continue
 
                     k_2 = int((regz - zt[jgrd, igrd])/d_z)
 
-#                    k_2 = int((regz-fgrid(jmod, imod))/d_z)
-                    if k_2 < 0:
-                        k_2 = 0
+                    k_2 = max(k_2, 0)
                     lfilt = self.lmod1.lith_index[i, j, k_2:] != -1
                     ufilt = self.lmod1.lith_index[i, j, :k_2] != -1
                     if lowerb != -999:
@@ -750,7 +744,6 @@ class ProfileDisplay(QtWidgets.QWidget):
         gtmp = self.get_model()
         self.mmc.init_grid(gtmp)
         self.mmc.init_grid_top()
-#        self.mmc.figure.canvas.draw()
 
     def rcopy(self):
         """
@@ -782,7 +775,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         Parameters
         ----------
         rcopy : RangedCopy
-            Handle of ranged copy gui.
+            Handle of ranged copy GUI.
 
         Returns
         -------
@@ -875,7 +868,7 @@ class ProfileDisplay(QtWidgets.QWidget):
                         udatad[ref] = []
                     udatad[ref].append(ltmp[zz, self.ipdx1+ixy])
 
-            for i2 in udatad:
+            for i2 in udatad.keys():
                 if 0 in udatad[i2]:
                     zcnt = udatad[i2].count(0)
                     if (zcnt/len(udatad[i2])) <= 0.8:
@@ -1324,8 +1317,8 @@ class ProfileDisplay(QtWidgets.QWidget):
         self.sb_prof_dir.setValue(pdirval)
         self.sb_prof_dir.valueChanged.connect(self.sprofdir)
 
-        self.sb_profnum.setValue(0.0)
-        self.hs_profnum.setValue(0.0)
+        self.sb_profnum.setValue(0)
+        self.hs_profnum.setValue(0)
         self.hs_sideview.setEnabled(False)
 
         self.sb_layer.setMaximum(self.lmod1.numz-1)
@@ -1378,7 +1371,7 @@ class ProfileDisplay(QtWidgets.QWidget):
         None.
 
         """
-# Display the calculated profile
+        # Display the calculated profile
         if self.viewmagnetics:
             data = self.lmod1.griddata['Calculated Magnetics']
             self.mmc.ptitle = 'Magnetic Intensity: '
@@ -1406,14 +1399,14 @@ class ProfileDisplay(QtWidgets.QWidget):
 
         if self.pscale_type == 'custmax':
             extent = [self.plot_custmin, self.plot_custmax]
-        elif self.pscale_type == 'calcmax' or self.pscale_type == 'allmax':
+        elif self.pscale_type in ('calcmax', 'allmax'):
             extent = [data.data.min()+regtmp, data.data.max()+regtmp]
         elif tmpprof.size > 0:
             extent = [tmpprof.min(), tmpprof.max()]
         else:
             extent = [data.data.min()+regtmp, data.data.max()+regtmp]
 
-# Load in observed data - if there is any
+        # Load in observed data - if there is any
         data2 = None
         tmprng2 = None
         tmpprof2 = None
@@ -1425,7 +1418,7 @@ class ProfileDisplay(QtWidgets.QWidget):
 
         if data2 is not None:
             data2.data = np.pad(data2.data, 1, 'edge')
-            data2.data = np.ma.masked_equal(data2.data, data2.nullvalue)
+            data2.data = np.ma.masked_equal(data2.data, data2.nodata)
 
             dtlx = data.extent[0]
             d2tlx = data2.extent[0]
@@ -1435,8 +1428,9 @@ class ProfileDisplay(QtWidgets.QWidget):
             rxxx2 = (dtlx-d2tlx+self.rxxx*data.xdim)/data2.xdim+1
             ryyy2 = (dbly-d2bly+self.ryyy*data.ydim)/data2.ydim+1
 
+            pdtmp = data2.data.filled(np.nan)
             tmprng2 = np.linspace(px1, px2, len(rxxx2))
-            tmpprof2 = ndimage.map_coordinates(data2.data[::-1],
+            tmpprof2 = ndimage.map_coordinates(pdtmp[::-1],
                                                [ryyy2-0.5, rxxx2-0.5],
                                                order=1, cval=np.nan)
 
@@ -1531,7 +1525,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         super().__init__(fig)
 
         self.lmod1 = parent.lmod1
-        self.cbar = cm.get_cmap('jet')
+        self.cbar = colormaps['jet']
         self.curmodel = 0
         self.mywidth = 1
         self.xold = None
@@ -1551,14 +1545,14 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.crd = None
         self.myparent = parent
 
-# Events
+        # Events
         self.figure.canvas.mpl_connect('motion_notify_event', self.move)
         self.figure.canvas.mpl_connect('button_press_event', self.button_press)
         self.figure.canvas.mpl_connect('button_release_event',
                                        self.button_release)
         self.figure.canvas.mpl_connect('resize_event', self.on_resize)
 
-# Initial Images
+        # Initial Images
         self.paxes = fig.add_subplot(222)
         self.paxes.yaxis.set_label_text('mGal')
         self.paxes.ticklabel_format(useOffset=False)
@@ -1569,7 +1563,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes = fig.add_subplot(224, sharex=self.paxes)
         self.axes.xaxis.set_label_text(self.xlabel)
         self.axes.yaxis.set_label_text('Altitude (m)')
-#        self.axes.callbacks.connect('xlim_changed', self.paxes_lim_update)
 
         self.laxes = fig.add_subplot(121)
 
@@ -1606,8 +1599,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Event variable.
 
         Returns
         -------
@@ -1618,7 +1611,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
             return
 
         nmode = event.inaxes.get_navigate_mode()
-#        nmode = self.figure.canvas.toolbar._active
         if event.button == 1 and nmode is None:
             self.press = True
             self.newline = True
@@ -1630,8 +1622,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Unused.
 
         Returns
         -------
@@ -1646,8 +1638,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Event variable.
 
         Returns
         -------
@@ -1662,7 +1654,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         if curaxes == self.axes:
             mdata = self.mdata
             xptp = self.lmod1.xrange[1]-self.lmod1.xrange[0]
-#            xmin = self.lmod1.xrange[0]
             xmin = 0
 
             dx = self.myparent.pdxy
@@ -1690,10 +1681,10 @@ class MyMplCanvas(FigureCanvasQTAgg):
             width *= self.mywidth
             height *= self.mywidth
 
-            if curaxes == self.axes:
-                width *= 10
-            width = np.ceil(width)
-            height = np.ceil(height)
+            # if curaxes == self.axes:
+            #     width *= 10
+            width = int(np.ceil(width))
+            height = int(np.ceil(height))
 
             cbit = QtGui.QBitmap(width, height)
             cbit.fill(QtCore.Qt.color1)
@@ -1710,8 +1701,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
                 rrr = np.sqrt((self.xold-xdata)**2+(self.yold-ydata)**2)
                 steps = int(rrr)+1
-#                steps = int(max(abs(self.xold-xdata),
-#                                abs(self.yold-ydata)))+1
                 xxx = np.linspace(self.xold, xdata, steps)
                 yyy = np.linspace(self.yold, ydata, steps)
 
@@ -1741,7 +1730,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
             else:
                 self.lmod1.lith_index[:, :, curlayer] = mdata.T
                 self.lmdata = mdata
-                self.mdata = self.lmod1.lith_index[xxx, yyy, ::-1].T
+                self.mdata = self.myparent.get_model()
 
             self.slide_grid(self.mdata)
             self.slide_grid_top()
@@ -1767,10 +1756,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
         None.
 
         """
-        if xdata < 0:
-            xdata = 0
-        if ydata < 0:
-            ydata = 0
+        xdata = max(xdata, 0)
+        ydata = max(ydata, 0)
 
         hwidth = self.mywidth/2
         xstart = max(0, xdata-hwidth)
@@ -1794,12 +1781,12 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         dat : numpy array
-            DESCRIPTION.
+            Input data.
 
         Returns
         -------
         tmp : numpy array
-            DESCRIPTION.
+            dat grid.
 
         """
         mlut = self.lmod1.mlut
@@ -1809,7 +1796,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
             if i == -1:
                 ctmp = [0, 0, 0, 0]
             else:
-                ctmp = np.array(mlut[i]+[255])/255.
+                ctmp = np.array(list(mlut[i])+[255])/255.
 
             tmp[dat[::-1] == i] = ctmp
 
@@ -1823,8 +1810,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Unused.
 
         Returns
         -------
@@ -1871,6 +1858,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
             self.ims2.set_data(dat2.data)
             self.ims2.set_extent(dat2.extent)
             self.ims2.set_clim(dat2.data.min(), dat2.data.max())
+            self.ims2.set_clim(dat2.data.min(), dat2.data.max())
         else:
             self.ims.set_alpha(1.0)
 
@@ -1922,7 +1910,22 @@ class MyMplCanvas(FigureCanvasQTAgg):
             dat2 = self.lmod1.griddata[str(dat2)]
             self.lims2.set_data(dat2.data)
             self.lims2.set_extent(dat2.extent)
-            self.lims2.set_clim(dat2.data.min(), dat2.data.max())
+
+            # vmin = dat2.data.min()
+            # vmax = dat2.data.max()
+
+            vstd = dat2.data.std()
+            vmean = dat2.data.mean()
+            vmin = vmean - 2.5 * vstd
+            vmax = vmean + 2.5 * vstd
+
+            # The next two lines must be called twice. Otherwize,
+            # depending on the previous limits, either vmin or vmax is changed,
+            # to ensure they are less or greater than the previous limits.
+
+            self.lims2.set_clim(vmin, vmax)
+            self.lims2.set_clim(vmin, vmax)
+
             self.colbar.ax.set_xlabel(dat2.units)
 
         left, right = self.lmod1.xrange
@@ -1973,7 +1976,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         if dat2 is not None:
             self.ims2.set_visible(True)
-#            self.ims2.set_alpha(self.opac)
 
         self.axes.draw_artist(self.ims)
         self.axes.draw_artist(self.ims2)
@@ -2156,8 +2158,8 @@ class MySlider(QtWidgets.QSlider):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Event variable.
 
         Returns
         -------
@@ -2177,8 +2179,8 @@ class MySlider(QtWidgets.QSlider):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Event variable.
 
         Returns
         -------
@@ -2335,9 +2337,22 @@ class PlotScale(QtWidgets.QDialog):
         vl_scale.addWidget(self.dsb_axis_custmax)
         vl_scale.addWidget(self.buttonbox)
 
+        self.dsb_axis_custmax.valueChanged.connect(self.custom)
+        self.dsb_axis_custmin.valueChanged.connect(self.custom)
         self.buttonbox.accepted.connect(self.accept)
         self.buttonbox.rejected.connect(self.reject)
 
+    def custom(self):
+        """
+        Set custom radiobutton when limits are changed.
+
+        Returns
+        -------
+        None.
+
+        """
+        self.rb_axis_custmax.setChecked(True)
+
 
 class RangedCopy(QtWidgets.QDialog):
     """Class to call up a dialog for ranged copying."""
@@ -2540,8 +2555,8 @@ class GaugeWidget(QtWidgets.QDial):
 
         Parameters
         ----------
-        event : TYPE
-            DESCRIPTION.
+        event : event
+            Event variable.
 
         Returns
         -------
@@ -2562,22 +2577,14 @@ class GaugeWidget(QtWidgets.QDial):
         painter.end()
 
 
-class ImportPicture(QtWidgets.QDialog):
+class ImportPicture(BasicModule):
     """Import Picture dialog."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-        self.parent = parent
         self.lmod = self.parent.lmod1
 
-        self.ifile = ''
-        self.indata = {}
-        self.outdata = {}
         self.grid = None
 
         self.dsb_x1 = QtWidgets.QDoubleSpinBox()
@@ -2723,16 +2730,17 @@ class ImportPicture(QtWidgets.QDialog):
 
     def settings(self, nodialog=False):
         """
-        Entrypoint into class.
+        Entry point into item.
 
-        This is called when the used double clicks the routine from the
-        main PyGMI interface.
-
-        This section also imports the picture.
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
-        None.
+        bool
+            True if successful, False otherwise.
 
         """
         temp = self.exec_()
@@ -2743,7 +2751,7 @@ class ImportPicture(QtWidgets.QDialog):
         x2 = self.dsb_x2.value()
         y1 = self.dsb_y1.value()
         y2 = self.dsb_y2.value()
-        zmin = self.dsb_zmin.value()
+        # zmin = self.dsb_zmin.value()
         zmax = self.dsb_zmax.value()
 
         # Check if the profile is within the model area
@@ -2809,11 +2817,6 @@ class ImportPicture(QtWidgets.QDialog):
 
         imptext = self.importfile.text()
         if imptext != '':
-            # x1a = self.dsb_x1.value()
-            # x2a = self.dsb_x2.value()
-            # y1a = self.dsb_y1.value()
-            # y2a = self.dsb_y2.value()
-
             dat = get_raster(imptext, showprocesslog=self.showprocesslog)
 
             if dat is None:
@@ -2829,9 +2832,11 @@ class ImportPicture(QtWidgets.QDialog):
             dat.isrgb = True
 
             ra = np.sqrt((x1a-x1)**2+(y1a-y1)**2)
-            rb = np.sqrt((x2a-x1)**2+(y2a-y1)**2)
+            # rb = np.sqrt((x2a-x1)**2+(y2a-y1)**2)
+            # xdim = (rb-ra)/dat.shape[1]
+            # ydim = (zmax-zmin)/dat.shape[0]
 
-            dat.extent = (ra, rb, zmin, zmax)
+            dat.set_transform(dat.xdim, ra, dat.ydim, zmax)
             self.lmod.profpics[curline] = dat
 
         return curline
@@ -2865,8 +2870,6 @@ class ImportPicture(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
 
@@ -2891,29 +2894,10 @@ def gridmatch2(cgrv, rgrv):
     """
     data = rgrv
     data2 = cgrv
-    orig_wkt = data.wkt
-    orig_wkt2 = data2.wkt
+    rows2, cols2 = data2.data.shape
 
-    doffset = 0.0
-    if data.data.min() <= 0:
-        doffset = data.data.min()-1.
-        data.data = data.data - doffset
-
-    drows, dcols = data.data.shape
-    d2rows, d2cols = data2.data.shape
-
-    gtr0 = data.get_gtr()
-    gtr = data2.get_gtr()
-    src = data_to_gdal_mem(data, gtr0, orig_wkt, dcols, drows)
-    dest = data_to_gdal_mem(data, gtr, orig_wkt2, d2cols, d2rows, True)
-
-    gdal.ReprojectImage(src, dest, orig_wkt, orig_wkt2, gdal.GRA_Bilinear)
-
-    dat = gdal_to_dat(dest, data.dataid)
-
-    if doffset != 0.0:
-        dat.data = dat.data + doffset
-        data.data = data.data + doffset
+    dat = data_reproject(data, data.crs, data2.crs, data2.transform,
+                         rows2, cols2)
 
     return dat.data
 
