@@ -27,14 +27,13 @@
 from PyQt5 import QtWidgets, QtCore
 from scipy import ndimage
 import numpy as np
-import matplotlib.cm as cm
+from matplotlib import colormaps
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib import collections as mc
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
-from matplotlib.pyplot import colormaps
 
-from pygmi.misc import frm
+from pygmi.misc import frm, ContextModule
 
 
 class MyMplCanvas(FigureCanvasQTAgg):
@@ -145,6 +144,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes.add_collection(lc)
 
         self.axes.autoscale()
+
         self.axes.set_xlabel('Eastings')
         self.axes.set_ylabel('Northings')
 
@@ -157,7 +157,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
     def update_raster(self, data1, scale=7, rotang=10, atype='dubois',
-                      cmap=cm.get_cmap('jet'), shade=False):
+                      cmap=colormaps['jet'], shade=False):
         """
         Update the raster plot.
 
@@ -172,7 +172,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         atype : str, optional
             Anaglyph type. The default is 'dubois'.
         cmap : matplotlib.colors.LinearSegmentedColormap, optional
-            Matplotlib colormap. The default is cm.jet.
+            Matplotlib colormap. The default is jet.
         shade : bool, optional
             Option to choose sunshading. The default is False.
 
@@ -197,7 +197,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         self.update_colors(shade, cmap, atype)
 
-    def update_colors(self, doshade=False, cmap=cm.get_cmap('jet'),
+    def update_colors(self, doshade=False, cmap=colormaps['jet'],
                       atype='dubois'):
         """
         Update colors.
@@ -207,7 +207,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         doshade : bool, optional
             Option to choose sunshading. The default is False.
         cmap : matplotlib.colors.LinearSegmentedColormap, optional
-            Matplotlib colormap. The default is cm.jet.
+            Matplotlib colormap. The default is jet.
         atype : str, optional
             Anaglyph type. The default is 'dubois'.
 
@@ -224,13 +224,13 @@ class MyMplCanvas(FigureCanvasQTAgg):
             tmp[tmp < 0] = 0.
             tmp[tmp > 1] = 1.
             self.red = cmap(tmp)
-            self.red[:, :, 3] = np.logical_not(self.red1.mask)
+            self.red[:, :, 3] = np.logical_not(np.ma.getmaskarray(self.red1))
 
             tmp = norm2(self.blue1, zmin, zmax)
             tmp[tmp < 0] = 0.
             tmp[tmp > 1] = 1.
             self.blue = cmap(tmp)
-            self.blue[:, :, 3] = np.logical_not(self.blue1.mask)
+            self.blue[:, :, 3] = np.logical_not(np.ma.getmaskarray(self.blue1))
         else:
             alpha = 0
             cell = 100
@@ -276,36 +276,24 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class PlotAnaglyph(QtWidgets.QDialog):
-    """
-    Graph Window - The QDialog window which will contain our image.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    """
+class PlotAnaglyph(ContextModule):
+    """Graph Window - The QDialog window which will contain our image."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.parent = parent
-        self.indata = {}
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Anaglyph (3D Image: Glasses Needed)')
 
         sizepolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum,
                                            QtWidgets.QSizePolicy.Minimum)
 
-
-# Define Layouts
+        # Define Layouts
         hbl = QtWidgets.QHBoxLayout(self)  # self is where layout is assigned
         vbl_left = QtWidgets.QVBoxLayout()
         vbl_right = QtWidgets.QVBoxLayout()
 
-
-# Define Widgets
+        # Define Widgets
         self.mmc = MyMplCanvas(self)
         mpl_toolbar = NavigationToolbar2QT(self.mmc, self.parent)
         self.combobox1 = QtWidgets.QComboBox()
@@ -319,7 +307,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
         self.slider_angle = QtWidgets.QSlider()
         self.slider_cnt = QtWidgets.QSlider()
 
-# Size policies
+        # Size policies
         self.combobox1.setSizePolicy(sizepolicy)
         self.combobox2.setSizePolicy(sizepolicy)
         self.cbox_cbar.setSizePolicy(sizepolicy)
@@ -331,7 +319,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
         self.slider_angle.setSizePolicy(sizepolicy)
         self.slider_cnt.setSizePolicy(sizepolicy)
 
-# Configure Widgets
+        # Configure Widgets
         self.combobox2.addItem('Dubois (Red-Green)')
         self.combobox2.addItem('Green-Magenta')
         self.combobox2.addItem('Amber-Blue')
@@ -341,8 +329,6 @@ class PlotAnaglyph(QtWidgets.QDialog):
 
         maps = sorted(m for m in colormaps() if not
                       m.startswith(('spectral', 'Vega', 'jet')))
-        # maps = sorted(m for m in cm.cmap_d.keys() if not
-        #               m.startswith(('spectral', 'Vega', 'jet')))
 
         self.cbox_cbar.addItem('jet')
         self.cbox_cbar.addItems(maps)
@@ -363,7 +349,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
         self.slider_angle.setTickPosition(QtWidgets.QSlider.TicksAbove)
         self.slider_angle.setValue(10)
 
-# Add widgets to layout
+        # Add widgets to layout
         vbl_left.addWidget(self.doimage)
         vbl_left.addWidget(self.docontour)
         vbl_left.addWidget(self.slider_cnt)
@@ -371,14 +357,13 @@ class PlotAnaglyph(QtWidgets.QDialog):
         vbl_left.addWidget(self.combobox1)
         vbl_left.addWidget(QtWidgets.QLabel('Type:'))
         vbl_left.addWidget(self.combobox2)
-        vbl_left.addWidget(QtWidgets.QLabel('Color Bar:'))
+        vbl_left.addWidget(QtWidgets.QLabel('Colour Bar:'))
         vbl_left.addWidget(self.cbox_cbar)
         vbl_left.addWidget(QtWidgets.QLabel('Scale (1-30):'))
         vbl_left.addWidget(self.slider_scale)
         vbl_left.addWidget(QtWidgets.QLabel('Image Angle (1-20):'))
         vbl_left.addWidget(self.slider_angle)
         vbl_left.addWidget(self.doshade)
-#        vbl_left.addStretch()
         vbl_right.addWidget(self.mmc)
         vbl_right.addWidget(mpl_toolbar)
         hbl.addLayout(vbl_left)
@@ -407,7 +392,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
         """
         i = self.combobox1.currentIndex()
         txt = str(self.cbox_cbar.currentText())
-        cbar = cm.get_cmap(txt)
+        cbar = colormaps[txt]
         shade = self.doshade.isChecked()
         scale = self.slider_scale.value()
         rotang = self.slider_angle.value()
@@ -423,7 +408,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
 
     def change_colors(self):
         """
-        Update color bar.
+        Update colour bar.
 
         Returns
         -------
@@ -431,7 +416,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
 
         """
         txt = str(self.cbox_cbar.currentText())
-        cbar = cm.get_cmap(txt)
+        cbar = colormaps[txt]
         shade = self.doshade.isChecked()
 
         self.mmc.update_colors(atype=self.combobox2.currentText(),
@@ -458,8 +443,6 @@ class PlotAnaglyph(QtWidgets.QDialog):
 
         """
         self.docontour.setChecked(True)
-#        self.slider_scale.setValue(3)
-#        self.slider_angle.setValue(5)
 
         i = self.combobox1.currentIndex()
         scale = self.slider_scale.value()
@@ -514,7 +497,7 @@ class PlotAnaglyph(QtWidgets.QDialog):
 
 
 def sunshade(data, azim=-np.pi/4., elev=np.pi/4., alpha=1, cell=100,
-             cmap=cm.get_cmap('terrain')):
+             cmap=colormaps['terrain']):
     """
     Perform Sunshading on data.
 
@@ -537,7 +520,7 @@ def sunshade(data, azim=-np.pi/4., elev=np.pi/4., alpha=1, cell=100,
     Returns
     -------
     colormap : numpy array
-        Output color mapped array (MxNx4).
+        Output colour mapped array (MxNx4).
 
     """
     mask = np.ma.getmaskarray(data)
@@ -545,7 +528,6 @@ def sunshade(data, azim=-np.pi/4., elev=np.pi/4., alpha=1, cell=100,
     sunshader = currentshader(data, cell, elev, azim, alpha)
     snorm = norm2(sunshader)
     pnorm = np.uint8(norm2(histcomp(data))*255)
-    # pnorm = uint8(norm2(data)*255)
 
     colormap = cmap(pnorm)
     colormap[:, :, 0] = colormap[:, :, 0]*snorm
@@ -617,12 +599,12 @@ def currentshader(data, cell, theta, phi, alpha):
     pinit = dzdx
     qinit = dzdy
 
-# Update cell
+    # Update cell
     p = pinit/cell
     q = qinit/cell
     sqrt_1p2q2 = np.sqrt(1+p**2+q**2)
 
-# Update angle
+    # Update angle
     cosg2 = np.cos(theta/2)
     p0 = -np.cos(phi)*np.tan(theta)
     q0 = -np.sin(phi)*np.tan(theta)
@@ -673,8 +655,7 @@ def histcomp(img, nbr_bins=256, perc=5.):
     evalue = bins[eindx]
 
     scnt = perc*(nbr_bins-1)
-    if scnt > sindx:
-        scnt = sindx
+    scnt = min(scnt, sindx)
 
     ecnt = perc*(nbr_bins-1)
     if ecnt > ((nbr_bins-1)-eindx):
@@ -694,7 +675,7 @@ def histcomp(img, nbr_bins=256, perc=5.):
 
 def anaglyph(red, blue, atype='dubois'):
     """
-    Color Anaglyph.
+    Colour Anaglyph.
 
     Parameters
     ----------
@@ -702,7 +683,7 @@ def anaglyph(red, blue, atype='dubois'):
         Dataset for red channel.
     blue : numpy array
         Dataset for blue channel.
-    atype : TYPE, optional
+    atype : str, optional
         Anaglyph type. The default is 'dubois'.
 
     Returns
