@@ -34,20 +34,16 @@ import copy
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
+from pygmi.misc import ProgressBarText, BasicModule
 
 
-class Gradients(QtWidgets.QDialog):
+class Gradients(BasicModule):
     """
     Class used to gather information via a GUI, for function gradients.
 
     Attributes
     ----------
-    parent : parent
-    indata : dictionary
-        PyGMI input data in a dictionary
-    outdata :
-        PyGMI input data in a dictionary
     azi : float
         Azimuth/filter direction (degrees)
     elev : float
@@ -58,27 +54,15 @@ class Gradients(QtWidgets.QDialog):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.azi = 45.
-        self.elev = 45.
+        self.azi = 45
         self.order = 1
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
 
         self.sb_order = QtWidgets.QSpinBox()
         self.sb_azi = QtWidgets.QSpinBox()
         self.rb_ddir = QtWidgets.QRadioButton('Directional Derivative')
         self.rb_vgrad = QtWidgets.QRadioButton('Vertical Derivative')
         self.rb_dratio = QtWidgets.QRadioButton('Derivative Ratio')
+        self.rb_thg = QtWidgets.QRadioButton('Total Horizonal Gradient')
         self.label_or = QtWidgets.QLabel('Strength Factor')
         self.label_az = QtWidgets.QLabel('Azimuth')
 
@@ -115,29 +99,40 @@ class Gradients(QtWidgets.QDialog):
         gridlayout.addWidget(self.rb_ddir, 0, 0, 1, 1)
         gridlayout.addWidget(self.rb_dratio, 1, 0, 1, 1)
         gridlayout.addWidget(self.rb_vgrad, 2, 0, 1, 1)
-        gridlayout.addWidget(self.label_az, 3, 0, 1, 1)
-        gridlayout.addWidget(self.sb_azi, 3, 1, 1, 1)
-        gridlayout.addWidget(self.label_or, 4, 0, 1, 1)
-        gridlayout.addWidget(self.sb_order, 4, 1, 1, 1)
-        gridlayout.addWidget(helpdocs, 5, 0, 1, 1)
-        gridlayout.addWidget(buttonbox, 5, 1, 1, 1)
+        gridlayout.addWidget(self.rb_thg, 3, 0, 1, 1)
+        gridlayout.addWidget(self.label_az, 4, 0, 1, 1)
+        gridlayout.addWidget(self.sb_azi, 4, 1, 1, 1)
+        gridlayout.addWidget(self.label_or, 5, 0, 1, 1)
+        gridlayout.addWidget(self.sb_order, 5, 1, 1, 1)
+        gridlayout.addWidget(helpdocs, 6, 0, 1, 1)
+        gridlayout.addWidget(buttonbox, 6, 1, 1, 1)
 
         buttonbox.accepted.connect(self.accept)
         buttonbox.rejected.connect(self.reject)
         self.rb_ddir.clicked.connect(self.radiochange)
         self.rb_dratio.clicked.connect(self.radiochange)
         self.rb_vgrad.clicked.connect(self.radiochange)
+        self.rb_thg.clicked.connect(self.radiochange)
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         if not nodialog:
             temp = self.exec_()
             if temp == 0:
@@ -155,6 +150,8 @@ class Gradients(QtWidgets.QDialog):
             elif self.rb_dratio.isChecked():
                 data[i].data = derivative_ratio(data[i].data, self.azi,
                                                 self.order)
+            elif self.rb_thg.isChecked():
+                data[i].data = thgrad(data[i].data, data[i].xdim, data[i].ydim)
             else:
                 if data[i].xdim != data[i].ydim:
                     self.showprocesslog('X and Y dimension are different. '
@@ -165,8 +162,13 @@ class Gradients(QtWidgets.QDialog):
                 dxy = data[i].xdim
                 data[i].data = np.ma.array(vertical(data[i].data, xint=dxy))
                 data[i].data.mask = mask
+
             data[i].units = ''
 
+            mask = data[i].data.mask
+            data[i].data = data[i].data.filled(data[i].nodata)
+            data[i].data = np.ma.array(data[i].data, mask=mask)
+
         self.outdata['Raster'] = data
 
         return True
@@ -283,6 +285,32 @@ def gradients(data, azi, xint, yint):
     return dt1
 
 
+def thgrad(data, xint, yint):
+    """
+    Gradients.
+
+    Compute total horizontal gradient.
+
+    Parameters
+    ----------
+    data : numpy array
+        input numpy data array
+    xint : float
+        X interval/distance.
+    yint : float
+        Y interval/distance.
+
+    Returns
+    -------
+    dt1 : float
+        returns gradient.
+    """
+    dy, dx = np.gradient(data, yint, xint)
+    dt1 = np.sqrt(dx**2+dy**2)
+
+    return dt1
+
+
 def derivative_ratio(data, azi, order):
     """
     Compute derivative ratio of image data. Based on code by Gordon Cooper.
@@ -316,17 +344,12 @@ def derivative_ratio(data, azi, order):
     return dr
 
 
-class Visibility2d(QtWidgets.QDialog):
+class Visibility2d(BasicModule):
     """
     Class used to gather information via a GUI, for function visibility2d.
 
     Attributes
     ----------
-    parent : parent
-    indata : dictionary
-        PyGMI input data in a dictionary
-    outdata :
-        PyGMI input data in a dictionary
     wsize : int
         window size, must be odd
     dh : float
@@ -335,22 +358,9 @@ class Visibility2d(QtWidgets.QDialog):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
         self.wsize = 11
         self.dh = 10
 
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
-
         self.sb_dh = QtWidgets.QSpinBox()
         self.sb_wsize = QtWidgets.QSpinBox()
 
@@ -377,7 +387,7 @@ class Visibility2d(QtWidgets.QDialog):
         self.sb_dh.setMinimum(1)
         self.sb_dh.setMaximum(10000)
         self.sb_wsize.setPrefix('')
-        self.sb_wsize.setMinimum(3)
+        self.sb_wsize.setMinimum(5)
         self.sb_wsize.setMaximum(100000)
         self.sb_wsize.setSingleStep(2)
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
@@ -399,12 +409,21 @@ class Visibility2d(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         if not nodialog:
             temp = self.exec_()
             if temp == 0:
@@ -422,6 +441,13 @@ class Visibility2d(QtWidgets.QDialog):
             vtot, vstd, vsum = visibility2d(datai.data, self.wsize,
                                             self.dh*data[i].data.std()/100.,
                                             self.piter)
+
+            buff = self.wsize//2
+
+            xmin, ymax = datai.transform*(buff, buff)
+            xdim = datai.transform[0]
+            ydim = abs(datai.transform[4])
+
             data2.append(copy.deepcopy(datai))
             data2.append(copy.deepcopy(datai))
             data2.append(copy.deepcopy(datai))
@@ -431,6 +457,14 @@ class Visibility2d(QtWidgets.QDialog):
             data2[-3].dataid += ' Total Visibility'
             data2[-2].dataid += ' Visibility Variation'
             data2[-1].dataid += ' Visibility Vector Resultant'
+            data2[-3].set_transform(xdim, xmin, ydim, ymax)
+            data2[-2].set_transform(xdim, xmin, ydim, ymax)
+            data2[-1].set_transform(xdim, xmin, ydim, ymax)
+
+            for i in [-3, -2, -1]:
+                mask = data2[i].data.mask
+                data2[i].data = data2[i].data.filled(data2[i].nodata)
+                data2[i].data = np.ma.array(data2[i].data, mask=mask)
 
         self.outdata['Raster'] = data2
         self.showprocesslog('Finished!')
@@ -716,3 +750,242 @@ def vertical(data, npts=None, xint=1, order=1):
     dz = np.real(fzinv[rdiff:nr+rdiff, cdiff:nc+cdiff])
 
     return dz
+
+
+class AGC(BasicModule):
+    """
+    Class used to gather information via a GUI, for function AGC.
+
+    Attributes
+    ----------
+    wsize : int
+        window size, must be odd
+    """
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.wsize = 11
+
+        self.sb_wsize = QtWidgets.QSpinBox()
+        self.rb_mean = QtWidgets.QRadioButton('Mean')
+        self.rb_median = QtWidgets.QRadioButton('Median')
+        self.rb_rms = QtWidgets.QRadioButton('RMS')
+
+        self.setupui()
+
+        self.sb_wsize.setValue(self.wsize)
+
+    def setupui(self):
+        """
+        Set up UI.
+
+        Returns
+        -------
+        None.
+
+        """
+        gridlayout = QtWidgets.QGridLayout(self)
+        buttonbox = QtWidgets.QDialogButtonBox()
+        helpdocs = menu_default.HelpButton('pygmi.raster.cooper.AGC')
+        label_2 = QtWidgets.QLabel('Window Size (Odd)')
+
+        self.sb_wsize.setPrefix('')
+        self.sb_wsize.setMinimum(3)
+        self.sb_wsize.setMaximum(100000)
+        self.sb_wsize.setSingleStep(2)
+        self.sb_wsize.setValue(self.wsize)
+        self.rb_mean.setChecked(True)
+        buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        buttonbox.setStandardButtons(buttonbox.Cancel | buttonbox.Ok)
+
+        self.setWindowTitle('Automatic Gain Control')
+
+        gridlayout.addWidget(self.rb_mean, 0, 0, 1, 1)
+        gridlayout.addWidget(self.rb_median, 1, 0, 1, 1)
+        gridlayout.addWidget(self.rb_rms, 2, 0, 1, 1)
+        gridlayout.addWidget(label_2, 3, 0, 1, 1)
+        gridlayout.addWidget(self.sb_wsize, 3, 1, 1, 1)
+        gridlayout.addWidget(helpdocs, 4, 0, 1, 1)
+        gridlayout.addWidget(buttonbox, 4, 1, 1, 1)
+
+        buttonbox.accepted.connect(self.accept)
+        buttonbox.rejected.connect(self.reject)
+
+    def settings(self, nodialog=False):
+        """
+        Entry point into item.
+
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
+        Returns
+        -------
+        bool
+            True if successful, False otherwise.
+
+        """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
+        if not nodialog:
+            temp = self.exec_()
+            if temp == 0:
+                return False
+
+        atype = 'mean'
+        if self.rb_median.isChecked():
+            atype = 'median'
+        if self.rb_rms.isChecked():
+            atype = 'rms'
+
+        self.wsize = self.sb_wsize.value()
+
+        data = copy.deepcopy(self.indata['Raster'])
+        data2 = []
+
+        for datai in data:
+            self.showprocesslog(datai.dataid+':')
+
+            agcdata = agc(datai.data, self.wsize, atype, nodata=datai.nodata,
+                          piter=self.piter)
+            data2.append(copy.deepcopy(datai))
+            data2[-1].data = agcdata
+            data2[-1].dataid += ' AGC'
+
+        self.outdata['Raster'] = data2
+        self.showprocesslog('Finished!')
+
+        return True
+
+    def loadproj(self, projdata):
+        """
+        Load project data into class.
+
+        Parameters
+        ----------
+        projdata : dictionary
+            Project data loaded from JSON project file.
+
+        Returns
+        -------
+        chk : bool
+            A check to see if settings was successfully run.
+
+        """
+        self.sb_wsize.setValue(projdata['wsize'])
+        self.sb_dh.setValue(projdata['vheight'])
+
+        return False
+
+    def saveproj(self):
+        """
+        Save project data from class.
+
+        Returns
+        -------
+        projdata : dictionary
+            Project data to be saved to JSON project file.
+
+        """
+        projdata = {}
+
+        projdata['wsize'] = self.sb_wsize.value()
+        projdata['vheight'] = self.sb_dh.value()
+
+        return projdata
+
+
+def agc(data, wsize, atype='mean', nodata=0., piter=iter):
+    """
+    AGC for map data, based on code by Gordon Cooper.
+
+    Parameters
+    ----------
+    data : numpy array
+        Raster data.
+    wsize : int
+        Window size, must be odd.
+    atype : str, optional
+        AGC type - can be median or mean, default is 'mean'.
+    nodata : float, optional
+        no data value, default is 0.
+    piter : iter
+        Progress bar iterable. The default is iter.
+
+    Returns
+    -------
+    agcdata : numpy array
+        Output AGC data
+    """
+    data = data.copy()-data.min()
+    data = np.abs(data)
+    data = data.astype(float)
+    nr, nc = data.shape
+
+    weight = np.ma.ones((nr, nc))
+    w2 = int(np.floor(wsize/2))
+
+    for i in piter(range(w2, nr-w2)):
+        for j in range(w2, nc-w2):
+            w = data[i-w2:i+w2+1, j-w2:j+w2+1]
+            if atype == 'mean':
+                weight[i, j] = np.ma.mean(w)
+            elif atype == 'median':
+                weight[i, j] = np.ma.median(w)
+            elif atype == 'rms':
+                weight[i, j] = np.ma.sqrt(np.ma.mean(w**2))
+
+    mask = np.ones((nr, nc), dtype=int)
+    mask[w2:-w2, w2:-w2] = 0
+    mask = data.mask | mask
+
+    agcdata = data/weight
+    agcdata.mask = mask
+    agcdata = agcdata.filled(nodata)
+    agcdata = np.ma.array(agcdata, mask=mask)
+
+    return agcdata
+
+
+def _testfn():
+    """Test Reprojection."""
+    import sys
+    from pygmi.raster.iodefs import get_raster
+    import matplotlib.pyplot as plt
+    from pygmi.misc import getinfo
+
+    ifile = r"d:\Workdata\testmag.tif"
+    ifile = r"d:\Workdata\people\mikedentith\perth_surf_win.grd"
+    ifile = r'C:/Workdata/raster/ER Mapper/magmicrolevel.PD.ers'
+
+    piter = ProgressBarText().iter
+
+    dat = get_raster(ifile, piter=piter)
+
+    dat[0].data = dat[0].data.filled(1e+20)
+    dat[0].data = np.ma.masked_equal(dat[0].data, 1e+20)
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    V2D = Visibility2d()
+    V2D.indata['Raster'] = dat
+
+    getinfo()
+    V2D.settings()
+    getinfo()
+
+    odat = V2D.outdata['Raster']
+
+    for idat in odat:
+        plt.figure(dpi=150)
+        plt.title(idat.dataid)
+        plt.imshow(idat.data, extent=idat.extent)
+        plt.colorbar()
+        plt.show()
+
+
+if __name__ == "__main__":
+    _testfn()
