@@ -28,10 +28,12 @@ import copy
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import numexpr as ne
-import pygmi.raster.dataprep as dataprep
+
+from pygmi.misc import BasicModule
+from pygmi.raster import dataprep
 
 
-class EquationEditor(QtWidgets.QDialog):
+class EquationEditor(BasicModule):
     """
     Equation Editor.
 
@@ -41,12 +43,6 @@ class EquationEditor(QtWidgets.QDialog):
 
     Attributes
     ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     equation : str
         string with the equation in it
     bands : dictionary
@@ -57,14 +53,6 @@ class EquationEditor(QtWidgets.QDialog):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.equation = None
         self.bands = {}
 
@@ -73,6 +61,7 @@ class EquationEditor(QtWidgets.QDialog):
         self.textbrowser = QtWidgets.QTextEdit()
         self.textbrowser2 = QtWidgets.QTextBrowser()
         self.label = QtWidgets.QLabel(': iall')
+        self.dtype = QtWidgets.QComboBox()
 
         self.setupui()
 
@@ -87,7 +76,11 @@ class EquationEditor(QtWidgets.QDialog):
         """
         gridlayout = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
+        label_1 = QtWidgets.QLabel('Data Band Key:')
         label_2 = QtWidgets.QLabel('Output Equation:')
+        label_3 = QtWidgets.QLabel('Output Data Type:')
+        self.dtype.addItems(['auto', 'uint8', 'int16', 'int32',
+                             'float32', 'float64'])
 
         self.textbrowser.setEnabled(True)
         self.resize(600, 480)
@@ -111,6 +104,10 @@ class EquationEditor(QtWidgets.QDialog):
                '<h2>Examples</h2>'
                '<p>Sum:</p>'
                '<pre>    i1 + 1000</pre>'
+               '<p>Mean (can be any number of arguments):</p>'
+               '<pre>    mean(i0, i1, i2) or mean(iall)</pre>'
+               '<p>Standard Deviation (can be any number of arguments):</p>'
+               '<pre>    std(i0, i1, i2) or std(iall)</pre>'
                '<p>Mosaic two bands into one:</p>'
                '<pre>    mosaic(i0, i1)</pre>'
                '<p>Threshold between values 1 and 98, substituting -999 as a '
@@ -137,10 +134,13 @@ class EquationEditor(QtWidgets.QDialog):
 
         gridlayout.addWidget(label_2, 0, 0, 1, 1)
         gridlayout.addWidget(self.textbrowser, 1, 0, 1, 2)
+        gridlayout.addWidget(label_1, 3, 0, 1, 1)
         gridlayout.addWidget(self.combobox, 4, 0, 1, 1)
         gridlayout.addWidget(self.label, 4, 1, 1, 1)
-        gridlayout.addWidget(self.textbrowser2, 5, 0, 1, 2)
-        gridlayout.addWidget(buttonbox, 6, 0, 1, 2)
+        gridlayout.addWidget(self.dtype, 6, 0, 1, 1)
+        gridlayout.addWidget(label_3, 5, 0, 1, 1)
+        gridlayout.addWidget(self.textbrowser2, 7, 0, 1, 2)
+        gridlayout.addWidget(buttonbox, 8, 0, 1, 2)
 
         self.combobox.currentIndexChanged.connect(self.combo)
         buttonbox.accepted.connect(self.accept)
@@ -177,10 +177,100 @@ class EquationEditor(QtWidgets.QDialog):
         neweq = str(self.equation)
         neweq = neweq.replace('ln', 'log')
         neweq = neweq.replace('^', '**')
-        neweq = neweq.replace('nodata', str(indata[0].nullvalue))
+        neweq = neweq.replace('nodata', str(indata[0].nodata))
 
         return neweq
 
+    def mean(self, eq, localdict):
+        """
+        Get mean pixel value of all input bands.
+
+        Parameters
+        ----------
+        eq : str
+            Equation with std command.
+        localdict : dictionary
+            Dictionary of data.
+
+        Returns
+        -------
+        findat : numpy array
+            Output array.
+
+        """
+        idx = eq.index('mean(')+5
+        eq2 = eq[idx:]
+        idx = eq2.index(')')
+        eq2 = eq2[:idx]
+        eq2 = eq2.replace(' ', '')
+        eq3 = eq2.split(',')
+
+        stack = []
+        mask = None
+        for i in localdict:
+            if i not in eq3:
+                continue
+            if mask is None:
+                mask = localdict[i].mask
+            else:
+                mask = np.logical_and(mask, localdict[i].mask)
+
+            if i == 'iall':
+                stack.append(localdict[i])
+            else:
+                stack.append([localdict[i]])
+
+        stack = np.ma.vstack(stack)
+        findat = np.ma.mean(stack, 0)
+        findat.mask = mask
+
+        return findat
+
+    def std(self, eq, localdict):
+        """
+        Get standard deviation pixel value of all input bands.
+
+        Parameters
+        ----------
+        eq : str
+            Equation with std command.
+        localdict : dictionary
+            Dictionary of data.
+
+        Returns
+        -------
+        findat : numpy array
+            Output array.
+
+        """
+        idx = eq.index('std(')+4
+        eq2 = eq[idx:]
+        idx = eq2.index(')')
+        eq2 = eq2[:idx]
+        eq2 = eq2.replace(' ', '')
+        eq3 = eq2.split(',')
+
+        stack = []
+        mask = None
+        for i in localdict:
+            if i not in eq3:
+                continue
+            if mask is None:
+                mask = localdict[i].mask
+            else:
+                mask = np.logical_and(mask, localdict[i].mask)
+
+            if i == 'iall':
+                stack.append(localdict[i])
+            else:
+                stack.append([localdict[i]])
+
+        stack = np.ma.vstack(stack)
+        findat = np.ma.std(stack, 0)
+        findat.mask = mask
+
+        return findat
+
     def mosaic(self, eq, localdict):
         """
         Mosaics data into a single band dataset.
@@ -235,7 +325,6 @@ class EquationEditor(QtWidgets.QDialog):
                 return None
             eq4[-1] = np.ma.array(eq4[-1], mask=mask[-1])
 
-        # master = np.ma.mean(eq4, 0)
         master = eq4.pop()
         for i in eq4[::-1]:
             master[~i.mask] = i.data[~i.mask]
@@ -248,8 +337,8 @@ class EquationEditor(QtWidgets.QDialog):
 
         Parameters
         ----------
-        equation : str, optional
-            Equation submitted (for testing). The default is None.
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
@@ -270,10 +359,10 @@ class EquationEditor(QtWidgets.QDialog):
         elif 'Raster' in self.indata:
             intype = 'Raster'
         else:
-            self.showprocesslog('No raster data')
+            self.showprocesslog('No raster data.')
             return False
 
-        indata = dataprep.merge(self.indata[intype])
+        indata = dataprep.lstack(self.indata[intype])
 
         for j, i in enumerate(indata):
             self.combobox.addItem(i.dataid)
@@ -314,25 +403,26 @@ class EquationEditor(QtWidgets.QDialog):
 
         if 'mosaic' in neweq:
             findat = self.mosaic(neweq, localdict)
-            if findat is None:
-                QtWidgets.QMessageBox.warning(
-                    self.parent, 'Error',
-                    'Nothing processed! '
-                    'Your equation most likely had an error.',
-                    QtWidgets.QMessageBox.Ok)
-                return False
             mask = findat.mask
-
+        elif 'mean' in neweq:
+            findat = self.mean(neweq, localdict)
+            mask = findat.mask
+        elif 'std' in neweq:
+            findat = self.std(neweq, localdict)
+            mask = findat.mask
         else:
             try:
                 findat = ne.evaluate(neweq, localdict)
             except Exception:
-                QtWidgets.QMessageBox.warning(
-                    self.parent, 'Error',
-                    'Nothing processed! '
-                    'Your equation most likely had an error.',
-                    QtWidgets.QMessageBox.Ok)
-                return False
+                findat = None
+
+        if findat is None:
+            QtWidgets.QMessageBox.warning(
+                self.parent, 'Error',
+                'Nothing processed! '
+                'Your equation most likely had an error.',
+                QtWidgets.QMessageBox.Ok)
+            return False
 
         outdata = []
 
@@ -347,17 +437,21 @@ class EquationEditor(QtWidgets.QDialog):
             findat.shape = (1, findat.shape[0], findat.shape[1])
 
         for i, findati in enumerate(findat):
-            findati[mask] = indata[i].nullvalue
+            findati = findati.astype(indata[i].data.dtype)
+            findati[mask] = indata[i].nodata
 
-            outdata.append(copy.copy(indata[i]))
+            outdata.append(copy.deepcopy(indata[i]))
             outdata[-1].data = np.ma.masked_equal(findati,
-                                                  indata[i].nullvalue)
-            outdata[-1].nullvalue = indata[i].nullvalue
+                                                  indata[i].nodata)
+            outdata[-1].nodata = indata[i].nodata
 
+        dtype = self.dtype.currentText()
         # This is needed to get rid of bad, unmasked values etc.
         for i, outdatai in enumerate(outdata):
-            outdatai.data.set_fill_value(indata[i].nullvalue)
+            outdatai.data.set_fill_value(indata[i].nodata)
             outdatai.data = np.ma.fix_invalid(outdatai.data)
+            if dtype != 'auto':
+                outdatai.data = outdatai.data.astype(dtype)
 
         if len(outdata) == 1:
             outdata[0].dataid = self.equation
@@ -429,3 +523,33 @@ def hmode(data):
     mode2 = (mmax-mmin)/2 + mmin
 
     return mode2
+
+
+def _test():
+    """Test."""
+    import sys
+    import matplotlib.pyplot as plt
+    from pygmi.raster.iodefs import get_raster
+    print('Starting')
+
+    ifile = r"C:\Workdata\testdata.hdr"
+
+    dat = get_raster(ifile)
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    EE = EquationEditor()
+    EE.indata['Raster'] = dat
+
+    EE.settings()
+
+
+    out = EE.outdata['Raster']
+
+    plt.figure(dpi=150)
+    plt.imshow(out[0].data)
+    plt.show()
+
+
+if __name__ == "__main__":
+    _test()
