@@ -36,13 +36,13 @@ menu. The following are supported:
 
 import numpy as np
 from PyQt5 import QtWidgets, QtCore
-from matplotlib import cm
+from matplotlib import colormaps
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 import matplotlib.colors as mcolors
 
-from pygmi.misc import frm
+from pygmi.misc import frm, ContextModule
 from pygmi.raster.modest_image import imshow
 
 
@@ -78,15 +78,26 @@ class MyMplCanvas(FigureCanvasQTAgg):
         None.
 
         """
+        cmap = colormaps['viridis']
+
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
-        self.axes.pcolor(dmat)
+        rdata = self.axes.pcolormesh(dmat, cmap=cmap)
         self.axes.axis('scaled')
         self.axes.set_title('Correlation Coefficients')
         for i in range(len(data1)):
             for j in range(len(data1)):
-                self.axes.text(i + .1, j + .4, format(float(dmat[i, j]),
-                                                      '4.2f'))
+                ctmp = np.array([1., 1., 1., 0.]) - np.array(cmap(dmat[i, j]))
+                ctmp = np.abs(ctmp)
+                ctmp = ctmp.tolist()
+
+                if dmat[i, j] < 0.01:
+                    atext = f'{dmat[i, j]:.1e}'
+                else:
+                    atext = f'{dmat[i, j]:.2f}'
+
+                self.axes.text(i+.5, j+.5, atext, c=ctmp, rotation=45,
+                               ha='center', va='center')
         dat_mat = [i.dataid for i in data1]
         self.axes.set_xticks(np.array(list(range(len(data1)))) + .5)
 
@@ -97,6 +108,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes.set_xlim(0, len(data1))
         self.axes.set_ylim(0, len(data1))
 
+        cbar = self.figure.colorbar(rdata, format=frm)
+
         self.figure.tight_layout()
         self.figure.canvas.draw()
 
@@ -119,25 +132,25 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
 
-        vstd = 2.5 * data1.data.std()
-        vmean = data1.data.mean()
-        vmin = vmean-vstd
-        vmax = vmean+vstd
-
         rdata = imshow(self.axes, data1.data, extent=data1.extent,
-                       cmap=cm.get_cmap(cmap), vmin=vmin, vmax=vmax,
-                       interpolation='nearest')
-
-        # rdata = self.axes.imshow(data1.data, extent=data1.extent,
-        #                          cmap=cm.get_cmap(cmap), vmin=vmin, vmax=vmax,
-        #                          interpolation='nearest')
+                       cmap=colormaps[cmap], interpolation='nearest')
 
         if not data1.isrgb:
+            rdata.set_clim_std(2.5)
             cbar = self.figure.colorbar(rdata, format=frm)
             cbar.set_label(data1.units)
 
-        self.axes.set_xlabel('Eastings')
-        self.axes.set_ylabel('Northings')
+        if data1.crs is not None:
+            crs = data1.crs.to_dict()
+        else:
+            crs = {}
+
+        if 'proj' in crs and crs['proj'] == 'longlat':
+            self.axes.set_xlabel('Longitude')
+            self.axes.set_ylabel('Latitude')
+        else:
+            self.axes.set_xlabel('Eastings')
+            self.axes.set_ylabel('Northings')
 
         self.axes.xaxis.set_major_formatter(frm)
         self.axes.yaxis.set_major_formatter(frm)
@@ -163,8 +176,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
-        x = data1.copy()
-        y = data2.copy()
+        x = data1.data.copy()
+        y = data2.data.copy()
 
         msk = np.logical_or(x.mask, y.mask)
         x.mask = msk
@@ -186,6 +199,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes.xaxis.set_major_formatter(frm)
         self.axes.yaxis.set_major_formatter(frm)
 
+        self.axes.set_xlabel(data1.units)
+        self.axes.set_ylabel(data2.units)
+
         self.figure.tight_layout()
         self.figure.canvas.draw()
 
@@ -220,7 +236,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         x = np.ma.array(x, mask=z.mask)
         y = np.ma.array(y, mask=z.mask)
 
-        cmap = cm.get_cmap(cmap)
+        cmap = colormaps[cmap]
 
         norml = mcolors.Normalize(vmin=z.min(), vmax=z.max())
 
@@ -250,7 +266,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
     def update_hist(self, data1, ylog):
         """
-        Update the hiostogram plot.
+        Update the histogram plot.
 
         Parameters
         ----------
@@ -283,23 +299,11 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class PlotCCoef(QtWidgets.QDialog):
-    """
-    Plot 2D Correlation Coefficients.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class PlotCCoef(ContextModule):
+    """Plot 2D Correlation Coefficients."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -310,6 +314,8 @@ class PlotCCoef(QtWidgets.QDialog):
         vbl.addWidget(self.mmc)
         vbl.addWidget(mpl_toolbar)
 
+        self.setMinimumSize(600, 600)
+
         self.setFocus()
 
     def run(self):
@@ -339,23 +345,11 @@ class PlotCCoef(QtWidgets.QDialog):
         self.mmc.update_ccoef(data, dummy_mat)
 
 
-class PlotRaster(QtWidgets.QDialog):
-    """
-    Plot Raster Class.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class PlotRaster(ContextModule):
+    """Plot Raster Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -417,23 +411,11 @@ class PlotRaster(QtWidgets.QDialog):
             self.combobox1.addItem(i.dataid)
 
 
-class PlotSurface(QtWidgets.QDialog):
-    """
-    Plot Surface Class.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class PlotSurface(ContextModule):
+    """Plot Surface Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -495,23 +477,15 @@ class PlotSurface(QtWidgets.QDialog):
             self.change_band()
 
 
-class PlotScatter(QtWidgets.QDialog):
+class PlotScatter(ContextModule):
     """
-    Plot Hexbin Class. A Hexbin is a type of scatter plot which is raster.
+    Plot Hexbin Class.
 
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
+    A Hexbin is a type of scatter plot which is raster.
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -551,9 +525,9 @@ class PlotScatter(QtWidgets.QDialog):
         i = self.combobox1.currentIndex()
         j = self.combobox2.currentIndex()
 
-        x = data[i].data
-        y = data[j].data
-        if x.mask.shape != y.mask.shape:
+        x = data[i]
+        y = data[j]
+        if x.data.shape != y.data.shape:
             QtWidgets.QMessageBox.warning(self, 'Warning',
                                           'Different size input datasets. '
                                           'Merge and resample your input data '
@@ -582,23 +556,11 @@ class PlotScatter(QtWidgets.QDialog):
         self.combobox2.setCurrentIndex(1)
 
 
-class PlotHist(QtWidgets.QDialog):
-    """
-    Plot Histogram Class.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class PlotHist(ContextModule):
+    """Plot Histogram Class."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -697,9 +659,14 @@ def corr2d(dat1, dat2):
         array of correlation coefficients
     """
     out = None
+
+    # These next two lines are critical to keep original data safe.
+    dat1 = dat1.copy()
+    dat2 = dat2.copy()
+
     if dat1.shape == dat2.shape:
         # These line are to avoid warnings due to powers of large fill values
-        mask = np.logical_or(dat1.mask, dat1.mask)
+        mask = np.logical_or(dat1.mask, dat2.mask)
         dat1.mask = mask
         dat2.mask = mask
         dat1 = dat1.compressed()
@@ -712,3 +679,26 @@ def corr2d(dat1, dat2):
         out = numerator / denominator
 
     return out
+
+
+def _testfn():
+    """Test."""
+    from pygmi.raster.iodefs import get_raster
+    import matplotlib.pyplot as plt
+
+    ifile = r"d:/Workdata/LULC/2001_stack_norm.tif"
+
+    data = get_raster(ifile)
+
+    cormat = np.array([[corr2d(i.data, j.data) for j in data] for i in data])
+
+    print(cormat)
+
+    plt.figure(dpi=150)
+    plt.imshow(cormat, cmap='jet')
+    plt.colorbar()
+    plt.show()
+
+
+if __name__ == "__main__":
+    _testfn()
