@@ -29,27 +29,19 @@ import warnings
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
 from pygmi.raster.ginterp import histeq
-
+from pygmi.misc import BasicModule
 
 warnings.simplefilter('always', RuntimeWarning)
 
 
-class Normalisation(QtWidgets.QDialog):
+class Normalisation(BasicModule):
     """Class Normalisation."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-        if parent is not None:
-            self.pbar = parent.pbar
-        else:
-            self.pbar = None
-
         self.radiobutton_interval = QtWidgets.QRadioButton('Interval [0 1]')
         self.radiobutton_mean = QtWidgets.QRadioButton('Mean: zero,  '
                                                        'Standard deviation: '
@@ -106,37 +98,43 @@ class Normalisation(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         if not nodialog:
             temp = self.exec_()
             if temp == 0:
                 return False
 
         data = copy.deepcopy(self.indata['Raster'])
-        # transform = np.zeros((2, 2))
+
         if self.radiobutton_interval.isChecked():
             for i in data:
                 tmp1 = i.data.min()
                 tmp2 = i.data.max() - i.data.min()
-                tmp3 = 'minmax'
-                i, transform = datacommon(i, tmp1, tmp2, tmp3)
+                i, _ = datacommon(i, tmp1, tmp2)
         elif self.radiobutton_mean.isChecked():
             for i in data:
                 tmp1 = i.data.mean()
                 tmp2 = i.data.std()
-                tmp3 = 'meanstd'
-                i, transform = datacommon(i, tmp1, tmp2, tmp3)
+                i, _ = datacommon(i, tmp1, tmp2)
         elif self.radiobutton_median.isChecked():
             for i in data:
                 tmp1 = np.median(i.data.compressed())
                 tmp2 = np.median(abs(i.data.compressed() - tmp1))
-                tmp3 = 'medmad'
-                i, transform = datacommon(i, tmp1, tmp2, tmp3)
+                i, _ = datacommon(i, tmp1, tmp2)
         elif self.radiobutton_8bit.isChecked():
             for i in data:
                 i.data = histeq(i.data)
@@ -144,7 +142,7 @@ class Normalisation(QtWidgets.QDialog):
 
         # Correct the null value
         for i in data:
-            i.data.data[i.data.mask] = i.nullvalue
+            i.data.data[i.data.mask] = i.nodata
 
         self.outdata['Raster'] = data
         if self.pbar is not None:
@@ -201,7 +199,7 @@ class Normalisation(QtWidgets.QDialog):
         return projdata
 
 
-def datacommon(data, tmp1, tmp2, tmp3):
+def datacommon(data, tmp1, tmp2):
     """
     Variables used in the process routine.
 
@@ -213,8 +211,6 @@ def datacommon(data, tmp1, tmp2, tmp3):
         Parameter 1. Can be min, mean or median.
     tmp2 : float
         Parameter 2. Can be range, std, or mad.
-    tmp3 : str
-        Text label. Can be 'minmax', 'meanstd' or 'medmad'.
 
     Returns
     -------
@@ -235,6 +231,5 @@ def datacommon(data, tmp1, tmp2, tmp3):
         dtmp /= tmp2
 
         data.data = np.ma.array(dtmp, mask=mtmp)
-#        n_norms = len(data.norm)
-#        data.norm[n_norms] = {'type': tmp3, 'transform': transform}
+
     return data, transform
