@@ -29,26 +29,16 @@ import warnings
 from PyQt5 import QtWidgets
 import numpy as np
 import scipy.signal as ssig
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
+from pygmi.misc import BasicModule
 
 
-class Smooth(QtWidgets.QDialog):
+class Smooth(BasicModule):
     """Smooth."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
 
         self.label = QtWidgets.QLabel('X:')
         self.spinbox_x = QtWidgets.QSpinBox()
@@ -149,12 +139,21 @@ class Smooth(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
             True if successful, False otherwise.
 
         """
+        if 'Raster' not in self.indata:
+            self.showprocesslog('No Raster Data.')
+            return False
+
         if not nodialog:
             temp = self.exec_()
             if temp == 0:
@@ -166,12 +165,14 @@ class Smooth(QtWidgets.QDialog):
         if self.radiobutton_2dmean.isChecked():
             for i, _ in enumerate(data):
                 data[i].data = self.mov_win_filt(data[i].data, self.fmat,
-                                                 '2D Mean', data[i].dataid)
+                                                 '2D Mean')
+                data[i].dataid = data[i].dataid+' 2D Mean'
 
         if self.radiobutton_2dmedian.isChecked():
             for i, _ in enumerate(data):
                 data[i].data = self.mov_win_filt(data[i].data, self.fmat,
-                                                 '2D Median', data[i].dataid)
+                                                 '2D Median')
+                data[i].dataid = data[i].dataid+' 2D Median'
 
         if not nodialog:
             self.parent.process_is_active(False)
@@ -343,7 +344,7 @@ class Smooth(QtWidgets.QDialog):
                     i = 127
                 else:
                     i = int(255*(self.fmat[row, col]-fmin)/frange)
-                ltmp = QtWidgets.QLabel('{:g}'.format(self.fmat[row, col]))
+                ltmp = QtWidgets.QLabel(f'{self.fmat[row, col]:g}')
                 ltmp.setStyleSheet(
                     'Background: rgb' +
                     str((red[i], green[i], blue[i])) + '; Color: rgb' +
@@ -371,24 +372,22 @@ class Smooth(QtWidgets.QDialog):
         QtWidgets.QMessageBox.warning(self.parent, title, message,
                                       QtWidgets.QMessageBox.Ok)
 
-    def mov_win_filt(self, dat, fmat, itype, title):
+    def mov_win_filt(self, dat, fmat, itype):
         """
         Apply moving window filter function to data.
 
         Parameters
         ----------
-        dat : numpy array.
+        dat : numpy masked array.
             Data for a PyGMI raster dataset.
-        fmat : TYPE
-            DESCRIPTION.
+        fmat : numpy array
+            Filter matrix.
         itype : str
             Filter type. Can be '2D Mean' or '2D Median'.
-        title : str
-            Text for reportback function.
 
         Returns
         -------
-        out : numpy array
+        out : numpy masked array
             Data for a PyGMI raster dataset.
 
         """
@@ -401,10 +400,8 @@ class Smooth(QtWidgets.QDialog):
         drr = round(rowf/2.0)
         dcc = round(colf/2.0)
 
-        dummy = np.zeros((rowd+rowf-1, cold+colf-1))*np.nan
+        dummy = np.ma.masked_all((rowd+rowf-1, cold+colf-1))
         dummy[drr-1:drr-1+rowd, dcc-1:dcc-1+cold] = dat
-        dummy = np.ma.masked_invalid(dummy)
-        dummy.mask[drr-1:drr-1+rowd, dcc-1:dcc-1+cold] = dat.mask
 
         dummy.data[dummy.mask] = np.nan
         dat = dat.astype(float)
@@ -421,14 +418,14 @@ class Smooth(QtWidgets.QDialog):
             dummy = dummy.data
 
             for i in self.piter(range(rowd)):
-                # self.showprocesslog(title+' Progress: ' +
-                #                     str(round(100*i/rowd))+'%', True)
                 for j in range(cold):
                     tmp1 = dummy[i:i+rowf, j:j+colf][fmat]
-                    if np.isnan(tmp1).min() == False:
+                    if np.any(~np.isnan(tmp1)):  # if any are False
                         out[i, j] = np.nanmedian(tmp1)
 
         out = np.ma.masked_invalid(out)
+        out = np.ma.array(out)
+
         out.shape = out.shape[0:2]
         out.mask[:rowf//2] = True
         out.mask[-rowf//2:] = True
@@ -463,7 +460,7 @@ def filters2d(filtertype, sze, *sigma):
 
     FSPECIAL - Create spatial filters for image processing
 
-    Usage:  f = fspecial(filtertype, optionalparams)
+    Usage:  f = fspecial(filtertype, optional parameters)
 
     filtertype can be
 
