@@ -26,7 +26,7 @@
 
 import datetime
 from pathlib import Path
-import xml.etree.ElementTree as ElementTree
+from xml.etree import ElementTree
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import pandas as pd
@@ -34,52 +34,28 @@ from osgeo import gdal, osr, ogr
 from shapely.geometry.polygon import Polygon
 import matplotlib.pyplot as plt
 from matplotlib.figure import Figure
+from matplotlib import colormaps
 import matplotlib.animation as manimation
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 
-# import pygmi.menu_default as menu_default
 from pygmi.raster.datatypes import Data
 from pygmi.misc import frm
 from pygmi.raster.ginterp import histcomp, norm255
-from pygmi.raster.modest_image import imshow
+from pygmi.misc import BasicModule
 
 
-class CreateSceneList(QtWidgets.QDialog):
+class CreateSceneList(BasicModule):
     """
     Create Scene List.
 
     This class creates a list of scenes for use in change detection.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    piter : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
-        self.parent = parent
         self.indata = {'tmp': True}
-        self.outdata = {}
-        self.ifile = ''
-        if parent is not None:
-            self.piter = self.parent.pbar.iter
-        else:
-            self.piter = iter
 
         self.shapefile = QtWidgets.QLineEdit('')
         self.scenefile = QtWidgets.QLineEdit('')
@@ -99,7 +75,7 @@ class CreateSceneList(QtWidgets.QDialog):
         """
         gridlayout_main = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
-        # helpdocs = menu_default.HelpButton('pygmi.grav.iodefs.importpointdata')
+        # helpdocs = menu_default.HelpButton('pygmi.rsense.change.cscene')
         pb_shape = QtWidgets.QPushButton('Load shapefile or kml file')
         pb_scene = QtWidgets.QPushButton('Set scene directory')
 
@@ -118,7 +94,6 @@ class CreateSceneList(QtWidgets.QDialog):
         gridlayout_main.addWidget(self.useall, 2, 0, 1, 2)
         gridlayout_main.addWidget(self.isrecursive, 3, 0, 1, 2)
 
-#        gridlayout_main.addWidget(helpdocs, 5, 0, 1, 1)
         gridlayout_main.addWidget(buttonbox, 5, 1, 1, 3)
 
         buttonbox.accepted.connect(self.accept)
@@ -130,6 +105,11 @@ class CreateSceneList(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -320,36 +300,21 @@ class CreateSceneList(QtWidgets.QDialog):
         self.scenefile.setText(directory)
 
 
-class LoadSceneList():
-    """
-    Load scene list.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
-    ext : str
-        filename extension
-    """
+class LoadSceneList(BasicModule):
+    """Load scene list."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -436,7 +401,14 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.fig.canvas.mpl_connect('button_release_event', self.onClick)
 
     def capture(self):
-        """Capture."""
+        """
+        Capture.
+
+        Returns
+        -------
+        None.
+
+        """
         self.capture_active = not self.capture_active
 
         if self.capture_active:
@@ -448,7 +420,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
                 return
 
             self.writer = manimation.PillowWriter(fps=4)
-            self.writer.setup(self.fig, wfile)  # , 100)
+            self.writer.setup(self.fig, wfile)
         else:
             self.writer.finish()
 
@@ -489,7 +461,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         extent = dat[self.bands[0]].extent
 
-        # self.im1 = imshow(self.ax1, dtmp, extent=extent)
         self.im1 = self.ax1.imshow(dtmp, extent=extent)
         self.ax1.plot(points[:, 0], points[:, 1])
         self.cbar = None
@@ -522,7 +493,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
             nir = nir.astype(float)
             dtmp = (green-nir)/(green+nir)
             self.im1.set_clim(-1, 1)
-            self.im1.set_cmap(plt.cm.get_cmap('PiYG_r'))
+            self.im1.set_clim(-1, 1)
+            self.im1.set_cmap(colormaps['PiYG_r'])
         elif self.manip == 'NDVI':
             if self.cbar is None:
                 self.cbar = self.figure.colorbar(self.im1, format=frm)
@@ -532,16 +504,13 @@ class MyMplCanvas(FigureCanvasQTAgg):
             nir = nir.astype(float)
             dtmp = (nir-red)/(nir+red)
             self.im1.set_clim(-1, 1)
-            self.im1.set_cmap(plt.cm.get_cmap('PiYG'))
+            self.im1.set_clim(-1, 1)
+            self.im1.set_cmap(colormaps('PiYG'))
         else:
             if self.cbar is not None:
                 self.cbar.remove()
                 self.cbar = None
 
-            # rtmp1 = dat[self.bands[2]].data
-            # rtmp2 = dat[self.bands[1]].data
-            # rtmp3 = dat[self.bands[0]].data
-
             mask = (dat[self.bands[2]].data == 0.)
             red = np.ma.array(dat[self.bands[2]].data, mask=mask)
             green = np.ma.array(dat[self.bands[1]].data, mask=mask)
@@ -559,25 +528,14 @@ class MyMplCanvas(FigureCanvasQTAgg):
             green[mask] = 0
             blue[mask] = 0
 
-            # rtmp1 = (rtmp1-rtmp1.min())
-            # rtmp2 = (rtmp2-rtmp2.min())
-            # rtmp3 = (rtmp3-rtmp3.min())
-
-            # if rtmp1.ptp() != 0.:
-            #     rtmp1 = rtmp1/rtmp1.ptp()
-            # if rtmp2.ptp() != 0.:
-            #     rtmp2 = rtmp2/rtmp2.ptp()
-            # if rtmp3.ptp() != 0.:
-            #     rtmp3 = rtmp3/rtmp3.ptp()
-
-            alpha = ~mask  # np.logical_not(red == 0.)
+            alpha = ~mask
             alpha = alpha*255
 
             dtmp = np.array([red, green, blue, alpha])
             dtmp = np.moveaxis(dtmp, 0, 2)
-            # dtmp = dtmp*255
             dtmp = dtmp.astype(np.uint8)
             self.im1.set_clim(0, 255)
+            self.im1.set_clim(0, 255)
 
         self.im1.set_data(dtmp)
         self.im1.set_extent(extent)
@@ -621,30 +579,15 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.parent.newdata(self.parent.curimage)
 
 
-class SceneViewer(QtWidgets.QDialog):
+class SceneViewer(BasicModule):
     """Scene Viewer."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.ifile = ''
 
         self.df = None
-        if parent is None:
-            self.piter = iter
-        else:
-            self.piter = self.parent.pbar.iter
+        self.pbar2 = QtWidgets.QProgressBar()
 
-        self.pbar = QtWidgets.QProgressBar()
-
-        # self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle("View Change Data")
 
         self.file_menu = QtWidgets.QMenu('&File', self)
@@ -686,7 +629,7 @@ class SceneViewer(QtWidgets.QDialog):
         vlayout.addWidget(self.cb_use)
         vlayout.addLayout(hlayout2)
         vlayout.addLayout(hlayout)
-        vlayout.addWidget(self.pbar)
+        vlayout.addWidget(self.pbar2)
 
         self.curimage = 0
 
@@ -702,6 +645,11 @@ class SceneViewer(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -709,6 +657,7 @@ class SceneViewer(QtWidgets.QDialog):
 
         """
         if 'SceneList' not in self.indata:
+            self.showprocesslog('You need a scene list.')
             return False
 
         self.df = self.indata['SceneList']
@@ -760,8 +709,6 @@ class SceneViewer(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
     def manip_change(self, event):
@@ -958,9 +905,9 @@ class SceneViewer(QtWidgets.QDialog):
 
         dataset = gdal.Open(ifile, gdal.GA_ReadOnly)
 
-        self.pbar.setMinimum(0)
-        self.pbar.setValue(0)
-        self.pbar.setMaximum(dataset.RasterCount-1)
+        self.pbar2.setMinimum(0)
+        self.pbar2.setValue(0)
+        self.pbar2.setMaximum(dataset.RasterCount-1)
 
         gtr = dataset.GetGeoTransform()
         cols = dataset.RasterXSize
@@ -970,7 +917,6 @@ class SceneViewer(QtWidgets.QDialog):
         dxlim = (gtr[0], gtr[0]+gtr[1]*cols)
         dylim = (gtr[3]+gtr[5]*rows, gtr[3])
 
-###############################################################################
         axes = self.canvas.ax1
 
         if firstrun is True:
@@ -997,10 +943,6 @@ class SceneViewer(QtWidgets.QDialog):
         xbuf = min(xsize, int(ext[0]))
         ybuf = min(ysize, int(ext[1]))
 
-        gtrnew = (gtr[0]+xoff*dx, xdim, 0, gtr[3]-yoff*dy, 0, -ydim)
-
-###############################################################################
-
         for i in range(dataset.RasterCount):
 
             rtmp = dataset.GetRasterBand(i+1)
@@ -1029,14 +971,13 @@ class SceneViewer(QtWidgets.QDialog):
                     nval = 1e+20
                 nval = float(nval)
 
-            dat.nullvalue = nval
+            dat.nodata = nval
             dat.xdim = xdim
             dat.ydim = ydim
-            dat.extent_from_gtr(gtrnew)
             dat.wkt = dataset.GetProjection()
             datall[i+1] = dat
 
-            self.pbar.setValue(i)
+            self.pbar2.setValue(i)
 
         if datall == {}:
             datall = None
@@ -1142,11 +1083,10 @@ def get_kml_coords(kml):
 def _testfn():
     """Test routine."""
     import sys
-#    sfile = r'C:\Work\Workdata\change\PlanetaryPolygon.shp'
-    sfile = r'C:\Work\Workdata\change\fl35.shp'
-    pdir = r'C:\Work\Workdata\change\Planet'
+    sfile = r'd:\Work\Workdata\change\fl35.shp'
+    pdir = r'd:\Work\Workdata\change\Planet'
 
-    APP = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
 
     CSL = CreateSceneList(None)
     CSL.isrecursive.setChecked(True)
@@ -1159,48 +1099,23 @@ def _testfn():
 
 def _testanim():
     """Test for animation."""
-    # import sys
-    from matplotlib import rcParams
-    # rcParams['axes.formatter.limits'] = [-12, 12]
-    # rcParams['axes.formatter.useoffset'] = False
-    wfile = r'C:\Work\Workdata\change\tmp.gif'
+    wfile = r'd:\Work\Workdata\change\tmp.gif'
 
-    rcParams['figure.dpi'] = 300
-    # rcParams['savefig.dpi'] = 300
-
-    # APP = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
-    fig = plt.figure()
+    fig = plt.figure(dpi=150)
 
     writer = manimation.PillowWriter(fps=4)
-    writer.setup(fig, wfile)  # , 100)
+    writer.setup(fig, wfile)
 
     tmp = np.random.rand(100, 100)
     im = plt.imshow(tmp)
     for i in range(20):
         red = np.random.rand(100, 100)
-        # green = np.random.rand(100, 100)
-        # blue = np.random.rand(100, 100)
-        # alpha = np.logical_not(red == 0.)
-        # dtmp = np.array([red, green, blue, alpha])
-        # dtmp = np.moveaxis(dtmp, 0, 2)
-        # dtmp = dtmp*255
-        # dtmp = dtmp.astype(np.uint8)
-        # im.set_clim(0, 255)
-
         im.set_data(red)
         fig.suptitle(str(i))
         writer.grab_frame()
     plt.show()
     writer.finish()
 
-    # CSL = LoadSceneList(None)
-    # CSL.ifile = r'C:\Work\Workdata\change\Planet\paddock.xlsx'
-    # CSL.settings(True)
-
-    # SV = SceneViewer()
-    # SV.indata = CSL.outdata
-    # SV.settings()
-
 
 if __name__ == "__main__":
     _testanim()
