@@ -22,59 +22,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # -----------------------------------------------------------------------------
-"""
-Hyperspectral Interpretation Routines.
-
-1) Spectral Feature Examination
-2) Spectral Interpretation and Processing
-3) Borehole display and interpretation
-
-Spectral feature examination is a GUI which allows for the comparison of
-spectra from the dataset with library spectra.
-
-Must be able to:
-
-1) Zoom into features
-2) Select features and apply settings such as threshold if necessary
-3) Combine multiple features into final filtered result.
-4) Save spectra into a library for feature examination now or later.
-5) Features can be depths, widths, or ratios
-6) Output successful feature combinations into formulae for processing.
-
-Note: feature or ratio x unit is wavelength and not band number.
-
-Spectral Interpretation and processing allows for processing of a scene.
-1) Must have a list of features or filters and thresholds
-2) Must have a 'quick name' dropdown for premade combinations.
-3) Must be able to save a combination
-4) Must be able to import combinations
-
-There should be a library file for features or filters
-There should be a combo file for combinations.
-
-Borehole display and interpretation shows a list of trays and can display core
-as one borehole. It can also allow for manual interpretation as well as
-comparison between existing logs and the core images.
-
-"""
+"""Hyperspectral Interpretation Routines."""
 import json
 import sys
 import re
+import os
 
 import numpy as np
 import numexpr as ne
+from numba import jit
 from PyQt5 import QtWidgets, QtCore
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 import matplotlib.patches as mpatches
+import matplotlib.pyplot as plt
 
-from pygmi.misc import frm
-import pygmi.menu_default as menu_default
+from pygmi.misc import frm, BasicModule
+from pygmi import menu_default
 from pygmi.raster.iodefs import get_raster
-from pygmi.misc import ProgressBarText
 from pygmi.raster.datatypes import numpy_to_pygmi
-from pygmi.raster.modest_image import imshow
+from pygmi.raster.iodefs import export_raster
+from pygmi.rsense import features
 
 
 class GraphMap(FigureCanvasQTAgg):
@@ -107,6 +76,8 @@ class GraphMap(FigureCanvasQTAgg):
         self.spectra = None
         self.refl = 1.
         self.rotate = False
+        self.nodata = 0.
+        self.ax1 = None
 
     def init_graph(self):
         """
@@ -117,27 +88,27 @@ class GraphMap(FigureCanvasQTAgg):
         None.
 
         """
-        dat = self.datarr[self.mindx]/self.refl
+        dat = self.datarr[self.mindx]
+
+        if self.refl != 1.:
+            dat = dat/self.refl
 
         rows, cols = dat.shape
 
         self.figure.clf()
         ax1 = self.figure.add_subplot(211)
+        self.ax1 = ax1
 
         ymin = dat.mean()-2*dat.std()
         ymax = dat.mean()+2*dat.std()
 
-        # if self.rotate is True:
-        #     self.csp = imshow(ax1, dat.T, vmin=ymin, vmax=ymax)
-        #     rows, cols = cols, rows
-        # else:
-        #     self.csp = imshow(ax1, dat, vmin=ymin, vmax=ymax)
-
         if self.rotate is True:
-            self.csp = ax1.imshow(dat.T, vmin=ymin, vmax=ymax)
+            self.csp = ax1.imshow(dat.T, vmin=ymin, vmax=ymax,
+                                  interpolation='none')
             rows, cols = cols, rows
         else:
-            self.csp = ax1.imshow(dat, vmin=ymin, vmax=ymax)
+            self.csp = ax1.imshow(dat, vmin=ymin, vmax=ymax,
+                                  interpolation='none')
 
         ax1.set_xlim((0, cols))
         ax1.set_ylim((0, rows))
@@ -150,7 +121,12 @@ class GraphMap(FigureCanvasQTAgg):
         ax1.plot(self.col, self.row, '+w')
 
         ax2 = self.figure.add_subplot(212)
-        prof = self.datarr[:, self.row, self.col]/self.refl
+
+        prof = []
+        for i in self.datarr:
+            prof.append(i[self.row, self.col])
+
+        prof = np.ma.stack(prof)/self.refl
 
         ax2.format_coord = lambda x, y: f'Wavelength: {x:1.2f}, Y: {y:1.2f}'
         ax2.grid(True)
@@ -184,7 +160,7 @@ class GraphMap(FigureCanvasQTAgg):
         bmax = self.feature[2]
 
         rect = mpatches.Rectangle((bmin, zmin), bmax-bmin, zmax-zmin)
-        rect.set_fc([0, 1, 0])
+        rect.set_facecolor([0, 1, 0])
         rect.set_alpha(0.5)
         ax2.add_patch(rect)
 
@@ -192,32 +168,12 @@ class GraphMap(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class AnalSpec(QtWidgets.QDialog):
-    """
-    Analyse spectra.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class AnalSpec(BasicModule):
+    """Analyse spectra."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-            self.piter = ProgressBarText().iter
-        else:
-            self.showprocesslog = parent.showprocesslog
-            self.piter = parent.pbar.iter
 
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.depthmarkers = {'Start': [0., 0., 0.]}
         self.nummarkers = 1
         self.depthfunc = None
@@ -250,7 +206,6 @@ class AnalSpec(QtWidgets.QDialog):
 
         self.canvas.mpl_connect('button_press_event',
                                 self.button_press_callback)
-        self.resize(800, 400)
 
     def setupui(self):
         """
@@ -288,7 +243,6 @@ class AnalSpec(QtWidgets.QDialog):
         grid_main.addWidget(self.lw_speclib, 4, 1, 1, 2)
 
         grid_main.addWidget(self.group_info, 5, 1, 8, 2)
-        # grid_main.addWidget(self.pb_save, 9, 1, 1, 2)
 
         grid_main.addWidget(self.map, 0, 0, 10, 1)
         grid_main.addWidget(self.mpl_toolbar, 11, 0)
@@ -323,10 +277,16 @@ class AnalSpec(QtWidgets.QDialog):
         if event.button != 1:
             return
 
-        ax = self.map.figure.gca()
+        if event.inaxes != self.map.ax1:
+            return
+
+        ax = event.inaxes
         if ax.get_navigate_mode() is not None:
             return
 
+        if ax != self.map.ax1:
+            return
+
         self.map.row = int(event.ydata)
         self.map.col = int(event.xdata)
 
@@ -342,7 +302,7 @@ class AnalSpec(QtWidgets.QDialog):
         Parameters
         ----------
         row : TYPE
-            DESCRIPTION.
+            Unused.
 
         Returns
         -------
@@ -392,7 +352,7 @@ class AnalSpec(QtWidgets.QDialog):
         ext = ('ENVI Spectral Library (*.sli)')
 
         filename, _ = QtWidgets.QFileDialog.getOpenFileName(
-                self.parent, 'Open File', '.', ext)
+            self.parent, 'Open File', '.', ext)
         if filename == '':
             return
 
@@ -416,7 +376,6 @@ class AnalSpec(QtWidgets.QDialog):
 
         """
         self.map.mindx = self.combo.currentIndex()
-        # self.map.currentmark = self.combo_dmark.currentText()
         self.map.init_graph()
 
     def rotate_view(self):
@@ -448,17 +407,19 @@ class AnalSpec(QtWidgets.QDialog):
         sdata['traylen'] = self.dsb_traylen.value()
         sdata['depthmarkers'] = self.depthmarkers
 
-        with open(ofile, 'w') as todisk:
+        with open(ofile, 'w', encoding='utf-8') as todisk:
             json.dump(sdata, todisk, indent=4)
 
         self.lbl_info.setText('Save complete.')
 
     def settings(self, nodialog=False):
         """
-        Entrypoint into class.
+        Entry point into item.
 
-        This is called when the used double clicks the routine from the
-        main PyGMI interface.
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
@@ -474,22 +435,45 @@ class AnalSpec(QtWidgets.QDialog):
 
         if 'wavelength' not in self.indata['Raster'][0].metadata['Raster']:
             self.showprocesslog('Error: Your data should have wavelengths in'
-                                'the metadata')
+                                ' the metadata')
             return False
 
         dat = self.indata['Raster']
-        self.map.refl = float(dat[0].metadata['Raster']['reflectance_scale_factor'])
+
+        needsmerge = False
+        rows, cols = dat[0].data.shape
+
+        for i in dat:
+            irows, icols = i.data.shape
+            if irows != rows or icols != cols:
+                needsmerge = True
+
+        if needsmerge is True:
+            self.showprocesslog('Error: Your data bands have different sizes. '
+                                'Use Layer Stack to fix this first')
+            return False
+
+        wavelengths = []
+        dat2 = []
+        for i in dat:
+            if 'wavelength' in i.metadata['Raster']:
+                wavelengths.append(i.metadata['Raster']['wavelength'])
+                dat2.append(i)
+
+        dat = [i for _, i in sorted(zip(wavelengths, dat2))]
+
+        if 'reflectance_scale_factor' in dat[0].metadata['Raster']:
+            self.map.refl = float(
+                dat[0].metadata['Raster']['reflectance_scale_factor'])
 
         dat2 = []
         wvl = []
         for j in dat:
-            if self.chk_rot.isChecked():
-                dat2.append(j.data.T)
-            else:
-                dat2.append(j.data)
+            dat2.append(j.data)
             wvl.append(float(j.metadata['Raster']['wavelength']))
 
-        self.map.datarr = np.array(dat2)
+        self.map.datarr = dat2
+        self.map.nodata = dat[0].nodata
         self.map.wvl = np.array(wvl)
         if self.map.wvl.max() < 20:
             self.map.wvl = self.map.wvl*1000.
@@ -502,15 +486,13 @@ class AnalSpec(QtWidgets.QDialog):
         self.combo.addItems(bands)
         self.combo.currentIndexChanged.connect(self.on_combo)
 
-        ftxt = [str(i) for i in self.feature.keys()]
+        ftxt = [str(i) for i in self.feature]
         self.combo_feature.disconnect()
         self.combo_feature.clear()
         self.combo_feature.addItems(ftxt)
         self.feature_change()
         self.combo_feature.currentIndexChanged.connect(self.feature_change)
 
-        # self.map.init_graph()
-
         tmp = self.exec_()
 
         if tmp == 0:
@@ -535,9 +517,6 @@ class AnalSpec(QtWidgets.QDialog):
             A check to see if settings was successfully run.
 
         """
-
-        # self.combo_class.setCurrentText(projdata['combo_class'])
-
         return False
 
     def saveproj(self):
@@ -552,44 +531,24 @@ class AnalSpec(QtWidgets.QDialog):
         """
         projdata = {}
 
-        # projdata['combo_class'] = self.combo_class.currentText()
-
         return projdata
 
 
-class ProcFeatures(QtWidgets.QDialog):
-    """
-    Process Hyperspectral Features.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
-    """
+class ProcFeatures(BasicModule):
+    """Process Hyperspectral Features."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-            pbar = ProgressBarText()
-            self.piter = pbar.iter
 
-        else:
-            self.showprocesslog = parent.showprocesslog
-            self.piter = parent.pbar.iter
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.product = {}
         self.ratio = {}
+        self.feature = None
 
-        # self.combo_sensor = QtWidgets.QComboBox()
         self.cb_ratios = QtWidgets.QComboBox()
+        self.rfiltcheck = QtWidgets.QCheckBox('If the final product is a '
+                                              'ratio, filter out values less '
+                                              'than 1.')
+        self.filtercheck = QtWidgets.QCheckBox('Filter Albedo and Vegetation')
         self.tablewidget = QtWidgets.QTableWidget()
 
         self.setupui()
@@ -608,7 +567,6 @@ class ProcFeatures(QtWidgets.QDialog):
         gridlayout_main = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
         helpdocs = menu_default.HelpButton('pygmi.rsense.pfeat')
-        # label_sensor = QtWidgets.QLabel('Sensor:')
         lbl_ratios = QtWidgets.QLabel('Product:')
         lbl_details = QtWidgets.QLabel('Details:')
 
@@ -617,6 +575,8 @@ class ProcFeatures(QtWidgets.QDialog):
         self.tablewidget.setHorizontalHeaderLabels(['Feature', 'Filter',
                                                     'Threshold'])
         self.tablewidget.resizeColumnsToContents()
+        self.filtercheck.setChecked(True)
+        self.rfiltcheck.setChecked(True)
 
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
         buttonbox.setCenterButtons(True)
@@ -628,11 +588,14 @@ class ProcFeatures(QtWidgets.QDialog):
         gridlayout_main.addWidget(self.cb_ratios, 1, 1, 1, 1)
         gridlayout_main.addWidget(lbl_details, 2, 0, 1, 1)
         gridlayout_main.addWidget(self.tablewidget, 2, 1, 1, 1)
+        gridlayout_main.addWidget(self.filtercheck, 3, 0, 1, 2)
+        gridlayout_main.addWidget(self.rfiltcheck, 4, 0, 1, 2)
 
         gridlayout_main.addWidget(helpdocs, 6, 0, 1, 1)
         gridlayout_main.addWidget(buttonbox, 6, 1, 1, 3)
 
         self.cb_ratios.currentIndexChanged.connect(self.product_change)
+        self.filtercheck.stateChanged.connect(self.product_change)
         buttonbox.accepted.connect(self.accept)
         buttonbox.rejected.connect(self.reject)
 
@@ -648,7 +611,11 @@ class ProcFeatures(QtWidgets.QDialog):
         txt = self.cb_ratios.currentText()
         self.tablewidget.clear()
 
-        product = self.product[txt]+self.product['filter']
+        product = self.product[txt]
+
+        if self.filtercheck.isChecked():
+            product = product + self.product['filter']
+
         numrows = len(product)
 
         self.tablewidget.setRowCount(numrows)
@@ -669,7 +636,7 @@ class ProcFeatures(QtWidgets.QDialog):
         item.setFlags(item.flags() & ~QtCore.Qt.ItemIsEditable)
         self.tablewidget.setItem(0, 2, item)
 
-        if isinstance(product[0], int):
+        if product[0] in self.feature:
             desc = 'Feature Depth'
         elif product[0] in self.ratio:
             desc = self.ratio[product[0]]
@@ -695,7 +662,10 @@ class ProcFeatures(QtWidgets.QDialog):
             item = QtWidgets.QTableWidgetItem(txt2[2])
             self.tablewidget.setItem(i, 2, item)
 
-            desc = self.ratio[txt2[0]]
+            if txt2[0] in self.ratio:
+                desc = self.ratio[txt2[0]]
+            else:
+                desc = 'Feature between ' + str(self.feature[txt2[0]])
             item = QtWidgets.QTableWidgetItem(desc)
             item.setFlags(item.flags() & ~QtCore.Qt.ItemIsEditable)
             self.tablewidget.setItem(i, 3, item)
@@ -704,10 +674,12 @@ class ProcFeatures(QtWidgets.QDialog):
 
     def settings(self, nodialog=False):
         """
-        Entrypoint into class.
+        Entry point into item.
 
-        This is called when the used double clicks the routine from the
-        main PyGMI interface.
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
@@ -720,42 +692,18 @@ class ProcFeatures(QtWidgets.QDialog):
             self.showprocesslog('No Satellite Data')
             return False
 
-        self.feature = {}
-        self.feature[900] = [776, 1050, 850, 910]
-        self.feature[1300] = [1260, 1420]
-        self.feature[1800] = [1740, 1820]
-        self.feature[2080] = [2000, 2150]
-        self.feature[2500] = [2500, 2500]
-        self.feature[2200] = [2120, 2245]
-        self.feature[2290] = [2270, 2330]
-        self.feature[2330] = [2120, 2370]
+        self.feature = features.feature
+        self.ratio = features.ratio
 
-        self.ratio = {}
-        self.ratio['NDVI'] = '(R860-R687)/(R860+R687)'
-        self.ratio['dryveg'] = '(R2006+R2153)/(R2081+R2100)'
-        self.ratio['albedo'] = 'R1650'
-
-        self.ratio['r2350De'] = '(R2326+R2376)/(R2343+R2359)'
-        self.ratio['r2160D2190'] = '(R2136+R2188)/(R2153+R2171)'  # Kaolin from non kaolin
-        self.ratio['r2250D'] = '(R2227+R2275)/(R2241+R2259)'  # Chlorite epidote biotite
-        self.ratio['r2380D'] = '(R2365+R2415)/(R2381+R2390)'  # Amphibole, talc
-        self.ratio['r2330D'] = '(R2265+R2349)/(R2316+R2333)'  # MgOH and CO3
-        self.ratio['r1100D'] = '(R921+R1650)/(R1020+R1231)'   # Ferrous iron
-
-        # Elementwise product would work for formula below
         self.cb_ratios.disconnect()
-        self.product = {}
-        self.product['mica'] = [2200, 'r2350De > 1.02', 'r2160D2190 < 1.005']
-        self.product['smectite'] = [2200, 'r2350De < 1.02', 'r2160D2190 < 1.005']
-        self.product['kaolin'] = [2200, 'r2160D2190 > 1.005']
-        self.product['chlorite, epidote'] = ['r2250D', 'r2330D > 1.06']
-        self.product['ferrous iron'] = ['r1100D']
+        self.product = features.product.copy()
+        del self.product['filter']
         self.cb_ratios.clear()
         self.cb_ratios.addItems(self.product)
 
         # The filter line is added after the other products so that it does
         # not make it into the list widget
-        self.product['filter'] = ['NDVI < .25', 'dryveg < 1.015', 'albedo > 1000']
+        self.product['filter'] = features.product['filter']
         self.cb_ratios.currentIndexChanged.connect(self.product_change)
         self.product_change()
 
@@ -786,14 +734,6 @@ class ProcFeatures(QtWidgets.QDialog):
             A check to see if settings was successfully run.
 
         """
-        # self.combo_sensor.setCurrentText(projdata['sensor'])
-        # self.setratios()
-
-        # for i in self.lw_ratios.selectedItems():
-        #     if i.text()[2:] not in projdata['ratios']:
-        #         i.setSelected(False)
-        # self.set_selected_ratios()
-
         return False
 
     def saveproj(self):
@@ -807,13 +747,6 @@ class ProcFeatures(QtWidgets.QDialog):
 
         """
         projdata = {}
-        # projdata['sensor'] = self.combo_sensor.currentText()
-
-        # rlist = []
-        # for i in self.lw_ratios.selectedItems():
-        #     rlist.append(i.text()[2:])
-
-        # projdata['ratios'] = rlist
 
         return projdata
 
@@ -831,9 +764,8 @@ class ProcFeatures(QtWidgets.QDialog):
         datfin = []
 
         mineral = self.cb_ratios.currentText()
+        rfilt = self.rfiltcheck.isChecked()
 
-        feature = self.feature
-        ratio = self.ratio
         product = self.product
 
         try:
@@ -842,18 +774,50 @@ class ProcFeatures(QtWidgets.QDialog):
             product = [self.tablewidget.item(0, 0).text()]
         for i in range(1, self.tablewidget.rowCount()):
             product.append(self.tablewidget.item(i, 0).text())
-            product[-1] += ' ' + self.tablewidget.cellWidget(i, 1).currentText()
+            product[-1] += (' ' +
+                            self.tablewidget.cellWidget(i, 1).currentText())
             product[-1] += ' ' + self.tablewidget.item(i, 2).text()
 
-        dat = self.indata['Raster']
-        datfin = calcfeatures(dat, mineral, feature, ratio, product,
-                              piter=self.piter)
+        if 'RasterFileList' in self.indata:
+            flist = self.indata['RasterFileList']
+            odir = os.path.join(os.path.dirname(flist[0]), 'feature')
+
+            os.makedirs(odir, exist_ok=True)
+            for ifile in flist:
+                self.showprocesslog('Processing '+os.path.basename(ifile))
+
+                dat = get_raster(ifile)
+                datfin = calcfeatures(dat, mineral, self.feature, self.ratio,
+                                      product, rfilt, piter=self.piter)
+
+                ofile = (os.path.basename(ifile).split('.')[0] + '_' +
+                         mineral.replace(' ', '_') + '.tif')
+                ofile = os.path.join(odir, ofile)
+                if datfin[0].data.mask.min() == True:
+                    self.showprocesslog(' Could not find any ' + mineral +
+                                        '. No data to export.')
+                else:
+                    self.showprocesslog('Exporting '+os.path.basename(ofile))
+                    export_raster(ofile, datfin, 'GTiff', piter=self.piter)
+
+        elif 'Raster' in self.indata:
+            dat = self.indata['Raster']
+            datfin = calcfeatures(dat, mineral, self.feature, self.ratio,
+                                  product, rfilt, piter=self.piter)
+
+        if datfin[0].data.mask.min() == True:
+            QtWidgets.QMessageBox.warning(self.parent, 'Warning',
+                                          ' Could not find any ' + mineral +
+                                          '. No data to export.',
+                                          QtWidgets.QMessageBox.Ok)
+            return False
 
         self.outdata['Raster'] = datfin
         return True
 
 
-def calcfeatures(dat, mineral, feature, ratio, product, piter=iter):
+def calcfeatures(dat, mineral, feature, ratio, product, rfilt=True,
+                 piter=iter):
     """
     Calculate feature dataset.
 
@@ -870,27 +834,29 @@ def calcfeatures(dat, mineral, feature, ratio, product, piter=iter):
     product : dictionary
         Final hyperspectral products. Each dictionary value, is a list of
         features or ratios with thresholds to be combined.
+    rfilt : bool
+        Flag to decide whether to filter final ratio products less than 1.0
     piter : iter, optional
         Progress bar iterable. The default is iter.
 
     Returns
     -------
-    datfin : TYPE
-        DESCRIPTION.
+    datfin : list
+        Output datasets.
 
     """
-    allfeatures = [i for i in product if isinstance(i, int)]
-    allratios = [i.split()[0] for i in product
-                 if not isinstance(i, int)]
-    # allratios += [i.split()[0] for i in product['filter']
-    #               if not isinstance(i, int)]
+    allfeatures = [i.split()[0] for i in product if i[0] == 'f']
+    allratios = [i.split()[0] for i in product if i[0] != 'f']
 
     # Get list of wavelengths and data
     dat2 = []
     xval = []
     for j in dat:
         dat2.append(j.data)
-        refl = round(float(re.findall(r'[\d\.\d]+', j.dataid)[-1])*1000, 2)
+        refl = float(re.findall(r'[\d\.\d]+', j.dataid)[-1])
+        if refl < 100.:
+            refl = refl * 1000
+        refl = round(refl, 2)
         xval.append(refl)
 
     xval = np.array(xval)
@@ -906,7 +872,7 @@ def calcfeatures(dat, mineral, feature, ratio, product, piter=iter):
     datfin = []
     # Calclate ratios
     datcalc = {}
-    for j in allratios:
+    for j in piter(allratios):
         if j in datcalc:
             continue
         tmp = indexcalc(ratio[j], RBands)
@@ -938,31 +904,25 @@ def calcfeatures(dat, mineral, feature, ratio, product, piter=iter):
         i1a = tmp[0]
         i2a = tmp[-1]
 
+        fdat = np.moveaxis(fdat, 0, -1)
+
         for i in piter(range(rows)):
-            for j in range(cols):
-                yval = fdat[:, i, j]
-                if True in np.ma.getmaskarray(yval):
-                    continue
-
-                yhull = phull(yval)
-                crem = yval/yhull
-                imin = crem[i1a:i2a].argmin()
-                dtmp[i, j] = 1. - crem[i1a:i2a][imin]
-                ptmp[i, j] = xdat[i1a:i2a][imin]
-
+            ptmp[i], dtmp[i] = fproc(fdat[i].data, ptmp[i], dtmp[i], i1a, i2a,
+                                     xdat)
         depths[fname] = dtmp
         wvl[fname] = ptmp
+        datcalc[fname] = dtmp
 
     datout = None
     datout2 = None
     tmpw = None
 
     for i in product:
-        if isinstance(i, int):
+        if '>' in i or '<' in i or '=' in i or i[0] == 'r':
+            tmp = ne.evaluate(i, datcalc)
+        else:
             tmp = depths[i]
             tmpw = wvl[i]
-        else:
-            tmp = ne.evaluate(i, datcalc)
 
         if datout is None:
             datout = np.nan_to_num(tmp)
@@ -970,15 +930,17 @@ def calcfeatures(dat, mineral, feature, ratio, product, piter=iter):
         else:
             if tmp.max() > 1:
                 print('Problem with filter. Max value greater that 1')
-                breakpoint()
+                return datfin
             datout = datout * np.nan_to_num(tmp)
             if datout2 is not None:
                 datout2 = datout2 * np.nan_to_num(tmp)
 
-    if isinstance(product[0], int):
+    if product[0][0] == 'f':
         label = f'{mineral} depth'
     else:
         label = f'{mineral} ratio'
+        if rfilt is True:
+            datout[datout < 1] = 0
 
     datout = np.ma.masked_equal(datout, 0)
     datfin.append(numpy_to_pygmi(datout, dat[0], label))
@@ -1018,13 +980,150 @@ def indexcalc(formula, dat):
     return out
 
 
-def phull(sample):
+@jit(nopython=True)
+def fproc(fdat, ptmp, dtmp, i1a, i2a, xdat):
+    """
+    Feature process.
+
+    This function finds the minimum value of a feature.
+
+    Parameters
+    ----------
+    fdat : numpy array
+        Feature data
+    ptmp : numpy array
+        Feature wavelengths.
+    dtmp : numpy array
+        Feature depths.
+    i1a : int
+        Start index of feature definition.
+    i2a : int
+        End Index of feature definition.
+    xdat : numpy array
+        Wavelengths of feature definition.
+
+    Returns
+    -------
+    ptmp : numpy array
+        Feature wavelengths.
+    dtmp : numpy array
+        Feature depths.
+
+    """
+    cols, _ = fdat.shape
+
+    for j in range(cols):
+        yval = fdat[j]
+        if yval.mean() == 0:
+            continue
+
+        yhull = phull(yval)
+        crem = yval/yhull
+
+        imin = crem[i1a:i2a].argmin()
+
+        if imin == 0 or imin == (i2a-i1a-1):
+            dtmp[j] = 1. - crem[i1a:i2a][imin]
+            ptmp[j] = xdat[i1a:i2a][imin]
+            continue
+
+        x, y = cubic_calc(xdat[i1a:i2a], crem[i1a:i2a], imin)
+
+        ptmp[j] = x
+        dtmp[j] = 1. - y
+
+    return ptmp, dtmp
+
+
+@jit(nopython=True)
+def cubic_calc(xdat, crem, imin):
+    """
+    Find minimum of function using an analytic cubic calculation for speed.
+
+    Parameters
+    ----------
+    xdat : numpy array
+        wavelengths - x data.
+    crem : numpy array
+        continuum removed data - y data.
+    imin : int
+        Index for estimated minimum.
+
+    Returns
+    -------
+    x : float
+        wavelength at minimum.
+    y : float
+        y value at minimum.
+
+    """
+    x1 = xdat[imin-1]
+    x2 = xdat[imin]
+    x3 = xdat[imin+1]
+
+    y1 = crem[imin-1]
+    y2 = crem[imin]
+    y3 = crem[imin+1]
+
+    x = x2
+    y = y2
+
+    a1 = (2*x1**3*x2*y3 - 2*x1**3*x3*y2 - x1**2*x2**2*y2 - 3*x1**2*x2**2*y3 +
+          2*x1**2*x2*x3*y2 + 2*x1**2*x3**2*y2 + x1*x2**3*y1 + x1*x2**3*y3 +
+          x1*x2**2*x3*y1 + x1*x2**2*x3*y2 - 2*x1*x2*x3**2*y1 -
+          2*x1*x2*x3**2*y2 - 2*x2**3*x3*y1 +
+          2*x2**2*x3**2*y1)/(2*(x1 - x2)**2*(x1 - x3)*(x2 - x3))
+    b1 = (2*x1**3*y2 - 2*x1**3*y3 - 4*x1*x2**2*y1 + x1*x2**2*y2 +
+          3*x1*x2**2*y3 + 2*x1*x2*x3*y1 - 2*x1*x2*x3*y2 + 2*x1*x3**2*y1 -
+          2*x1*x3**2*y2 + x2**3*y1 - x2**3*y3 + x2**2*x3*y1 - x2**2*x3*y2 -
+          2*x2*x3**2*y1 + 2*x2*x3**2*y2)/(2*(x1 - x2)**2*(x1 - x3)*(x2 - x3))
+    c1 = -3*x1*(x1*y2 - x1*y3 - x2*y1 + x2*y3 + x3*y1 -
+                x3*y2)/(2*(x1 - x2)**2*(x1 - x3)*(x2 - x3))
+    d1 = (x1*y2 - x1*y3 - x2*y1 + x2*y3 + x3*y1 -
+          x3*y2)/(2*(x1 - x2)**2*(x1 - x3)*(x2 - x3))
+
+    a2 = (2*x1**2*x2**2*y3 - 2*x1**2*x2*x3*y2 - 2*x1**2*x2*x3*y3 +
+          2*x1**2*x3**2*y2 - 2*x1*x2**3*y3 + x1*x2**2*x3*y2 + x1*x2**2*x3*y3 +
+          2*x1*x2*x3**2*y2 - 2*x1*x3**3*y2 + x2**3*x3*y1 + x2**3*x3*y3 -
+          3*x2**2*x3**2*y1 - x2**2*x3**2*y2 +
+          2*x2*x3**3*y1)/(2*(x1 - x2)*(x1 - x3)*(x2 - x3)**2)
+    b2 = (2*x1**2*x2*y2 - 2*x1**2*x2*y3 - 2*x1**2*x3*y2 + 2*x1**2*x3*y3 -
+          x1*x2**2*y2 + x1*x2**2*y3 - 2*x1*x2*x3*y2 + 2*x1*x2*x3*y3 -
+          x2**3*y1 + x2**3*y3 + 3*x2**2*x3*y1 + x2**2*x3*y2 - 4*x2**2*x3*y3 -
+          2*x3**3*y1 + 2*x3**3*y2)/(2*(x1 - x2)*(x1 - x3)*(x2 - x3)**2)
+    c2 = 3*x3*(x1*y2 - x1*y3 - x2*y1 + x2*y3 + x3*y1 -
+               x3*y2)/(2*(x1 - x2)*(x1 - x3)*(x2 - x3)**2)
+    d2 = -(x1*y2 - x1*y3 - x2*y1 + x2*y3 + x3*y1 -
+           x3*y2)/(2*(x1 - x2)*(x1 - x3)*(x2 - x3)**2)
+
+    if abs(d1) > 2.22e+16:
+        min1 = [(-c1 + np.sqrt(-3*b1*d1 + c1**2))/(3*d1),
+                -(c1 + np.sqrt(-3*b1*d1 + c1**2))/(3*d1)]
+        for i in min1:
+            if x1 < i < x2:
+                x = i
+                y = a1+b1*x+c1*x**2+d1*x**3
+
+    if abs(d2) > 2.22e+16:
+        min2 = [(-c2 + np.sqrt(-3*b2*d2 + c2**2))/(3*d2),
+                -(c2 + np.sqrt(-3*b2*d2 + c2**2))/(3*d2)]
+
+        for i in min2:
+            if x2 < i < x3:
+                x = i
+                y = a2+b2*x+c2*x**2+d2*x**3
+
+    return x, y
+
+
+@jit(nopython=True)
+def phull(sample1):
     """
     Hull Calculation.
 
     Parameters
     ----------
-    sample : numpy array
+    sample1 : numpy array
         Sample to create a hull for.
 
     Returns
@@ -1033,10 +1132,12 @@ def phull(sample):
         Output hull.
 
     """
-    xvals = np.arange(sample.size)
-    sample = np.transpose([xvals, sample])
+    xvals = np.arange(sample1.size, dtype=np.int64)
+    sample = np.empty((sample1.size, 2))
+    sample[:, 0] = xvals
+    sample[:, 1] = sample1
 
-    edge = sample[:1]
+    edge = sample[:1].copy()
     rest = sample[1:]
 
     hull = [0]
@@ -1044,14 +1145,17 @@ def phull(sample):
         grad = rest - edge
         grad = grad[:, 1]/grad[:, 0]
         pivot = np.argmax(grad)
-        edge = rest[pivot]
+        edge[0, 0] = rest[pivot, 0]
+        edge[0, 1] = rest[pivot, 1]
         rest = rest[pivot+1:]
         hull.append(pivot)
 
     hull = np.array(hull) + 1
     hull = hull.cumsum()-1
-    out = np.transpose([hull, np.take(sample[:, 1], hull)])
-    out = np.interp(xvals, out[:, 0], out[:, 1])
+
+    take = np.take(sample[:, 1], hull)
+
+    out = np.interp(xvals, hull, take)
 
     return out
 
@@ -1070,7 +1174,7 @@ def readsli(ifile):
     spectra : dictionary
         Dictionary of spectra with wavelengths and reflectances.
     """
-    with open(ifile[:-4]+'.hdr') as file:
+    with open(ifile[:-4]+'.hdr', encoding='utf-8') as file:
         hdr = file.read()
 
     hdr = hdr.split('\n')
@@ -1095,7 +1199,7 @@ def readsli(ifile):
         if len(tmp) > 1:
             hdr3[tmp[0].strip()] = tmp[1].strip()
 
-    for i in hdr3:
+    for i in hdr3.keys():
         if i in ['samples', 'lines', 'bands', 'header offset', 'data type']:
             hdr3[i] = int(hdr3[i])
             continue
@@ -1134,6 +1238,13 @@ def readsli(ifile):
     data.shape = (hdr3['lines'], hdr3['samples'])
 
     spectra = {}
+    wmult = 1.
+
+    if hdr3['wavelength units'].lower() == 'micrometers':
+        wmult = 1000.
+
+    hdr3['wavelength'] = np.array(hdr3['wavelength'])*wmult
+
     for i, val in enumerate(hdr3['spectra names']):
         spectra[val] = {'wvl': hdr3['wavelength'],
                         'refl': data[i]}
@@ -1143,45 +1254,13 @@ def readsli(ifile):
 
 def _testfn():
     """Test routine."""
-    pbar = ProgressBarText()
+    from pygmi.rsense.iodefs import get_data
 
-    app = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
 
-    ifile = r'c:\work\Workdata\Richtersveld\Reprocessed\057_0818-1117_ref_rect_BSQ.hdr'
-
-    xoff = 0
-    yoff = 2000
-    xsize = None
-    ysize = 1000
-    nodata = 15000
-    nodata = 0
-
-    iraster = (xoff, yoff, xsize, ysize)
-    # iraster = None
-
-    data = get_raster(ifile, nval=nodata, iraster=iraster, piter=pbar.iter)
-
-    # data = get_raster(ifile, piter=pbar.iter)
-
-    tmp = ProcFeatures(None)
-    tmp.indata['Raster'] = data
-    tmp.settings()
-
-
-def _testfn2():
-    """Test routine."""
-    pbar = ProgressBarText()
-
-    # ifile = (r'c:\work\Workdata\HyperspectralScanner\PTest\smile\FENIX\\'
-    #          r'clip_BV1_17_118m16_125m79_2020-06-30_12-43-14.dat')
-
-    # ifile = (r'C:\Work\Workdata\HyperspectralScanner\Processed Data\\'
-    #           r'FENIX L201 Data Preparation v0810\BV1_17_extracted_image.img')
-
-    # data = get_raster(ifile, piter=pbar.iter)
-
-    ifile = r'e:\Workdata\Richtersveld\Reprocessed\057_0818-1117_ref_rect_BSQ.hdr'
-    ifile = r'e:\Workdata\Richtersveld\Reprocessed\030_0815-1050_ref_rect.hdr'
+    # ifile = r"d:\Workdata\Hyperspectral\080_0824-0920_ref_rect_clip.hdr"
+    # ifile = r"d:\Workdata\Remote Sensing\hyperion\EO1H1760802013198110KF_1T.ZIP"
+    ifile = r"D:\Workdata\PyGMI Test Data\Remote Sensing\Import\hyperspectral\071_0818-0932_ref_rect_BSQ.hdr"
 
     xoff = 0
     yoff = 2000
@@ -1193,13 +1272,189 @@ def _testfn2():
     iraster = (xoff, yoff, xsize, ysize)
     iraster = None
 
-    data = get_raster(ifile, nval=nodata, iraster=iraster, piter = pbar.iter)
+    # data = get_raster(ifile, nval=nodata, iraster=iraster, piter=pbar.iter)
+    data = get_data(ifile)
 
-    app = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    # data = get_raster(ifile, piter=pbar.iter)
+
+    tmp = ProcFeatures(None)
+    tmp.indata['Raster'] = data
+    # tmp.cb_ratios.setCurrentText('ferric iron')
+    tmp.settings()
+
+    dat = tmp.outdata['Raster'][0]
+
+    plt.figure(dpi=150)
+    plt.imshow(dat.data)
+    plt.colorbar()
+    plt.show()
+
+    print(dat.data.mean())
+
+    plt.figure(dpi=150)
+    plt.hist(dat.data.flatten(), bins=200)
+    plt.show()
+
+    tmp = np.histogram(dat.data[dat.data > 0])
+
+
+def _testfn2():
+    """Test routine."""
+    from pygmi.rsense.iodefs import get_data
+    from pygmi.raster.dataprep import lstack
+
+    ifile = r"D:\Workdata\PyGMI Test Data\Remote Sensing\Import\hyperspectral\071_0818-0932_ref_rect_BSQ.hdr"
+    # ifile = r"c:\Workdata\Remote Sensing\hyperion\EO1H1760802013198110KF_1T.ZIP"
+
+    data = get_data(ifile)
+
+    data = lstack(data)
+
+    app = QtWidgets.QApplication(sys.argv)
     tmp = AnalSpec()
     tmp.indata['Raster'] = data
     tmp.settings()
 
 
+def _testfn3():
+    """Test."""
+    ifile1 = r'd:\Workdata\Lithosphere\merge\cut-087-0824_iMNF15.hdr'
+    ifile2 = r'd:\Workdata\Lithosphere\merge\cut-088-0824_iMNF15.hdr'
+    ifile3 = r'd:\Workdata\Lithosphere\merge\cut-089-0824_iMNF15.hdr'
+
+    feat = 18
+
+    yoff = 2425
+    ysize = 400
+    nodata = 0
+    iraster = (0, yoff, None, ysize)
+
+    data1 = get_raster(ifile1, nval=nodata, iraster=iraster)
+    data2 = get_raster(ifile2, nval=nodata, iraster=iraster)
+    data3 = get_raster(ifile3, nval=nodata, iraster=iraster)
+
+    plt.figure(dpi=150)
+    plt.imshow(data1[0].data, extent=data1[0].extent)
+    plt.plot(277545, 6774900, '+k')
+    plt.show()
+
+    plt.figure(dpi=150)
+    plt.imshow(data2[0].data, extent=data2[0].extent)
+    plt.plot(277545, 6774900, '+k')
+    plt.plot(279900, 6774900, '+k')
+    plt.show()
+
+    plt.figure(dpi=150)
+    plt.imshow(data3[0].data, extent=data3[0].extent)
+    plt.plot(279900, 6774900, '+k')
+    plt.show()
+
+    yoff = 2625
+    ysize = 1
+    nodata = 0
+    iraster = (0, yoff, None, ysize)
+
+    data1 = get_raster(ifile1, nval=nodata, iraster=iraster)
+    data2 = get_raster(ifile2, nval=nodata, iraster=iraster)
+    data3 = get_raster(ifile3, nval=nodata, iraster=iraster)
+
+    for i in range(2):
+        if i == 0:
+            xxx = 277545
+            yyy = 6774900
+        else:
+            xxx = 279900
+            yyy = 6774900
+            data1 = data3
+
+        i1 = int((xxx-data1[0].extent[0])/data1[0].xdim)
+        i2 = int((xxx-data2[0].extent[0])/data2[0].xdim)
+
+        # Get list of wavelengths and data
+        dat1 = []
+        xval1 = []
+        for j in data1:
+            dat1.append(j.data)
+            refl = round(float(re.findall(r'[\d\.\d]+', j.dataid)[-1])*1000, 2)
+            xval1.append(refl)
+
+        xval1 = np.array(xval1)
+        dat1 = np.array(dat1)
+
+        dat2 = []
+        xval2 = []
+        for j in data2:
+            dat2.append(j.data)
+            refl = round(float(re.findall(r'[\d\.\d]+', j.dataid)[-1])*1000, 2)
+            xval2.append(refl)
+
+        xval2 = np.array(xval2)
+        dat2 = np.array(dat2)
+
+        dat1 = dat1[55: 98]
+        dat2 = dat2[55: 98]
+        xval1 = xval1[55: 98]
+        xval2 = xval2[55: 98]
+
+        fdat = []
+        xdat = []
+        fdat.append(dat1[:, 0, i1])
+        fdat.append(dat2[:, 0, i2])
+        xdat = xval1
+
+        fdat = np.array(fdat)
+
+        ptmp = np.array([0, 0])
+        dtmp = np.array([0, 0])
+
+        ptmp, dtmp = fproc(fdat, ptmp, dtmp, 11, 19, xdat)
+
+        spec1 = dat1[:, 0, i1]
+        spec2 = dat2[:, 0, i2]
+
+        plt.figure(dpi=150)
+        plt.title(str(i)+': '+str(i1))
+        plt.plot(xval1, spec1)
+        plt.plot(xval2, spec2, '-.')
+        ymin, ymax = plt.gca().get_ylim()
+        plt.vlines(ptmp[0], ymin, ymax, 'k')
+        plt.vlines(ptmp[1], ymin, ymax, 'k')
+
+        spec1 = spec1/phull(spec1)
+        spec2 = spec2/phull(spec2)
+
+        plt.figure(dpi=150)
+        plt.title(str(i)+': '+str(i1))
+        plt.plot(xval1, spec1)
+        plt.plot(xval2, spec2, 'r-.')
+
+        plt.plot(xval1[11:19], spec1[11:19], '.')
+        plt.plot(xval2[11:19], spec2[11:19], '.')
+
+        ymin, ymax = plt.gca().get_ylim()
+        plt.vlines(ptmp[0], ymin, ymax, label=str(ptmp[0]))
+        plt.vlines(ptmp[1], ymin, ymax, colors='r', linestyles='-.',
+                   label=str(ptmp[1]))
+        plt.legend()
+        plt.show()
+
+        plt.figure(dpi=150)
+
+        dat1 = np.ma.masked_equal(dat1, 0)
+        dat2 = np.ma.masked_equal(dat2, 0)
+
+        x1 = np.linspace(data1[0].extent[0], data1[0].extent[1],
+                         dat1[feat, 0].size)
+        x2 = np.linspace(data2[0].extent[0], data2[0].extent[1],
+                         dat2[feat, 0].size)
+        plt.plot(x1, dat1[feat, 0])
+        plt.plot(x2, dat2[feat, 0], '-.')
+
+        ymin, ymax = plt.gca().get_ylim()
+
+        plt.vlines(xxx, ymin, ymax, 'k')
+        plt.show()
+
+
 if __name__ == "__main__":
-    _testfn2()
+    _testfn()
