@@ -30,6 +30,8 @@ from pygmi.rsense import change
 from pygmi.rsense import iodefs
 from pygmi.rsense import ratios
 from pygmi.rsense import hyperspec
+from pygmi.rsense import transforms
+from pygmi.rsense.landsat_composite import LandsatComposite
 
 
 class MenuWidget():
@@ -48,24 +50,20 @@ class MenuWidget():
     def __init__(self, parent=None):
 
         self.parent = parent
-        self.parent.add_to_context('Remote Sensing')
+        self.parent.add_to_context('RasterFileList')
+        context_menu = self.parent.context_menu
 
-# Normal menus
+        # Normal menus
         self.menu = QtWidgets.QMenu('Remote Sensing')
         parent.menubar.addAction(self.menu.menuAction())
 
         self.menu3 = self.menu.addMenu('Import Data')
 
-        # self.action_import_ged = QtWidgets.QAction('Import ASTER Global '
-        #                                            'Emissivity Data')
-        # self.menu3.addAction(self.action_import_ged)
-        # self.action_import_ged.triggered.connect(self.import_ged)
-
         self.action_import_aster = QtWidgets.QAction('Import ASTER')
         self.menu3.addAction(self.action_import_aster)
         self.action_import_aster.triggered.connect(self.import_aster)
 
-        self.action_import_landsat = QtWidgets.QAction('Import Landsat 4 to 8')
+        self.action_import_landsat = QtWidgets.QAction('Import Landsat 4 to 9')
         self.menu3.addAction(self.action_import_landsat)
         self.action_import_landsat.triggered.connect(self.import_landsat)
 
@@ -73,6 +71,11 @@ class MenuWidget():
         self.menu3.addAction(self.action_import_sentinel2)
         self.action_import_sentinel2.triggered.connect(self.import_sentinel2)
 
+        self.action_import_sentinel2b = QtWidgets.QAction('Import Sentinel-2 '
+                                                          '(bands only)')
+        self.menu3.addAction(self.action_import_sentinel2b)
+        self.action_import_sentinel2b.triggered.connect(self.import_sentinel2b)
+
         self.action_import_sentinel5p = QtWidgets.QAction('Import Sentinel-5P')
         self.menu3.addAction(self.action_import_sentinel5p)
         self.action_import_sentinel5p.triggered.connect(self.import_sentinel5p)
@@ -81,6 +84,19 @@ class MenuWidget():
         self.menu3.addAction(self.action_import_modis)
         self.action_import_modis.triggered.connect(self.import_modis)
 
+        self.action_import_hyperion = QtWidgets.QAction('Import Hyperion L1T')
+        self.menu3.addAction(self.action_import_hyperion)
+        self.action_import_hyperion.triggered.connect(self.import_hyperion)
+
+        self.action_import_wv = QtWidgets.QAction('Import WorldView Tiles')
+        self.menu3.addAction(self.action_import_wv)
+        self.action_import_wv.triggered.connect(self.import_wv)
+
+        self.action_import_ged = QtWidgets.QAction('Import ASTER Global '
+                                                   'Emissivity Database')
+        self.menu3.addAction(self.action_import_ged)
+        self.action_import_ged.triggered.connect(self.import_ged)
+
         self.menu3.addSeparator()
 
         self.action_batch_list = QtWidgets.QAction('Create Batch List')
@@ -93,6 +109,22 @@ class MenuWidget():
         self.menu.addAction(self.action_calc_ratios)
         self.action_calc_ratios.triggered.connect(self.calc_ratios)
 
+        self.action_calc_ci = QtWidgets.QAction('Calculate Condition Indices')
+        self.menu.addAction(self.action_calc_ci)
+        self.action_calc_ci.triggered.connect(self.calc_ci)
+
+        self.action_lsat_comp = QtWidgets.QAction('Calculate Landsat '
+                                                  'Temporal Composite')
+        self.menu.addAction(self.action_lsat_comp)
+        self.action_lsat_comp.triggered.connect(self.lsat_comp)
+
+        self.action_mnf = QtWidgets.QAction('MNF Transform')
+        self.menu.addAction(self.action_mnf)
+        self.action_mnf.triggered.connect(self.mnf)
+
+        self.action_pca = QtWidgets.QAction('PCA Transform')
+        self.menu.addAction(self.action_pca)
+        self.action_pca.triggered.connect(self.pca)
         self.menu.addSeparator()
 
         self.menu4 = self.menu.addMenu('Hyperspectral Imaging')
@@ -121,6 +153,17 @@ class MenuWidget():
         self.menu2.addAction(self.action_data_viewer)
         self.action_data_viewer.triggered.connect(self.view_change)
 
+        # Context menus
+        context_menu['RasterFileList'].addSeparator()
+
+        self.action_exportlist = QtWidgets.QAction('Export Raster File List')
+        context_menu['RasterFileList'].addAction(self.action_exportlist)
+        self.action_exportlist.triggered.connect(self.exportlist)
+
+    def exportlist(self):
+        """Export Raster File List."""
+        self.parent.launch_context_item(iodefs.ExportBatch)
+
     def create_scene(self):
         """Create Scene."""
         self.parent.item_insert('Step', 'Create Scene List',
@@ -141,6 +184,26 @@ class MenuWidget():
         self.parent.item_insert('Step', 'Calculate Band Ratios',
                                 ratios.SatRatios)
 
+    def calc_ci(self):
+        """Calculate Condition Indices."""
+        self.parent.item_insert('Step', 'Calculate Condition Indices',
+                                ratios.ConditionIndices)
+
+    def lsat_comp(self):
+        """Calculate Landsat Composite."""
+        self.parent.item_insert('Io', 'Calculate Landsat Temporal Composite',
+                                LandsatComposite)
+
+    def mnf(self):
+        """Calculate MNF."""
+        self.parent.item_insert('Step', 'MNF Transform',
+                                transforms.MNF)
+
+    def pca(self):
+        """Calculate PCA."""
+        self.parent.item_insert('Step', 'PCA Transform',
+                                transforms.PCA)
+
     def anal_spec(self):
         """Analyse Spectra."""
         self.parent.item_insert('Step', 'Analyse Spectra', hyperspec.AnalSpec)
@@ -158,33 +221,43 @@ class MenuWidget():
     def import_sentinel2(self):
         """Import Sentinel 2 data."""
         self.parent.item_insert('Io', 'Import Sentinel-2', iodefs.ImportData,
-                                params='Sentinel-2 (*.xml);;')
+                                extscene='Sentinel-2 (*.xml *.zip);;')
+
+    def import_sentinel2b(self):
+        """Import Sentinel 2 data, bands only."""
+        self.parent.item_insert('Io', 'Import Sentinel-2', iodefs.ImportData,
+                                extscene='Sentinel-2 Bands Only (*.xml *.zip);;')
 
     def import_modis(self):
         """Import MODIS data."""
         self.parent.item_insert('Io', 'Import MODIS v6', iodefs.ImportData,
-                                params='MODIS (*.hdf);;')
+                                extscene='MODIS (*.hdf);;')
 
     def import_aster(self):
         """Import ASTER HDF data."""
         self.parent.item_insert('Io', 'Import ASTER', iodefs.ImportData,
-                                params='ASTER (AST*.hdf AST*.zip);;')
+                                extscene='ASTER (AST*.hdf AST*.zip);;')
 
-    def import_hdf(self):
+    def import_ged(self):
         """Import HDF data."""
-        self.parent.item_insert('Io', 'Import HDF', iodefs.ImportData,
-                                params='hdf (*.hdf *.h5);;')
+        self.parent.item_insert('Io', 'Import ASTER Global Emissivity Data',
+                                iodefs.ImportData,
+                                extscene='h5 (*.h5);;')
 
     def import_landsat(self):
         """Import Landsat data."""
         self.parent.item_insert('Io', 'Import Landsat', iodefs.ImportData,
-                                params='Landsat (L*.tar L*.tar.gz L*_MTL.txt);;')
+                                extscene='Landsat (L*.tar L*.tar.gz L*_MTL.txt);;')
 
-    def import_ged(self):
-        """Import GED data."""
-        self.parent.item_insert('Io', 'Import ASTER Global Emissivity Data',
-                                iodefs.ImportData,
-                                params='ASTER GED (*.bin);;')
+    def import_hyperion(self):
+        """Import Hyperion data."""
+        self.parent.item_insert('Io', 'Import Hyperion L1T', iodefs.ImportData,
+                                extscene='Hyperion L1T (*.zip);;')
+
+    def import_wv(self):
+        """Import WorldView data."""
+        self.parent.item_insert('Io', 'Import WorldView', iodefs.ImportData,
+                                extscene='WorldView (*.xml);;')
 
     def batch_list(self):
         """Import batch list."""
