@@ -33,7 +33,7 @@ import os
 import numpy as np
 import numexpr as ne
 from PyQt5 import QtWidgets
-import matplotlib.cm as cm
+from matplotlib import colormaps
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
@@ -41,6 +41,8 @@ from matplotlib import patches
 from osgeo import ogr, osr
 import scipy.spatial.distance as sdist
 
+from pygmi.misc import BasicModule
+
 
 class MyMplCanvas(FigureCanvasQTAgg):
     """Canvas for the actual plot."""
@@ -55,7 +57,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         # figure stuff
         self.htype = 'Linear'
-        self.cbar = cm.get_cmap('jet')
+        self.cbar = colormaps['jet']
         self.data = []
         self.gmode = None
         self.argb = [None, None, None]
@@ -72,8 +74,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.isgeog = True
 
         self.axes = fig.add_subplot(111)
-#        self.axes.xaxis.set_visible(False)
-#        self.axes.yaxis.set_visible(False)
 
         self.setParent(parent)
 
@@ -124,20 +124,12 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class BeachBall(QtWidgets.QDialog):
+class BeachBall(BasicModule):
     """Create shapefiles with beachballs."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
         self.algorithm = 'FPFIT'
         self.nofps = False
         self.stype = 'Seis'
@@ -413,11 +405,17 @@ class BeachBall(QtWidgets.QDialog):
 
     def settings(self, nodialog=False):
         """
-        Run.
+        Entry point into item.
+
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
 
         Returns
         -------
-        None.
+        bool
+            True if successful, False otherwise.
 
         """
         if self.nofps:
@@ -459,8 +457,6 @@ class BeachBall(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
 
@@ -497,6 +493,8 @@ def beachball(fm, centerx, centery, diam, isgeog, showprocesslog=print):
         draw beachball with this diameter.
     isgeog : bool
         True if in geographic coordinates, False otherwise.
+    showprocesslog : function, optional
+        Routine to show text messages. The default is print.
 
     Returns
     -------
@@ -684,18 +682,18 @@ def auxplane(s1, d1, r1):
     z = (s1+90)/r2d
     z2 = d1/r2d
     z3 = r1/r2d
-    # /* slick vector in plane 1 */
+    # slick vector in plane 1
     sl1 = -np.cos(z3)*np.cos(z)-np.sin(z3)*np.sin(z)*np.cos(z2)
     sl2 = np.cos(z3)*np.sin(z)-np.sin(z3)*np.cos(z)*np.cos(z2)
     sl3 = np.sin(z3)*np.sin(z2)
     strike, dip = strikedip(sl2, sl1, sl3)
 
-    n1 = np.sin(z)*np.sin(z2)  # /* normal vector to plane 1 */
+    n1 = np.sin(z)*np.sin(z2)  # normal vector to plane 1
     n2 = np.cos(z)*np.sin(z2)
-#    n3 = np.cos(z2)
-    h1 = -sl2  # /* strike vector of plane 2 */
+    # n3 = np.cos(z2)
+    h1 = -sl2  # strike vector of plane 2
     h2 = sl1
-    # /* note h3=0 always so we leave it out */
+    # note h3=0 always so we leave it out
 
     z = h1*n1 + h2*n2
     z = z/np.sqrt(h1*h1 + h2*h2)
