@@ -39,6 +39,10 @@ from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 from matplotlib.patches import Ellipse
+import contextily as ctx
+import pyproj
+
+from pygmi.misc import ContextModule
 
 
 class MyMplCanvas(FigureCanvasQTAgg):
@@ -47,7 +51,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
     Attributes
     ----------
-    axes : matplotlib subplot
+    axes : matplotlib axes
+        axes for matplotlib subplot
     parent : parent
         reference to the parent routine
     """
@@ -80,6 +85,11 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         """
         self.figure.clear()
+        self.axes = self.figure.add_subplot(111)
+
+        if len(datd) == 0:
+            self.figure.canvas.draw()
+            return
 
         x = np.ma.masked_invalid(datd['1_longitude'])
         y = np.ma.masked_invalid(datd['1_latitude'])
@@ -89,17 +99,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         ymin = y.min()-0.5
         ymax = y.max()+0.5
 
-        self.axes = self.figure.add_subplot(111)  # , projection=ccrs.PlateCarree())
         self.axes.set_xlim(xmin, xmax)
         self.axes.set_ylim(ymin, ymax)
 
-#        extent = [xmin, xmax, ymax, ymin]
-#        request = cimgt.GoogleTiles(style='satellite')
-#        self.axes.set_extent(extent, crs=ccrs.PlateCarree())
-
-#        self.axes.add_image(request, 6)
-#        self.axes.gridlines(draw_labels=True)
-
         for dat in dats:
             if 'E' not in dat:
                 continue
@@ -150,12 +152,18 @@ class MyMplCanvas(FigureCanvasQTAgg):
                           width=demaj, height=demin,
                           angle=ang, color='black')
             ell.set_facecolor('none')
-            #    ell.set_edgecolor('black')
 
-#            self.axes.add_artist(ell)
             self.ellipses.append(ell.get_verts())
             self.axes.add_artist(ell)
 
+
+        try:
+            ctx.add_basemap(self.axes, crs=pyproj.CRS.from_epsg(4326),
+                            source=ctx.providers.OpenStreetMap.Mapnik)
+        except:
+            print('No internet')
+
+
         self.figure.tight_layout()
         self.figure.canvas.draw()
 
@@ -186,6 +194,11 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+
+        if len(data1) == 0 or len(data2) == 0:
+            self.figure.canvas.draw()
+            return
+
         x = np.ma.masked_invalid(data1)
         y = np.ma.masked_invalid(data2)
 
@@ -212,13 +225,14 @@ class MyMplCanvas(FigureCanvasQTAgg):
         if np.unique(y).size < bins[1]:
             bins = (bins[0], np.unique(y).size)
 
-#        hbin = self.axes.hexbin(x, y, gridsize=50, mincnt=1)
         hbin = self.axes.hist2d(x, y, bins=bins, cmin=1, range=rng)
-#        self.axes.axis([xmin, xmax, ymin, ymax])
         self.axes.set_xlabel(xlbl, fontsize=8)
         self.axes.set_ylabel(ylbl, fontsize=8)
         self.axes.set_xticks(np.arange(x.min(), x.max()+1))
 
+        for tick in self.axes.get_xticklabels():
+            tick.set_rotation(90)
+
         cbar = self.figure.colorbar(hbin[3])
         cbar.set_label('Number of Events')
 
@@ -235,9 +249,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         data1 : numpy array.
             raster dataset to be used
         xlbl : str, optional
-            X-axis label. The default is 'Time'.
+            X-axis label. The default is 'Data Value'.
         ylbl : str, optional
-            Y-axis label. The default is 'ML'.
+            Y-axis label. The default is 'Number of Observations'.
         bins : int or str, optional
             Number of bins or binning strategy. See matplotlib.pyplot.hist.
             The default is 'doane'.
@@ -251,6 +265,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+        if len(data1) == 0:
+            self.figure.canvas.draw()
+            return
 
         dattmp = np.array(data1)
         dattmp = dattmp[~np.isnan(dattmp)]
@@ -267,6 +284,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
             self.axes.set_xlim(rng[0], rng[1])
             self.axes.set_xticks(np.arange(int(rng[0]+1), int(rng[1])+1))
 
+        for tick in self.axes.get_xticklabels():
+            tick.set_rotation(90)
+
         self.figure.tight_layout()
         self.figure.canvas.draw()
 
@@ -289,6 +309,10 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+        if len(data1a) == 0:
+            self.figure.canvas.draw()
+            return
+
         data1 = np.ma.masked_invalid(data1a)
         data1 = data1.compressed()
 
@@ -358,6 +382,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+        if len(data1) == 0:
+            self.figure.canvas.draw()
+            return
 
         pid = np.array(data1['4_phase_id'])
         tres = np.array(data1['4_travel_time_residual'])
@@ -387,7 +414,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
     def update_residual(self, dat, res='ML'):
         """
-        Update the residualplot.
+        Update the residual plot.
 
         Parameters
         ----------
@@ -403,6 +430,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+        if len(dat) == 0:
+            self.figure.canvas.draw()
+            return
 
         A = {}
         T = {}
@@ -412,7 +442,8 @@ class MyMplCanvas(FigureCanvasQTAgg):
             for rec in event['4']:
                 if (rec.quality+rec.phase_id).strip() in ['IAML', 'AML',
                                                           'ES', 'E']:
-                    if rec.amplitude is None or rec.epicentral_distance is None:
+                    if (rec.amplitude is None or
+                            rec.epicentral_distance is None):
                         continue
                     ML = (np.log10(rec.amplitude) +
                           1.149*np.log10(rec.epicentral_distance) +
@@ -476,7 +507,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         min_vps : float, optional
             Minimum VPS. The default is 1.53.
         max_vps : float, optional
-            Maximu VPS. The default is 1.93.
+            Maximum VPS. The default is 1.93.
 
         Returns
         -------
@@ -485,6 +516,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         """
         self.figure.clear()
         self.axes = self.figure.add_subplot(111)
+        if len(dat) == 0:
+            self.figure.canvas.draw()
+            return
 
         VPS = []
         for event in dat:
@@ -493,7 +527,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
             for rec in event['4']:
                 if rec.weighting_indicator == 9:
                     continue
-                # breakpoint()
                 time = rec.hour*3600+rec.minutes*60+rec.seconds
                 if rec.phase_id == 'P   ':
                     P[rec.station_name] = time
@@ -546,19 +579,11 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.canvas.draw()
 
 
-class GraphWindow(QtWidgets.QDialog):
-    """
-    Graph Window - The QDialog window which will contain our image.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    """
+class GraphWindow(ContextModule):
+    """Graph Window - The QDialog window which will contain our image."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.parent = parent
 
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
@@ -621,23 +646,14 @@ class PlotQC(GraphWindow):
         reference to GraphWindow's label2
     combobox2 : QComboBox
         reference to GraphWindow's combobox2
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
+
         self.label2.hide()
         self.combobox2.hide()
-        self.indata = {}
-        self.parent = parent
         self.datd = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def change_band(self):
         """
@@ -670,7 +686,6 @@ class PlotQC(GraphWindow):
             self.mmc.update_hist(self.datd['1_number_of_stations_used'],
                                  i, bins=bins, rng=(bmin, bmax))
 
-            # self.mmc.update_hist(self.datd['1_number_of_stations_used'], i)
         elif i == 'RMS of time residuals':
             rts = np.array(self.datd['1_rms_of_time_residuals'])
             self.mmc.update_hist(rts, i)
@@ -708,7 +723,7 @@ class PlotQC(GraphWindow):
 
     def run(self):
         """
-        Run.
+        Entry point to run routine.
 
         Returns
         -------
@@ -789,7 +804,6 @@ class PlotQC(GraphWindow):
         # create the layer
         layer = data_source.CreateLayer('Fault Plane Solution', srs,
                                         ogr.wkbPolygon)
-#        layer.CreateField(ogr.FieldDefn('Strike', ogr.OFTReal))
 
         # Calculate BeachBall
         indata = self.mmc.ellipses
@@ -804,8 +818,6 @@ class PlotQC(GraphWindow):
 
             feature = ogr.Feature(layer.GetLayerDefn())
 
-#            feature.SetField('Strike', np1[0])
-
             feature.SetGeometry(poly)
             # Create the feature in the layer (shapefile)
             layer.CreateFeature(feature)
@@ -835,6 +847,11 @@ def import_for_plots(dat):
     """
     datd = {}
 
+    # Next 3 lines are so that certain plots don't break
+    datd['1_ML'] = []
+    datd['1_ML_year'] = []
+    datd['1_ML_time'] = []
+
     for event in dat:
         if '1' not in event:
             continue
@@ -880,7 +897,7 @@ def import_for_plots(dat):
 
 def eigsorted(cov):
     """
-    Calculate and sort eigen values.
+    Calculate and sort eigenvalues.
 
     Parameters
     ----------
@@ -905,11 +922,9 @@ def _testfn():
     import sys
     from pygmi.seis.iodefs import ImportSeisan
 
-    app = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
     tmp = ImportSeisan()
-    tmp.ifile = r'D:\Workdata\seismology\collect1.out'
-    # tmp.ifile = r'C:\Work\Workdata\seismology\march2021\select.out'
-    # tmp.ifile = r'C:\Work\Workdata\seismology\march2021\wadati_err2.out'
+    tmp.ifile = r"D:\Workdata\PyGMI Test Data\Sesimology\collect2.out"
     tmp.settings(True)
 
     data = tmp.outdata['Seis']
@@ -930,7 +945,7 @@ def _testfn2():
 
     ifile = r'D:\Workdata\seismology\april2021\collect.out'
 
-    app = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
     tmp = ImportSeisan()
     tmp.ifile = ifile
     tmp.settings(True)
@@ -939,7 +954,7 @@ def _testfn2():
 
     ifile = r'D:\Workdata\seismology\april2021\mulplt.wav'
 
-    with open(ifile) as pntfile:
+    with open(ifile, encoding='utf-8') as pntfile:
         ltmp = pntfile.read()
 
     ltmp = ltmp.split('\n')
@@ -969,8 +984,6 @@ def _testfn2():
         plt.plot(x, y)
         plt.show()
 
-    breakpoint()
-
 
 if __name__ == "__main__":
-    _testfn2()
+    _testfn()
