@@ -22,7 +22,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # -----------------------------------------------------------------------------
-"""Import Data."""
+"""Import Seismology Data."""
 
 import os
 import re
@@ -30,8 +30,10 @@ from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import pandas as pd
 import geopandas as gpd
+
 import pygmi.seis.datatypes as sdt
-import pygmi.menu_default as menu_default
+from pygmi import menu_default
+from pygmi.misc import ContextModule, BasicModule
 
 
 def sform(strform, val, tmp, col1, col2=None, nval=-999):
@@ -49,12 +51,14 @@ def sform(strform, val, tmp, col1, col2=None, nval=-999):
         This string must be of the form {0:4.1f}, where 4.1f can be changed.
     val : float, int, str
         input value
-    nval : float, int
-        null value which gets substituted by spaces
+    tmp : str
+        Input string
     col1 : int
         start column (1 is first column)
-    col2 : inr
-        end column
+    col2 : int
+        end column. The default is None.
+    nval : float, int
+        null value which gets substituted by spaces. The default is -999.
 
     Returns
     -------
@@ -129,23 +133,16 @@ def str2int(inp):
     return int(inp)
 
 
-class ImportSeisan():
+class ImportSeisan(BasicModule):
     """Import SEISAN Data."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
+        super().__init__(parent)
 
         idir = os.path.dirname(os.path.realpath(__file__))
         self.tfile = os.path.join(idir, r'descriptions.txt')
 
-        with open(self.tfile) as inp:
+        with open(self.tfile, encoding='utf-8') as inp:
             self.rnames = inp.read()
 
         self.rnames = self.rnames.split('\n')
@@ -154,6 +151,11 @@ class ImportSeisan():
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -171,7 +173,7 @@ class ImportSeisan():
 
         os.chdir(os.path.dirname(self.ifile))
 
-        with open(self.ifile) as pntfile:
+        with open(self.ifile, encoding='utf-8') as pntfile:
             ltmp = pntfile.readlines()
 
         if len(ltmp[0]) < 80:
@@ -308,7 +310,7 @@ class ImportSeisan():
                 self.showprocesslog('Error: Problem with file')
                 self.showprocesslog('Process stopping, please see errors '
                                     'in '+self.ifile+'.log')
-            fout = open(self.ifile+'.log', 'w')
+            fout = open(self.ifile+'.log', 'w', encoding='utf-8')
             for i in file_errors:
                 fout.write(i[0]+'\n')
                 fout.write(i[1]+'\n')
@@ -406,6 +408,11 @@ def read_record_type_1(i):
     tmp.type_of_magnitude_3 = i[75]
     tmp.magnitude_reporting_agency_3 = i[76:79]
 
+    # make sure coordinates have the correct range
+    tmp.longitude = ((tmp.longitude - 180) % 360) - 180
+    tmp.latitude = ((tmp.latitude - 90) % 180) - 90
+
+
     return tmp
 
 
@@ -793,7 +800,7 @@ def read_record_type_p(i):
     return tmp
 
 
-class ImportGenericFPS():
+class ImportGenericFPS(BasicModule):
     """
     Import Generic Fault Plane Solution Data.
 
@@ -801,15 +808,17 @@ class ImportGenericFPS():
     """
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -835,7 +844,7 @@ class ImportGenericFPS():
 
         dlim = ','
 
-        with open(self.ifile) as pntfile:
+        with open(self.ifile, encoding='utf-8') as pntfile:
             ltmp = pntfile.readline()
 
         isheader = any(c.isalpha() for c in ltmp)
@@ -920,20 +929,14 @@ class ImportGenericFPS():
         return projdata
 
 
-class ExportSeisan():
+class ExportSeisan(ContextModule):
     """Export SEISAN Data."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
+
         self.lmod = None
         self.fobj = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def run(self):
         """
@@ -960,7 +963,7 @@ class ExportSeisan():
 
         os.chdir(os.path.dirname(filename))
 
-        self.fobj = open(filename, 'w')
+        self.fobj = open(filename, 'w', encoding='utf-8')
 
         for i in data:
             self.write_record_type_1(i)
@@ -1428,20 +1431,14 @@ class ExportSeisan():
         self.fobj.write(tmp)
 
 
-class ExportCSV():
+class ExportCSV(ContextModule):
     """Export SEISAN Data."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
+
         self.lmod = None
         self.fobj = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def run(self):
         """
@@ -1466,7 +1463,7 @@ class ExportCSV():
             return
         os.chdir(os.path.dirname(filename))
 
-        self.fobj = open(filename, 'w')
+        self.fobj = open(filename, 'w', encoding='utf-8')
 
         headi = ('last_action_done, date_time_of_last_action, operator, '
                  'status, id, new_id_created, id_locked, ')
@@ -1974,20 +1971,14 @@ class ExportCSV():
         return tmp
 
 
-class ExportSummary():
+class ExportSummary(ContextModule):
     """Export SEISAN Data."""
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
+        super().__init__(parent)
+
         self.lmod = None
         self.fobj = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def run(self):
         """
@@ -2082,7 +2073,6 @@ class ExportSummary():
             geom = gpd.points_from_xy(df.Longitude, df.Latitude)
             gdf = gpd.GeoDataFrame(df, geometry=geom)
             gdf = gdf.set_crs("EPSG:4326")
-            # breakpoint()
             gdf.to_file(filename)
 
 
@@ -2129,29 +2119,16 @@ def mercalli(mag):
     return merc
 
 
-class FilterSeisan(QtWidgets.QDialog):
+class FilterSeisan(BasicModule):
     """
     Filter Data.
 
     This filters data using thresholds.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
-
         self.datlimits = None
 
         self.dsb_from = QtWidgets.QDoubleSpinBox()
@@ -2179,7 +2156,6 @@ class FilterSeisan(QtWidgets.QDialog):
         gridlayout_main = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
         helpdocs = menu_default.HelpButton('pygmi.raster.dataprep.datagrid')
-        # label_dind = QtWidgets.QLabel('Distance Indicator:')
         label_rectype = QtWidgets.QLabel('Record Type:')
         label_recdesc = QtWidgets.QLabel('Description:')
         label_from = QtWidgets.QLabel('From')
@@ -2234,7 +2210,7 @@ class FilterSeisan(QtWidgets.QDialog):
         Parameters
         ----------
         state : TYPE
-            DESCRIPTION.
+            State of checkbox.
 
         Returns
         -------
@@ -2363,7 +2339,7 @@ class FilterSeisan(QtWidgets.QDialog):
                             datd[newkey].append(tmp[j])
 
         slist = []
-        for event in datd:
+        for event in datd.keys():
             datd[event] = [min(datd[event]), max(datd[event])]
             if isinstance(datd[event][0], str):
                 slist.append(event)
@@ -2377,9 +2353,14 @@ class FilterSeisan(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
-        tmp : bool
+        bool
             True if successful, False otherwise.
 
         """
@@ -2526,7 +2507,7 @@ class FilterSeisan(QtWidgets.QDialog):
                 if self.rinc.isChecked() and (testval < minval or
                                               testval > maxval):
                     continue
-                elif not self.rinc.isChecked() and (minval <= testval <= maxval):
+                if not self.rinc.isChecked() and (minval <= testval <= maxval):
                     continue
             else:
                 for j in i[rectype]:
@@ -2542,7 +2523,7 @@ class FilterSeisan(QtWidgets.QDialog):
                     if self.rinc.isChecked() and (testval < minval or
                                                   testval > maxval):
                         break
-                    elif not self.rinc.isChecked() and (minval <= testval <= maxval):
+                    if not self.rinc.isChecked() and (minval <= testval <= maxval):
                         break
                     badrec = False
                 if badrec is True:
