@@ -27,36 +27,20 @@
 import difflib
 import os
 from PyQt5 import QtWidgets, QtCore
-# import pygmi.menu_default as menu_default
+
+from pygmi.misc import BasicModule
 
 
-class CorrectDescriptions(QtWidgets.QDialog):
+class CorrectDescriptions(BasicModule):
     """
     Correct SEISAN descriptions.
 
     This compares the descriptions found in SEISAN type 3 lines to a custom
     list.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
 
         idir = os.path.dirname(os.path.realpath(__file__))
         tfile = os.path.join(idir, r'descriptions.txt')
@@ -76,7 +60,7 @@ class CorrectDescriptions(QtWidgets.QDialog):
         """
         gridlayout_main = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
-        # helpdocs = menu_default.HelpButton('pygmi.grav.iodefs.importpointdata')
+        # helpdocs = menu_default.HelpButton('pygmi.seis.utils.cordesc')
         pb_textfile = QtWidgets.QPushButton('Load Description List')
 
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
@@ -88,7 +72,7 @@ class CorrectDescriptions(QtWidgets.QDialog):
         gridlayout_main.addWidget(self.textfile, 0, 0, 1, 1)
         gridlayout_main.addWidget(pb_textfile, 0, 1, 1, 1)
 
-#        gridlayout_main.addWidget(helpdocs, 5, 0, 1, 1)
+        # gridlayout_main.addWidget(helpdocs, 5, 0, 1, 1)
         gridlayout_main.addWidget(buttonbox, 5, 1, 1, 3)
 
         buttonbox.accepted.connect(self.accept)
@@ -123,9 +107,14 @@ class CorrectDescriptions(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
-        tmp : bool
+        bool
             True if successful, False otherwise.
 
         """
@@ -170,8 +159,6 @@ class CorrectDescriptions(QtWidgets.QDialog):
         """
         projdata = {}
 
-#        projdata['ftype'] = '2D Mean'
-
         return projdata
 
     def acceptall(self):
@@ -186,7 +173,7 @@ class CorrectDescriptions(QtWidgets.QDialog):
 
         """
         filename = self.textfile.text()
-        with open(filename) as fno:
+        with open(filename, encoding='utf-8') as fno:
             tmp = fno.read()
 
         masterlist = tmp.split('\n')
@@ -226,17 +213,17 @@ def _testfn():
     None.
 
     """
-    import pygmi.seis.iodefs as iodefs
+    from pygmi.seis import iodefs
 
     idir = os.path.dirname(os.path.realpath(__file__))
     tfile = os.path.join(idir, r'descriptions.txt')
 
-    with open(tfile) as fno:
+    with open(tfile, encoding='utf-8') as fno:
         tmp = fno.read()
 
     masterlist = tmp.split('\n')
 
-    ifile = r'C:\WorkData\seismology\2000to2018.out'
+    ifile = r'd:\WorkData\seismology\2000to2018.out'
 
     IO = iodefs.ImportSeisan(None)
     IO.settings(ifile)
