@@ -1,5 +1,5 @@
 # -----------------------------------------------------------------------------
-# Name:        test_cluster.py (part of PyGMI)
+# Name:        test_gravity.py (part of PyGMI)
 #
 # Author:      Patrick Cole
 # E-Mail:      pcole@geoscience.org.za
@@ -45,13 +45,13 @@ def test_process_data():
     grvfile = os.path.join(idir, r'testdata\GravityCG5.txt')
     gpsfile = os.path.join(idir, r'testdata\GravityDGPS.csv')
 
-# Import Data
+    # Import Data
     IO = iodefs.ImportCG5(None)
     IO.get_cg5(grvfile)
     IO.get_gps(gpsfile)
     IO.settings(True)
 
-# Process Data
+    # Process Data
     PD = dataprep.ProcessData()
     PD.indata = IO.outdata
     PD.settings(True)
@@ -59,16 +59,21 @@ def test_process_data():
     datout = PD.outdata['Line']
 
     boug = datout['Gravity']['BOUGUER']
+    boug = np.array(boug)
 
-    datout2 = np.array([-716.54569535, -716.53438794, -716.32502705,
-                        -716.33644437, -716.19896636, -716.25688484,
-                        -716.09051822, -716.04696756, -716.02400454,
-                        -715.95874222, -715.98742539, -716.08405924,
-                        -715.97707272, -715.83871245, -716.07362109,
-                        -715.72504247, -715.72801816, -715.61688398,
-                        -715.77093313, -715.66479048, -715.49094074,
-                        -715.60267113, -715.59686646, -715.62371456,
-                        -715.54105161])
+    datout2 = np.array([-716.5271207830797, -716.5167383494435,
+                        -716.3040775379834, -716.3141448945184,
+                        -716.1779168502552, -716.2380602844478,
+                        -716.0745685905054, -716.0335678754984,
+                        -716.0125048061484, -715.9501174250662,
+                        -715.9810255366001, -716.0802093258817,
+                        -715.9757727522261, -715.8399624210209,
+                        -716.0774210031059, -715.7307423337603,
+                        -715.7356179855093, -715.62703373732,
+                        -715.7836328320539, -715.6793901363467,
+                        -715.5077653510672, -715.622045673794,
+                        -715.6187909466202, -715.6485139794081,
+                        -715.5684009646598])
 
     np.testing.assert_array_almost_equal(datout2, boug)
 
