@@ -29,47 +29,40 @@ import copy
 from PyQt5 import QtWidgets, QtCore, QtGui
 import numpy as np
 import matplotlib.path as mplPath
-from osgeo import osr, ogr
+# from osgeo import ogr
+from scipy.interpolate import griddata
+from scipy.ndimage import distance_transform_edt
+import geopandas as gpd
+from pyproj import CRS, Transformer
 
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
 from pygmi.raster.dataprep import GroupProj
 from pygmi.raster.datatypes import Data
+from pygmi.vector.minc import minc
+from pygmi.misc import BasicModule
 
 
-class PointCut():
+class PointCut(BasicModule):
     """
     Cut Data using shapefiles.
 
     This class cuts point datasets using a boundary defined by a polygon
     shapefile.
-
-    Attributes
-    ----------
-    ifile : str
-        input file name.
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
-        self.ifile = ''
-        self.pbar = parent.pbar
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
+        super().__init__(parent)
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -84,7 +77,7 @@ class PointCut():
             self.showprocesslog('No point data')
             return False
 
-        nodialog = False
+        # nodialog = False
         if not nodialog:
             ext = 'Shape file (*.shp)'
             self.ifile, _ = QtWidgets.QFileDialog.getOpenFileName(
@@ -103,7 +96,8 @@ class PointCut():
                                           QtWidgets.QMessageBox.Ok)
             return False
 
-        self.pbar.to_max()
+        if self.pbar is not None:
+            self.pbar.to_max()
         self.outdata['Line'] = {key: data}
 
         return True
@@ -145,41 +139,27 @@ class PointCut():
         return projdata
 
 
-class DataGrid(QtWidgets.QDialog):
+class DataGrid(BasicModule):
     """
     Grid Point Data.
 
     This class grids point data using a nearest neighbourhood technique.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.dxy = None
         self.dataid_text = None
 
         self.dsb_dxy = QtWidgets.QLineEdit('1.0')
         self.dsb_null = QtWidgets.QLineEdit('0.0')
+        self.bdist = QtWidgets.QLineEdit('4.0')
 
         self.dataid = QtWidgets.QComboBox()
+        self.grid_method = QtWidgets.QComboBox()
         self.label_rows = QtWidgets.QLabel('Rows: 0')
         self.label_cols = QtWidgets.QLabel('Columns: 0')
+        self.label_bdist = QtWidgets.QLabel('Blanking Distance:')
 
         self.setupui()
 
@@ -194,10 +174,11 @@ class DataGrid(QtWidgets.QDialog):
         """
         gridlayout_main = QtWidgets.QGridLayout(self)
         buttonbox = QtWidgets.QDialogButtonBox()
-        helpdocs = menu_default.HelpButton('pygmi.raster.dataprep.datagrid')
+        helpdocs = menu_default.HelpButton('pygmi.vector.dataprep.datagrid')
         label_band = QtWidgets.QLabel('Column to Grid:')
         label_dxy = QtWidgets.QLabel('Cell Size:')
         label_null = QtWidgets.QLabel('Null Value:')
+        label_method = QtWidgets.QLabel('Gridding Method:')
 
         val = QtGui.QDoubleValidator(0.0000001, 9999999999.0, 9)
         val.setNotation(QtGui.QDoubleValidator.ScientificNotation)
@@ -206,22 +187,29 @@ class DataGrid(QtWidgets.QDialog):
         self.dsb_dxy.setValidator(val)
         self.dsb_null.setValidator(val)
 
+        self.grid_method.addItems(['Nearest Neighbour', 'Linear', 'Cubic',
+                                   'Minimum Curvature'])
+
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
         buttonbox.setCenterButtons(True)
         buttonbox.setStandardButtons(buttonbox.Cancel | buttonbox.Ok)
 
         self.setWindowTitle('Dataset Gridding')
 
-        gridlayout_main.addWidget(label_dxy, 0, 0, 1, 1)
-        gridlayout_main.addWidget(self.dsb_dxy, 0, 1, 1, 1)
-        gridlayout_main.addWidget(self.label_rows, 1, 0, 1, 2)
-        gridlayout_main.addWidget(self.label_cols, 2, 0, 1, 2)
-        gridlayout_main.addWidget(label_band, 3, 0, 1, 1)
-        gridlayout_main.addWidget(self.dataid, 3, 1, 1, 1)
-        gridlayout_main.addWidget(label_null, 4, 0, 1, 1)
-        gridlayout_main.addWidget(self.dsb_null, 4, 1, 1, 1)
-        gridlayout_main.addWidget(helpdocs, 5, 0, 1, 1)
-        gridlayout_main.addWidget(buttonbox, 5, 1, 1, 3)
+        gridlayout_main.addWidget(label_method, 0, 0, 1, 1)
+        gridlayout_main.addWidget(self.grid_method, 0, 1, 1, 1)
+        gridlayout_main.addWidget(label_dxy, 1, 0, 1, 1)
+        gridlayout_main.addWidget(self.dsb_dxy, 1, 1, 1, 1)
+        gridlayout_main.addWidget(self.label_rows, 2, 0, 1, 2)
+        gridlayout_main.addWidget(self.label_cols, 3, 0, 1, 2)
+        gridlayout_main.addWidget(label_band, 4, 0, 1, 1)
+        gridlayout_main.addWidget(self.dataid, 4, 1, 1, 1)
+        gridlayout_main.addWidget(label_null, 5, 0, 1, 1)
+        gridlayout_main.addWidget(self.dsb_null, 5, 1, 1, 1)
+        gridlayout_main.addWidget(self.label_bdist, 6, 0, 1, 1)
+        gridlayout_main.addWidget(self.bdist, 6, 1, 1, 1)
+        gridlayout_main.addWidget(helpdocs, 7, 0, 1, 1)
+        gridlayout_main.addWidget(buttonbox, 7, 1, 1, 3)
 
         buttonbox.accepted.connect(self.accept)
         buttonbox.rejected.connect(self.reject)
@@ -229,7 +217,7 @@ class DataGrid(QtWidgets.QDialog):
 
     def dxy_change(self):
         """
-        When dxy is changed on the interface, this update rows and columns.
+        When dxy is changed on the interface, this updates rows and columns.
 
         Returns
         -------
@@ -255,10 +243,31 @@ class DataGrid(QtWidgets.QDialog):
         self.label_rows.setText('Rows: '+str(rows))
         self.label_cols.setText('Columns: '+str(cols))
 
+    def grid_method_change(self):
+        """
+        When grid method is changed, this updated hidden controls.
+
+        Returns
+        -------
+        None.
+
+        """
+        if self.grid_method.currentText() == 'Minimum Curvature':
+            self.label_bdist.show()
+            self.bdist.show()
+        else:
+            self.label_bdist.hide()
+            self.bdist.hide()
+
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -306,6 +315,14 @@ class DataGrid(QtWidgets.QDialog):
             if tmp != 1:
                 return False
 
+        try:
+            float(self.dsb_dxy.text())
+            float(self.dsb_null.text())
+            float(self.bdist.text())
+        except ValueError:
+            self.showprocesslog('Value Error')
+            return False
+
         self.acceptall()
 
         return True
@@ -329,9 +346,9 @@ class DataGrid(QtWidgets.QDialog):
         self.dataid_text = projdata['band']
         self.dsb_dxy.setText(projdata['dxy'])
         self.dsb_null.setText(projdata['nullvalue'])
+        self.bdist.setText(projdata['bdist'])
 
         self.dsb_dxy.textChanged.connect(self.dxy_change)
-#        self.dxy_change()
 
         return False
 
@@ -349,6 +366,7 @@ class DataGrid(QtWidgets.QDialog):
 
         projdata['dxy'] = self.dsb_dxy.text()
         projdata['nullvalue'] = self.dsb_null.text()
+        projdata['bdist'] = self.bdist.text()
         projdata['band'] = self.dataid_text
 
         return projdata
@@ -365,43 +383,66 @@ class DataGrid(QtWidgets.QDialog):
 
         """
         dxy = float(self.dsb_dxy.text())
+        method = self.grid_method.currentText()
         nullvalue = float(self.dsb_null.text())
+        bdist = float(self.bdist.text())
         key = list(self.indata['Line'].keys())[0]
         data = self.indata['Line'][key]
         data = data.dropna()
         newdat = []
 
+        if bdist < 1:
+            bdist = None
+            self.showprocesslog('Blanking distance too small.')
+
         filt = (data[self.dataid.currentText()] != nullvalue)
         x = data.pygmiX.values[filt]
         y = data.pygmiY.values[filt]
         z = data[self.dataid.currentText()].values[filt]
 
-        tmp = quickgrid(x, y, z, dxy, showprocesslog=self.showprocesslog)
-        mask = np.ma.getmaskarray(tmp)
-        gdat = tmp.data
+        if method == 'Minimum Curvature':
+            gdat = minc(x, y, z, dxy, showprocesslog=self.showprocesslog,
+                        bdist=bdist)
+            gdat = np.ma.filled(gdat, fill_value=nullvalue)
+        else:
+            extent = np.array([x.min(), x.max(), y.min(), y.max()])
 
-# Create dataset
+            rows = int((extent[3] - extent[2])/dxy+1)
+            cols = int((extent[1] - extent[0])/dxy+1)
+
+            xxx = np.linspace(extent[0], extent[1], cols)
+            yyy = np.linspace(extent[2], extent[3], rows)
+            xxx, yyy = np.meshgrid(xxx, yyy)
+
+            points = np.transpose([x.flatten(), y.flatten()])
+
+            if method == 'Nearest Neighbour':
+                gdat = griddata(points, z, (xxx, yyy), method='nearest')
+                gdat = blanking(gdat, x, y, bdist, extent, dxy, nullvalue)
+            elif method == 'Linear':
+                gdat = griddata(points, z, (xxx, yyy), method='linear',
+                                fill_value=nullvalue)
+                gdat = blanking(gdat, x, y, bdist, extent, dxy, nullvalue)
+            elif method == 'Cubic':
+                gdat = griddata(points, z, (xxx, yyy), method='cubic',
+                                fill_value=nullvalue)
+                gdat = blanking(gdat, x, y, bdist, extent, dxy, nullvalue)
+
+            gdat = gdat[::-1]
+        gdat = np.ma.masked_equal(gdat, nullvalue)
+
+        # Create dataset
         dat = Data()
-        dat.data = np.ma.masked_invalid(gdat[::-1])
-        dat.data.mask = mask[::-1]
-        dat.nullvalue = nullvalue
+        dat.data = gdat
+        dat.nodata = nullvalue
         dat.dataid = self.dataid.currentText()
-        dat.xdim = dxy
-        dat.ydim = dxy
-        # dat.extent = [x.min(), x.max(), y.min(), y.max()]
 
-        rows, cols = dat.data.shape
-        # left = x.min()
-        # top = y.max()
-        # right = left + dxy*cols
-        # bottom = top - dxy*rows
+        rows, _ = dat.data.shape
 
         left = x.min()
-        bottom = y.min()
-        top = bottom + dxy*rows
-        right = left + dxy*cols
+        top = y.min() + dxy*rows
 
-        dat.extent = (left, right, bottom, top)
+        dat.set_transform(dxy, left, dxy, top)
 
         newdat.append(dat)
 
@@ -409,32 +450,15 @@ class DataGrid(QtWidgets.QDialog):
         self.outdata['Line'] = self.indata['Line']
 
 
-class DataReproj(QtWidgets.QDialog):
+class DataReproj(BasicModule):
     """
     Reprojections.
 
-    This class reprojects datasets using the GDAL routines.
-
-    Attributes
-    ----------
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    outdata : dictionary
-        dictionary of output datasets
+    This class reprojects datasets using the rasterio routines.
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.indata = {}
-        self.outdata = {}
-        self.parent = parent
         self.orig_wkt = None
         self.targ_wkt = None
 
@@ -494,26 +518,15 @@ class DataReproj(QtWidgets.QDialog):
         key = list(self.indata['Line'].keys())[0]
         data = self.indata['Line'][key]
 
-# Input stuff
+        # Input stuff
         orig_wkt = self.in_proj.wkt
 
-        orig = osr.SpatialReference()
-        orig.ImportFromWkt(orig_wkt)
-        orig.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
-
-# Output stuff
+        # Output stuff
         targ_wkt = self.out_proj.wkt
 
-        targ = osr.SpatialReference()
-        targ.ImportFromWkt(targ_wkt)
-        targ.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+        x2, y2 = reprojxy(data.pygmiX, data.pygmiY, orig_wkt, targ_wkt)
 
-# Set transformation
-        ctrans = osr.CoordinateTransformation(orig, targ)
-
-        dd = np.transpose([data.pygmiX, data.pygmiY])
-        xy = ctrans.TransformPoints(dd)
-        xy = np.array(xy)
+        xy = np.transpose([x2, y2])
 
         if np.inf in xy:
             self.showprocesslog('Note: inf values in reprojected results. '
@@ -532,6 +545,11 @@ class DataReproj(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -547,26 +565,18 @@ class DataReproj(QtWidgets.QDialog):
             self.orig_wkt = self.indata['Vector'][firstkey].crs.to_wkt()
 
         if self.orig_wkt is None:
-            indx = self.in_proj.combobox.findText(r'WGS 84 / '
-                                                  r'Geodetic Geographic')
-            self.in_proj.combobox.setCurrentIndex(indx)
+            indx = self.in_proj.combodatum.findText(r'WGS 84')
+            self.in_proj.combodatum.setCurrentIndex(indx)
             self.orig_wkt = self.in_proj.wkt
         else:
             self.in_proj.set_current(self.orig_wkt)
         if self.targ_wkt is None:
-            indx = self.in_proj.combobox.findText(r'WGS 84 / UTM zone 35S')
-            self.out_proj.combobox.setCurrentIndex(indx)
+            indx = self.in_proj.combodatum.findText(r'WGS 84')
+            self.out_proj.combodatum.setCurrentIndex(indx)
             self.targ_wkt = self.out_proj.wkt
         else:
             self.out_proj.set_current(self.targ_wkt)
 
-#        iwkt = self.in_proj.epsg_proj['WGS 84 / Geodetic Geographic'].wkt
-#        indx = self.in_proj.combobox.findText('WGS 84 / Geodetic Geographic')
-#        self.in_proj.combobox.setCurrentIndex(indx)
-
-#        indx = self.in_proj.combobox.findText('WGS 84 / UTM zone 35S')
-#        self.out_proj.combobox.setCurrentIndex(indx)
-
         if not nodialog:
             tmp = self.exec_()
 
@@ -637,36 +647,44 @@ def cut_point(data, ifile):
 
     Returns
     -------
-    Data
+    data : Data
         PyGMI Dataset
     """
-    shapef = ogr.Open(ifile)
-    if shapef is None:
-        return None
-    lyr = shapef.GetLayer()
-    poly = lyr.GetNextFeature()
-    if lyr.GetGeomType() is not ogr.wkbPolygon or poly is None:
-        shapef = None
-        return None
+    # shapef = ogr.Open(ifile)
+    # if shapef is None:
+    #     return None
+    # lyr = shapef.GetLayer()
+    # poly = lyr.GetNextFeature()
+    # if lyr.GetGeomType() is not ogr.wkbPolygon or poly is None:
+    #     shapef = None
+    #     return None
 
-    points = []
-    geom = poly.GetGeometryRef()
+    # points = []
+    # geom = poly.GetGeometryRef()
 
-    ifin = 0
-    imax = 0
-    if geom.GetGeometryName() == 'MULTIPOLYGON':
-        for i in range(geom.GetGeometryCount()):
-            geom.GetGeometryRef(i)
-            itmp = geom.GetGeometryRef(i)
-            itmp = itmp.GetGeometryRef(0).GetPointCount()
-            if itmp > imax:
-                imax = itmp
-                ifin = i
-        geom = geom.GetGeometryRef(ifin)
+    # ifin = 0
+    # imax = 0
+    # if geom.GetGeometryName() == 'MULTIPOLYGON':
+    #     for i in range(geom.GetGeometryCount()):
+    #         geom.GetGeometryRef(i)
+    #         itmp = geom.GetGeometryRef(i)
+    #         itmp = itmp.GetGeometryRef(0).GetPointCount()
+    #         if itmp > imax:
+    #             imax = itmp
+    #             ifin = i
+    #     geom = geom.GetGeometryRef(ifin)
 
-    pts = geom.GetGeometryRef(0)
-    for pnt in range(pts.GetPointCount()):
-        points.append((pts.GetX(pnt), pts.GetY(pnt)))
+    # pts = geom.GetGeometryRef(0)
+    # for pnt in range(pts.GetPointCount()):
+    #     points.append((pts.GetX(pnt), pts.GetY(pnt)))
+
+    gdf = gpd.read_file(ifile)
+    gdf = gdf[gdf.geometry != None]
+
+    if 'Polygon' in gdf.geom_type.iloc[0]:
+        dat = gdf.geometry.iloc[0]
+
+    points = list(dat.exterior.coords)
 
     bbpath = mplPath.Path(points)
 
@@ -675,7 +693,7 @@ def cut_point(data, ifile):
 
     data = data[chk]
 
-    shapef = None
+    # shapef = None
     return data
 
 
@@ -695,9 +713,9 @@ def quickgrid(x, y, z, dxy, numits=4, showprocesslog=print):
         cell size for the grid, in both the x and y direction.
     numits : int
         number of iterations. By default its 4. If this is negative, a maximum
-        numits will be calculated and used.
-    showprogresslog : print, optional
-        Routine for displaying messages. Default is print
+        will be calculated and used.
+    showprocesslog : function, optional
+        Routine to show text messages. The default is print.
 
     Returns
     -------
@@ -762,33 +780,144 @@ def quickgrid(x, y, z, dxy, numits=4, showprocesslog=print):
     return newz
 
 
+def blanking(gdat, x, y, bdist, extent, dxy, nullvalue):
+    """
+    Blanks area further than a defined number of cells from input data.
+
+    Parameters
+    ----------
+    gdat : numpy array
+        grid data to blank.
+    x : numpy array
+        x coordinates.
+    y : numpy array
+        y coordinates.
+    bdist : int
+        Blanking distance in units for cell.
+    extent : list
+        extent of grid.
+    dxy : float
+        Cell size.
+    Nullvalue : float
+        Null or nodata value.
+
+    Returns
+    -------
+    mask : numpy array
+        Mask to be used for blanking.
+
+    """
+    mask = np.zeros_like(gdat)
+
+    points = np.transpose([x, y])
+
+    for xy in points:
+        col = int((xy[0]-extent[0])/dxy)
+        row = int((xy[1]-extent[2])/dxy)
+
+        mask[row, col] = 1
+
+    dist = distance_transform_edt(np.logical_not(mask))
+    mask = (dist > bdist)
+
+    gdat[mask] = nullvalue
+
+    return gdat
+
+
+def reprojxy(x, y, iwkt, owkt):
+    """
+    Reproject x and y coordinates.
+
+    Parameters
+    ----------
+    x : numpy array or float
+        x coordinates
+    y : numpy array or float
+        y coordinates
+    iwkt : str, int
+        Input wkt description or EPSG code (int)
+    owkt : str, int
+        Output wkt description or EPSG code (int)
+
+    Returns
+    -------
+    xout : numpy array
+        x coordinates.
+    yout : numpy array
+        y coordinates.
+
+    """
+    if isinstance(iwkt, int):
+        crs_from = CRS.from_epsg(iwkt)
+    else:
+        crs_from = CRS.from_wkt(iwkt)
+
+    if isinstance(owkt, int):
+        crs_to = CRS.from_epsg(owkt)
+    else:
+        crs_to = CRS.from_wkt(owkt)
+
+    transformer = Transformer.from_crs(crs_from, crs_to, always_xy=True)
+    xout, yout = transformer.transform(x, y)
+
+    return xout, yout
+
+
 def _testfn():
     """Test routine."""
     import sys
     import matplotlib.pyplot as plt
-    from pygmi.vector.iodefs import ImportLineData, ImportShapeData
+    from pygmi.vector.iodefs import ImportLineData
 
-    APP = QtWidgets.QApplication(sys.argv)  # Necessary to test Qt Classes
+    app = QtWidgets.QApplication(sys.argv)
 
-    ifile = r'D:\Workdata\vector\linecut\test2.csv'
-    sfile = r'D:\Workdata\vector\linecut\test2_cut_outline.shp'
+    ifile = r"D:\Workdata\PyGMI Test Data\Vector\Line Data\SPECARCHIVE.XYZ"
+
+    IO = ImportLineData()
+    IO.ifile = ifile
+    IO.filt = 'Geosoft XYZ (*.xyz)'
+    IO.settings(True)
+
+    # DG = DataGrid()
+    # DG.indata = IO.outdata
+    # DG.settings()
+
+    # dat = DG.outdata['Raster'][0].data
+
+    # plt.imshow(dat)
+    # plt.show()
+
+    DR = DataReproj()
+    DR.indata = IO.outdata
+    DR.settings()
+
+    breakpoint()
+
+
+def _testfn_pointcut():
+    """Test routine."""
+    import sys
+    from pygmi.vector.iodefs import ImportLineData
+
+    app = QtWidgets.QApplication(sys.argv)
+
+    ifile = r"D:\Workdata\PyGMI Test Data\Vector\linecut\test2.csv"
+    sfile = r"D:\Workdata\PyGMI Test Data\Vector\linecut\test2_cut_outline.shp"
 
     IO = ImportLineData()
     IO.ifile = ifile
     IO.filt = 'Comma Delimited (*.csv)'
     IO.settings(True)
 
-    line = list(IO.outdata['Line'].values())
-    plt.plot(line[0].x, line[0].y)
-    plt.show()
-
-
-    DR = DataReproj()
+    DR = PointCut()
     DR.indata = IO.outdata
+    DR.ifile = sfile
     DR.settings(True)
 
     breakpoint()
 
 
+
 if __name__ == "__main__":
-    _testfn()
+    _testfn_pointcut()
