@@ -27,24 +27,22 @@
 import math
 import numpy as np
 from PyQt5 import QtWidgets, QtCore
-from scipy.stats import median_absolute_deviation
+from scipy.stats import median_abs_deviation
 import matplotlib.collections as mc
-from matplotlib import cm
+from matplotlib import colormaps
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5 import NavigationToolbar2QT
 from pandas.api.types import is_numeric_dtype
 
-from pygmi.misc import frm
+from pygmi.misc import frm, ContextModule
 
 
-class GraphWindow(QtWidgets.QDialog):
+class GraphWindow(ContextModule):
     """Graph Window - Main QT Dialog class for graphs."""
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.parent = parent
-
         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
         self.setWindowTitle('Graph Window')
 
@@ -117,7 +115,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         event : TYPE
-            DESCRIPTION.
+            Button release event.
 
         Returns
         -------
@@ -137,7 +135,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         event : TYPE
-            DESCRIPTION.
+            Motion notify event.
 
         Returns
         -------
@@ -166,12 +164,12 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         event : TYPE
-            DESCRIPTION.
+            Picker event.
 
         Returns
         -------
         bool
-            Return TRUE if pick succeeded.
+            Return TRUE if pick succeeded, False otherwise.
 
         """
         if event.mouseevent.inaxes is None:
@@ -193,7 +191,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         event : TYPE
-            DESCRIPTION.
+            Unused.
 
         Returns
         -------
@@ -231,7 +229,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.figure.clear()
 
         self.axes = self.figure.add_subplot(111, label='Profile')
-        # self.axes.ticklabel_format(useOffset=False, style='plain')
 
         self.axes.set_title('Profile')
         self.axes.set_xlabel('Distance')
@@ -240,7 +237,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.axes.xaxis.set_major_formatter(frm)
         self.axes.yaxis.set_major_formatter(frm)
 
-        # self.figure.tight_layout()
         self.figure.canvas.draw()
         self.background = self.figure.canvas.copy_from_bbox(self.axes.bbox)
 
@@ -256,10 +252,12 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         Parameters
         ----------
-        data : Pandas dataframe object
-            Point data
-        ival : dictionary key
-            dictionary key Point Data
+        xdata : numpy array
+            1D x-data array.
+        ydata : numpy array
+            1D y-data array.
+        zdata : numpy array
+            1D z-data array.
 
         Returns
         -------
@@ -271,7 +269,6 @@ class MyMplCanvas(FigureCanvasQTAgg):
         ax1 = self.figure.add_subplot(111, label='Map')
         ax1.ticklabel_format(style='plain')
 
-#        ax1.set_title(data1.dataid)
         self.axes = ax1
         self.axes.tick_params(axis='x', rotation=90)
         self.axes.tick_params(axis='y', rotation=0)
@@ -280,13 +277,11 @@ class MyMplCanvas(FigureCanvasQTAgg):
         self.background = self.figure.canvas.copy_from_bbox(ax1.bbox)
 
         if is_numeric_dtype(zdata):
-            scat = ax1.scatter(xdata, ydata, c=zdata, cmap=cm.get_cmap('jet'))
+            scat = ax1.scatter(xdata, ydata, c=zdata, cmap=colormaps['jet'])
         else:
             return
-#            scat = ax1.scatter(xdata, ydata)
 
         self.figure.colorbar(scat, ax=ax1, format=frm)
-#        data.plot(ival, cmap=jet, ax=ax1)
 
         self.axes.xaxis.set_major_formatter(frm)
         self.axes.yaxis.set_major_formatter(frm)
@@ -328,7 +323,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
         zdata = data[ival]
         med = np.median(zdata)
-        std = 2.5 * median_absolute_deviation(zdata, axis=None)
+        std = 2.5 * median_abs_deviation(zdata, axis=None, scale=1/1.4826)
 
         if std == 0:
             std = 1
@@ -378,7 +373,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
 
             if uselabels:
                 textang = np.rad2deg(ang[0])
-                ax1.text(x[0], y[0], line, rotation=textang)
+                ax1.text(x[0], y[0], line[0], rotation=textang)
             ax1.plot(x, y, 'c')
             ax1.plot(qx, qy, 'k')
 
@@ -395,7 +390,9 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         data : dictionary
-            Geopandas data in a dictionary.
+            GeoPandas data in a dictionary.
+        col : str
+            Label for column to extract.
 
         Returns
         -------
@@ -432,6 +429,17 @@ class MyMplCanvas(FigureCanvasQTAgg):
             self.axes.autoscale()
             self.axes.axis('equal')
 
+        elif 'MultiPolygon' in data:
+            tmp = []
+            for j in data['MultiPolygon'].geometry:
+                for i in list(j.geoms):
+                    tmp.append(np.array(i.exterior.coords[:])[:, :2].tolist())
+
+            lcol = mc.LineCollection(tmp)
+            self.axes.add_collection(lcol)
+            self.axes.autoscale()
+            self.axes.axis('equal')
+
         elif 'Point' in data:
             if col != '':
                 self.axes.scatter(data['Point'].geometry.x,
@@ -454,11 +462,13 @@ class MyMplCanvas(FigureCanvasQTAgg):
         Parameters
         ----------
         data : dictionary
-            Geopandas data in a dictionary. It should be 'LineString'
+            GeoPandas data in a dictionary. It should be 'LineString'
         rtype : int
             Rose diagram type. Can be either 0 or 1.
         nbins : int, optional
-            Number of bins used in rose diagram.
+            Number of bins used in rose diagram. The default is 8.
+        equal : bool, optional
+            Option for an equal area rose diagram. The default is False.
 
         Returns
         -------
@@ -511,7 +521,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
         fcnt = np.array(fcnt)
         flen = np.array(flen)
         bwidth = np.pi/nbins
-        Set1 = cm.get_cmap('Set1')
+        Set1 = colormaps['Set1']
         bcols = Set1(np.arange(nbins+1)/nbins)
         np.random.shuffle(bcols)
 
@@ -524,7 +534,7 @@ class MyMplCanvas(FigureCanvasQTAgg):
             if equal is True:
                 radii = .01*radii.max()*np.sqrt(100*radii/radii.max())
 
-            xtheta = theta[:-1]  # +(theta[1]-theta[0])/2
+            xtheta = theta[:-1]
             bcols2 = bcols[(xtheta/bwidth).astype(int)]
             ax1.bar(xtheta, radii, width=bwidth, color=bcols2)
             ax1.bar(xtheta+np.pi, radii, width=bwidth, color=bcols2)
@@ -560,18 +570,12 @@ class PlotPoints(GraphWindow):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
         self.spinbox.hide()
         self.label3.hide()
         self.combobox2.hide()
         self.label2.hide()
         self.xdata = None
         self.ydata = None
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
 
     def change_band(self):
         """
@@ -591,7 +595,7 @@ class PlotPoints(GraphWindow):
 
     def run(self):
         """
-        Combo box to choose band.
+        Entry point to run class.
 
         Returns
         -------
@@ -626,8 +630,6 @@ class PlotLines(GraphWindow):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
         self.spinbox.hide()
         self.label3.hide()
         self.xcol = ''
@@ -673,7 +675,7 @@ class PlotLines(GraphWindow):
 
     def run(self):
         """
-        Run.
+        Entry point to run class.
 
         Returns
         -------
@@ -714,8 +716,6 @@ class PlotLineMap(GraphWindow):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
         self.combobox2.hide()
         self.label2.hide()
         self.checkbox.show()
@@ -739,7 +739,7 @@ class PlotLineMap(GraphWindow):
 
     def run(self):
         """
-        Run.
+        Entry point to run class.
 
         Returns
         -------
@@ -782,8 +782,6 @@ class PlotRose(GraphWindow):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
         self.combobox2.hide()
         self.label2.hide()
         self.spinbox.setValue(8)
@@ -819,7 +817,7 @@ class PlotRose(GraphWindow):
 
     def run(self):
         """
-        Run.
+        Entry point to run class.
 
         Returns
         -------
@@ -839,8 +837,6 @@ class PlotVector(GraphWindow):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.indata = {}
-        self.parent = parent
         self.label1.hide()
         self.combobox2.hide()
         self.label2.hide()
@@ -856,18 +852,14 @@ class PlotVector(GraphWindow):
         None.
 
         """
-#        key = list(self.indata['Vector'].keys())[0]
         data = self.indata['Vector']
-#        data = data.dropna()
         i = self.combobox1.currentText()
 
-#        self.mmc.update_map(data.pygmiX, data.pygmiY, data[i])
-
         self.mmc.update_vector(data, i)
 
     def run(self):
         """
-        Run.
+        Entry point to run class.
 
         Returns
         -------
@@ -910,13 +902,13 @@ def histogram(x, y=None, xmin=None, xmax=None, bins=10):
     x : numpy array
         Input data
     y : numpy array
-        Input data weights. A value of 1 is default behaviour
+        Input data weights. The default is None.
     xmin : float
-        Lower value for the bins
+        Lower value for the bins. The default is None.
     xmax : float
-        Upper value for the bins
+        Upper value for the bins. The default is None.
     bins : int
-        number of bins
+        number of bins. The default is 10.
 
     Returns
     -------
