@@ -26,44 +26,26 @@
 
 import os
 import re
+from io import StringIO
+
 from PyQt5 import QtWidgets, QtCore
 import numpy as np
 import pandas as pd
 import geopandas as gpd
-import pygmi.menu_default as menu_default
+
+from pygmi import menu_default
+from pygmi.misc import ProgressBarText, BasicModule, ContextModule
 
 
-class ImportLineData(QtWidgets.QDialog):
+class ImportLineData(BasicModule):
     """
     Import Line Data.
 
     This class imports ASCII point data.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
     """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
-
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.ifile = ''
         self.filt = ''
 
         self.xchan = QtWidgets.QComboBox()
@@ -114,6 +96,11 @@ class ImportLineData(QtWidgets.QDialog):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
@@ -121,11 +108,11 @@ class ImportLineData(QtWidgets.QDialog):
 
         """
         if not nodialog:
-            ext = ('Geosoft XYZ (*.xyz);;'
+            ext = ('Comma Delimited (*.csv);;'
+                   'Geosoft XYZ (*.xyz);;'
                    'ASCII XYZ (*.xyz);;'
-                   'Comma Delimited (*.csv);;'
-                   'Tab Delimited (*.txt);;'
-                   'All Files (*.*)')
+                   'Space Delimited (*.txt);;'
+                   'Tab Delimited (*.txt)')
 
             self.ifile, self.filt = QtWidgets.QFileDialog.getOpenFileName(
                 self.parent, 'Open File', '.', ext)
@@ -140,6 +127,8 @@ class ImportLineData(QtWidgets.QDialog):
             gdf = self.get_delimited(',')
         elif self.filt == 'Tab Delimited (*.txt)':
             gdf = self.get_delimited('\t')
+        elif self.filt == 'Space Delimited (*.txt)':
+            gdf = self.get_delimited(' ')
         else:
             return False
 
@@ -189,6 +178,7 @@ class ImportLineData(QtWidgets.QDialog):
 
         gdf['pygmiX'] = gdf[xcol]
         gdf['pygmiY'] = gdf[ycol]
+
         gdf['line'] = gdf['line'].astype(str)
 
         if 'Line' not in self.outdata:
@@ -250,39 +240,57 @@ class ImportLineData(QtWidgets.QDialog):
 
         Returns
         -------
-        dat : DataFrame
+        df2 : DataFrame
             Pandas dataframe.
 
         """
-        with open(self.ifile) as fno:
-            head = fno.readline()
+        with open(self.ifile, encoding='utf-8') as fno:
             tmp = fno.read()
-        if r'/' not in head:
+
+        chktxt = tmp[:tmp.index('\n')].lower()
+
+        if r'/' not in chktxt and 'line' not in chktxt and 'tie' not in chktxt:
             self.showprocesslog('Not Geosoft XYZ format')
             return None
-        head = head.split()
-        head.pop(0)
+
+        head = None
+        while r'/' in tmp[:tmp.index('\n')]:
+            head = tmp[:tmp.index('\n')]
+            tmp = tmp[tmp.index('\n')+1:]
+            head = head.split()
+            head.pop(0)
+
+        while r'/' in tmp:
+            t1 = tmp[:tmp.index(r'/')]
+            t2 = tmp[tmp.index(r'/')+1:]
+            t3 = t2[t2.index('\n')+1:]
+            tmp = t1+t3
+
         tmp = tmp.lower()
+        tmp = tmp.lstrip()
         tmp = re.split('(line|tie)', tmp)
         if tmp[0] == '':
             tmp.pop(0)
 
-        dtype = {}
-        dtype['names'] = head
-        dtype['formats'] = ['f4']*len(head)
-
         df2 = None
-        for i in range(0, len(tmp), 2):
+        dflist = []
+        for i in self.piter(range(0, len(tmp), 2)):
             tmp2 = tmp[i+1]
-            tmp2 = tmp2.split('\n')
-            line = tmp[i]+tmp2.pop(0)
-            tmp2 = np.genfromtxt(tmp2, names=head)
-            df1 = pd.DataFrame(tmp2)
+
+            line = tmp[i]+' '+tmp2[:tmp2.index('\n')].strip()
+            tmp2 = tmp2[tmp2.index('\n')+1:]
+            if head is None:
+                head = [f'Column {i+1}' for i in
+                        range(len(tmp2[:tmp2.index('\n')].split()))]
+
+            tmp2 = tmp2.replace('*', 'NaN')
+            df1 = pd.read_csv(StringIO(tmp2), sep='\s+', names=head)
+
             df1['line'] = line
-            if df2 is None:
-                df2 = df1
-            else:
-                df2 = df2.append(df1, ignore_index=True)
+            dflist.append(df1)
+
+        # Concat in all df in one go is much faster
+        df2 = pd.concat(dflist, ignore_index=True)
 
         return df2
 
@@ -290,58 +298,32 @@ class ImportLineData(QtWidgets.QDialog):
         """
         Get a delimited line file.
 
+        Parameters
+        ----------
+        delimiter : str, optional
+            Delimiter type. The default is ','.
+
         Returns
         -------
-        gdf : DataFrame
+        gdf : Dataframe
             Pandas dataframe.
 
         """
-        with open(self.ifile) as fno:
-            head = fno.readline()
-            tmp = fno.read()
+        gdf = pd.read_csv(self.ifile, delimiter=delimiter, index_col=False)
 
-        head = head.split(delimiter)
-        head = [i.lower() for i in head]
-        tmp = tmp.lower()
+        gdf.columns = gdf.columns.str.lower()
 
-        dtype = {}
-        dtype['names'] = head
-        dtype['formats'] = ['f4']*len(head)
-
-        tmp = tmp.split('\n')
-        tmp2 = np.genfromtxt(tmp, names=head, delimiter=delimiter, dtype=None,
-                             encoding=None)
-        gdf = pd.DataFrame(tmp2)
-
-        if 'line' not in head:
+        if 'line' not in gdf.columns:
             gdf['line'] = 'None'
 
         return gdf
 
 
-class ExportLine():
-    """
-    Export Line Data.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class ExportLine(ContextModule):
+    """Export Line Data."""
 
     def __init__(self, parent=None):
-        self.parent = parent
-        self.indata = {}
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
+        super().__init__(parent)
 
     def run(self):
         """
@@ -378,29 +360,11 @@ class ExportLine():
         return True
 
 
-class ExportShapeData():
-    """
-    Export Line Data.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    indata : dictionary
-        dictionary of input datasets
-    """
+class ExportShapeData(ContextModule):
+    """Export Line Data."""
 
     def __init__(self, parent=None):
-        self.parent = parent
-        self.indata = {}
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
+        super().__init__(parent)
 
     def run(self):
         """
@@ -432,47 +396,27 @@ class ExportShapeData():
             data.to_file(filename, driver='GeoJSON')
         else:
             data.to_file(filename)
-#        dfall = data.drop(['pygmiX', 'pygmiY'], axis=1)
-
-#        dfall.to_csv(filename, index=False)
 
         self.showprocesslog('Export completed')
 
         return True
 
 
-class ImportShapeData():
-    """
-    Import Shapefile Data.
-
-    Attributes
-    ----------
-    name : str
-        item name
-    pbar : progressbar
-        reference to a progress bar.
-    parent : parent
-        reference to the parent routine
-    outdata : dictionary
-        dictionary of output datasets
-    ifile : str
-        input file name. Used in main.py
-    """
+class ImportShapeData(BasicModule):
+    """Import Shapefile Data."""
 
     def __init__(self, parent=None):
-        self.parent = parent
-        self.indata = {}
-        self.outdata = {}
-        self.ifile = ''
-        if parent is None:
-            self.showprocesslog = print
-        else:
-            self.showprocesslog = parent.showprocesslog
+        super().__init__(parent)
 
     def settings(self, nodialog=False):
         """
         Entry point into item.
 
+        Parameters
+        ----------
+        nodialog : bool, optional
+            Run settings without a dialog. The default is False.
+
         Returns
         -------
         bool
