@@ -29,7 +29,19 @@ This routine is used as a convenience function, typically if you do NOT
 formally install PyGMI as a library and prefer to run it from within the
 default extracted directory structure.
 """
+import sys
+
 from pygmi.main import main
+# import warnings
+
+# warnings.simplefilter("default")
+
+# useful regex for find: \.(?!py) in spyder.ini
+
 
 if __name__ == "__main__":
-    main()
+    if len(sys.argv) == 2 and 'true' in sys.argv[1].lower():
+        nocgs = True
+    else:
+        nocgs = False
+    main(nocgs=nocgs)
